@@ -62,14 +62,18 @@ int
 main (int argc, char **argv)
 {
 	AsbContext *ctx = NULL;
+	AsbContextProcessFlags flags = AS_CONTEXT_PARSE_FLAG_NONE;
 	GOptionContext *option_context;
 	const gchar *filename;
 	gboolean add_cache_id = FALSE;
+	gboolean embedded_icons = FALSE;
+	gboolean hidpi_enabled = FALSE;
 	gboolean no_net = FALSE;
 	gboolean ret;
 	gboolean verbose = FALSE;
 	gdouble api_version = 0.0f;
 	gint max_threads = 4;
+	gint min_icon_size = 32;
 	guint i;
 	_cleanup_dir_close_ GDir *dir = NULL;
 	_cleanup_error_free_ GError *error = NULL;
@@ -80,12 +84,14 @@ main (int argc, char **argv)
 	_cleanup_free_ gchar *extra_screenshots = NULL;
 	_cleanup_free_ gchar *log_dir = NULL;
 	_cleanup_free_ gchar *old_metadata = NULL;
+	_cleanup_free_ gchar *origin = NULL;
 	_cleanup_free_ gchar *output_dir = NULL;
-	_cleanup_free_ gchar *packages_dir = NULL;
 	_cleanup_free_ gchar *screenshot_dir = NULL;
 	_cleanup_free_ gchar *screenshot_uri = NULL;
 	_cleanup_free_ gchar *temp_dir = NULL;
+	_cleanup_free_ gchar **veto_ignore = NULL;
 	_cleanup_ptrarray_unref_ GPtrArray *packages = NULL;
+	_cleanup_strv_free_ gchar **packages_dirs = NULL;
 	_cleanup_timer_destroy_ GTimer *timer = NULL;
 	const GOptionEntry options[] = {
 		{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
@@ -97,13 +103,19 @@ main (int argc, char **argv)
 		{ "add-cache-id", '\0', 0, G_OPTION_ARG_NONE, &add_cache_id,
 			/* TRANSLATORS: command line option */
 			_("Add a cache ID to each component"), NULL },
+		{ "enable-hidpi", '\0', 0, G_OPTION_ARG_NONE, &hidpi_enabled,
+			/* TRANSLATORS: command line option */
+			_("Add HiDPI icons to the tarball"), NULL },
+		{ "enable-embed", '\0', 0, G_OPTION_ARG_NONE, &embedded_icons,
+			/* TRANSLATORS: command line option */
+			_("Add encoded icons to the XML"), NULL },
 		{ "log-dir", '\0', 0, G_OPTION_ARG_FILENAME, &log_dir,
 			/* TRANSLATORS: command line option */
 			_("Set the logging directory"), "DIR" },
 		{ "screenshot-dir", '\0', 0, G_OPTION_ARG_FILENAME, &screenshot_dir,
 			/* TRANSLATORS: command line option */
 			_("Set the screenshots directory"), "DIR" },
-		{ "packages-dir", '\0', 0, G_OPTION_ARG_FILENAME, &packages_dir,
+		{ "packages-dir", '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &packages_dirs,
 			/* TRANSLATORS: command line option */
 			_("Set the packages directory"), "DIR" },
 		{ "temp-dir", '\0', 0, G_OPTION_ARG_FILENAME, &temp_dir,
@@ -121,15 +133,21 @@ main (int argc, char **argv)
 		{ "output-dir", '\0', 0, G_OPTION_ARG_FILENAME, &output_dir,
 			/* TRANSLATORS: command line option */
 			_("Set the output directory"), "DIR" },
-		{ "cache-dir", '\0', 0, G_OPTION_ARG_FILENAME, &output_dir,
+		{ "cache-dir", '\0', 0, G_OPTION_ARG_FILENAME, &cache_dir,
 			/* TRANSLATORS: command line option */
 			_("Set the cache directory"), "DIR" },
 		{ "basename", '\0', 0, G_OPTION_ARG_STRING, &basename,
+			/* TRANSLATORS: command line option */
+			_("Set the basenames of the output files"), "NAME" },
+		{ "origin", '\0', 0, G_OPTION_ARG_STRING, &origin,
 			/* TRANSLATORS: command line option */
 			_("Set the origin name"), "NAME" },
 		{ "max-threads", '\0', 0, G_OPTION_ARG_INT, &max_threads,
 			/* TRANSLATORS: command line option */
 			_("Set the number of threads"), "THREAD_COUNT" },
+		{ "min-icon-size", '\0', 0, G_OPTION_ARG_INT, &min_icon_size,
+			/* TRANSLATORS: command line option */
+			_("Set the minimum icon size in pixels"), "ICON_SIZE" },
 		{ "api-version", '\0', 0, G_OPTION_ARG_DOUBLE, &api_version,
 			/* TRANSLATORS: command line option */
 			_("Set the AppStream version"), "API_VERSION" },
@@ -139,6 +157,9 @@ main (int argc, char **argv)
 		{ "old-metadata", '\0', 0, G_OPTION_ARG_FILENAME, &old_metadata,
 			/* TRANSLATORS: command line option */
 			_("Set the old metadata location"), "DIR" },
+		{ "veto-ignore", '\0', 0, G_OPTION_ARG_STRING_ARRAY, &veto_ignore,
+			/* TRANSLATORS: command line option */
+			_("Ignore certain types of veto"), "NAME" },
 		{ NULL}
 	};
 
@@ -169,8 +190,8 @@ main (int argc, char **argv)
 	/* set defaults */
 	if (api_version < 0.01)
 		api_version = 0.41;
-	if (packages_dir == NULL)
-		packages_dir = g_strdup ("./packages");
+	if (packages_dirs == NULL)
+		packages_dirs = g_strsplit ("./packages", "@", 1);
 	if (temp_dir == NULL)
 		temp_dir = g_strdup ("./tmp");
 	if (log_dir == NULL)
@@ -182,9 +203,11 @@ main (int argc, char **argv)
 	if (cache_dir == NULL)
 		cache_dir = g_strdup ("./cache");
 	if (basename == NULL)
-		basename = g_strdup ("fedora-21");
+		basename = g_strdup ("appstream");
+	if (origin == NULL)
+		origin = g_strdup ("example");
 	if (screenshot_uri == NULL)
-		screenshot_uri = g_strdup ("http://alt.fedoraproject.org/pub/alt/screenshots/f21/");
+		screenshot_uri = g_strdup ("http://www.example.com/screenshots/");
 	if (extra_appstream == NULL)
 		extra_appstream = g_strdup ("./appstream-extra");
 	if (extra_appdata == NULL)
@@ -197,6 +220,8 @@ main (int argc, char **argv)
 	asb_context_set_no_net (ctx, no_net);
 	asb_context_set_api_version (ctx, api_version);
 	asb_context_set_add_cache_id (ctx, add_cache_id);
+	asb_context_set_hidpi_enabled (ctx, hidpi_enabled);
+	asb_context_set_embedded_icons (ctx, embedded_icons);
 	asb_context_set_old_metadata (ctx, old_metadata);
 	asb_context_set_extra_appstream (ctx, extra_appstream);
 	asb_context_set_extra_appdata (ctx, extra_appdata);
@@ -208,7 +233,9 @@ main (int argc, char **argv)
 	asb_context_set_output_dir (ctx, output_dir);
 	asb_context_set_cache_dir (ctx, cache_dir);
 	asb_context_set_basename (ctx, basename);
+	asb_context_set_origin (ctx, origin);
 	asb_context_set_max_threads (ctx, max_threads);
+	asb_context_set_min_icon_size (ctx, min_icon_size);
 	ret = asb_context_setup (ctx, &error);
 	if (!ret) {
 		/* TRANSLATORS: error message */
@@ -219,10 +246,12 @@ main (int argc, char **argv)
 	/* scan each package */
 	packages = g_ptr_array_new_with_free_func (g_free);
 	if (argc == 1) {
-		if (!as_builder_search_path (packages, packages_dir, &error)) {
-			/* TRANSLATORS: error message */
-			g_warning ("%s: %s", _("Failed to open packages"), error->message);
-			goto out;
+		for (i = 0; packages_dirs[i] != NULL; i++) {
+			if (!as_builder_search_path (packages, packages_dirs[i], &error)) {
+				/* TRANSLATORS: error message */
+				g_warning ("%s: %s", _("Failed to open packages"), error->message);
+				goto out;
+			}
 		}
 	} else {
 		for (i = 1; i < (guint) argc; i++)
@@ -239,7 +268,7 @@ main (int argc, char **argv)
 		/* add to list */
 		if (!asb_context_add_filename (ctx, filename, &error_local)) {
 			/* TRANSLATORS: error message */
-			g_print ("%s %s: %s\n", _("Failed to add package"),
+			g_debug ("%s %s: %s", _("Failed to add package"),
 				 filename, error_local->message);
 			continue;
 		}
@@ -251,8 +280,25 @@ main (int argc, char **argv)
 		}
 	}
 
+	/* parse the context flags */
+	if (veto_ignore != NULL) {
+		for (i = 0; veto_ignore[i] != NULL; i++) {
+			if (g_strcmp0 (veto_ignore[i], "missing-info") == 0) {
+				flags |= AS_CONTEXT_PARSE_FLAG_IGNORE_MISSING_INFO;
+				continue;
+			}
+			if (g_strcmp0 (veto_ignore[i], "missing-parents") == 0) {
+				flags |= AS_CONTEXT_PARSE_FLAG_IGNORE_MISSING_PARENTS;
+				continue;
+			}
+			g_warning ("Unknown flag name: %s, "
+				   "expected 'missing-info' or 'missing-parents'",
+				   veto_ignore[i]);
+		}
+	}
+
 	/* process all packages in the pool */
-	ret = asb_context_process (ctx, &error);
+	ret = asb_context_process (ctx, flags, &error);
 	if (!ret) {
 		/* TRANSLATORS: error message */
 		g_warning ("%s: %s", _("Failed to generate metadata"), error->message);
