@@ -262,8 +262,8 @@ as_util_convert_appdata (GFile *file_input,
 	n2 = as_node_find (n, "metadata_license");
 	if (n2 == NULL) {
 		action_required = TRUE;
-		n2 = as_node_insert (n, "metadata_license", "FIXME: Insert SPDX ID Here",
-				     AS_NODE_INSERT_FLAG_NONE, NULL);
+		n2 = as_node_insert (n, "metadata_license", "<!-- Insert SPDX ID Here -->",
+				     AS_NODE_INSERT_FLAG_PRE_ESCAPED, NULL);
 	} else {
 		tmp = as_node_get_data (n2);
 
@@ -292,8 +292,9 @@ as_util_convert_appdata (GFile *file_input,
 	n2 = as_node_find (n, "project_license");
 	if (n2 == NULL) {
 		action_required = TRUE;
-		n2 = as_node_insert (n, "project_license", "FIXME: Insert SPDX ID Here",
-				     AS_NODE_INSERT_FLAG_NONE, NULL);
+		n2 = as_node_insert (n, "project_license", "<!-- Insert SPDX ID Here -->",
+				     AS_NODE_INSERT_FLAG_PRE_ESCAPED, NULL);
+		as_node_set_comment (n2, "FIXME: Use https://spdx.org/licenses/", -1);
 	} else {
 		/* ensure in SPDX format */
 		if (!as_utils_is_spdx_license_id (as_node_get_data (n2))) {
@@ -315,27 +316,31 @@ as_util_convert_appdata (GFile *file_input,
 		n3 = as_node_find (n, "project_group");
 		if (n3 != NULL) {
 			if (g_strcmp0 (as_node_get_data (n3), "GNOME") == 0) {
-				as_node_insert (n, "developer_name", "The GNOME Project",
-						flags_translate, NULL);
+				n3 = as_node_insert (n, "developer_name", "The GNOME Project",
+						     flags_translate, NULL);
+				as_node_set_comment (n3, "FIXME: this is a translatable version of project_group", -1);
 			} else if (g_strcmp0 (as_node_get_data (n3), "KDE") == 0) {
-				as_node_insert (n, "developer_name", "KDE e.V.",
-						AS_NODE_INSERT_FLAG_NONE, NULL);
+				n3 = as_node_insert (n, "developer_name", "The KDE Community",
+						     AS_NODE_INSERT_FLAG_NONE, NULL);
+				as_node_set_comment (n3, "FIXME: this is a translatable version of project_group", -1);
 			} else if (g_strcmp0 (as_node_get_data (n3), "XFCE") == 0) {
-				as_node_insert (n, "developer_name", "Xfce Development Team",
-						flags_translate, NULL);
+				n3 = as_node_insert (n, "developer_name", "Xfce Development Team",
+						     flags_translate, NULL);
+				as_node_set_comment (n3, "FIXME: this is a translatable version of project_group", -1);
 			} else if (g_strcmp0 (as_node_get_data (n3), "MATE") == 0) {
-				as_node_insert (n, "developer_name", "The MATE Project",
-						flags_translate, NULL);
+				n3 = as_node_insert (n, "developer_name", "The MATE Project",
+						     flags_translate, NULL);
+				as_node_set_comment (n3, "FIXME: this is a translatable version of project_group", -1);
 			} else {
 				action_required = TRUE;
-				n3 = as_node_insert (n, "developer_name", "FIXME: Company Name",
-						     AS_NODE_INSERT_FLAG_NONE, NULL);
+				n3 = as_node_insert (n, "developer_name", "<!-- Company Name -->",
+						     AS_NODE_INSERT_FLAG_PRE_ESCAPED, NULL);
 				as_node_set_comment (n3, "FIXME: compulsory_for_desktop was not recognised", -1);
 			}
 		} else {
 			action_required = TRUE;
-			n3 = as_node_insert (n, "developer_name", "FIXME: Company Name",
-					     AS_NODE_INSERT_FLAG_NONE, NULL);
+			n3 = as_node_insert (n, "developer_name", "<!-- Company Name -->",
+					     AS_NODE_INSERT_FLAG_PRE_ESCAPED, NULL);
 			as_node_set_comment (n3, "FIXME: You can use a project or "
 					     "developer name if there's no company", -1);
 		}
@@ -348,36 +353,36 @@ as_util_convert_appdata (GFile *file_input,
 	/* ensure each URL type exists */
 	if (as_node_find_with_attribute (n, "url", "type", "homepage") == NULL) {
 		if (g_strcmp0 (project_group, "GNOME") == 0) {
-			n3 = as_node_insert (n, "url", "FIXME: https://wiki.gnome.org/Apps/FIXME",
-					     AS_NODE_INSERT_FLAG_NONE,
+			n3 = as_node_insert (n, "url", "<!-- https://wiki.gnome.org/Apps/FIXME -->",
+					     AS_NODE_INSERT_FLAG_PRE_ESCAPED,
 					     "type", "homepage", NULL);
 		} else {
-			n3 = as_node_insert (n, "url", "FIXME: http://www.homepage.com/",
-					     AS_NODE_INSERT_FLAG_NONE,
+			n3 = as_node_insert (n, "url", "<!-- http://www.homepage.com/ -->",
+					     AS_NODE_INSERT_FLAG_PRE_ESCAPED,
 					     "type", "homepage", NULL);
 			as_node_set_comment (n3, "FIXME: homepage for the application", -1);
 		}
 	}
 	if (as_node_find_with_attribute (n, "url", "type", "bugtracker") == NULL) {
 		if (g_strcmp0 (project_group, "GNOME") == 0) {
-			n3 = as_node_insert (n, "url", "FIXME: https://bugzilla.gnome.org/enter_bug.cgi?product=FIXME",
-					     AS_NODE_INSERT_FLAG_NONE,
+			n3 = as_node_insert (n, "url", "<!-- https://bugzilla.gnome.org/enter_bug.cgi?product=FIXME -->",
+					     AS_NODE_INSERT_FLAG_PRE_ESCAPED,
 					     "type", "bugtracker", NULL);
 		} else if (g_strcmp0 (project_group, "KDE") == 0) {
-			n3 = as_node_insert (n, "url", "FIXME: https://bugs.kde.org/enter_bug.cgi?format=guided",
-					     AS_NODE_INSERT_FLAG_NONE,
+			n3 = as_node_insert (n, "url", "<!-- https://bugs.kde.org/enter_bug.cgi?format=guided -->",
+					     AS_NODE_INSERT_FLAG_PRE_ESCAPED,
 					     "type", "bugtracker", NULL);
 		} else if (g_strcmp0 (project_group, "XFCE") == 0) {
-			n3 = as_node_insert (n, "url", "FIXME: https://bugzilla.xfce.org/enter_bug.cgi",
-					     AS_NODE_INSERT_FLAG_NONE,
+			n3 = as_node_insert (n, "url", "<!-- https://bugzilla.xfce.org/enter_bug.cgi -->",
+					     AS_NODE_INSERT_FLAG_PRE_ESCAPED,
 					     "type", "bugtracker", NULL);
 		} else {
-			n3 = as_node_insert (n, "url", "FIXME: http://www.homepage.com/where-to-report_bug.html",
-					     AS_NODE_INSERT_FLAG_NONE,
+			n3 = as_node_insert (n, "url", "<!-- http://www.homepage.com/where-to-report_bug.html -->",
+					     AS_NODE_INSERT_FLAG_PRE_ESCAPED,
 					     "type", "bugtracker", NULL);
-			as_node_set_comment (n3, "FIXME: where to report bugs for "
-					     "the application, or empty for none", -1);
 		}
+		as_node_set_comment (n3, "FIXME: where to report bugs for "
+				     "the application", -1);
 	}
 	if (as_node_find_with_attribute (n, "url", "type", "donation") == NULL) {
 		if (g_strcmp0 (project_group, "GNOME") == 0) {
@@ -386,34 +391,42 @@ as_util_convert_appdata (GFile *file_input,
 					     "type", "donation", NULL);
 			as_node_set_comment (n3, "GNOME Projects usually have no per-app donation page", -1);
 		} else {
-			n3 = as_node_insert (n, "url", "FIXME: http://www.homepage.com/donation.html",
-					     AS_NODE_INSERT_FLAG_NONE,
+			n3 = as_node_insert (n, "url", "<!-- http://www.homepage.com/donation.html -->",
+					     AS_NODE_INSERT_FLAG_PRE_ESCAPED,
 					     "type", "donation", NULL);
 			as_node_set_comment (n3, "FIXME: where to donate to the application", -1);
 		}
 	}
 	if (as_node_find_with_attribute (n, "url", "type", "help") == NULL) {
 		if (g_strcmp0 (project_group, "GNOME") == 0) {
-			n3 = as_node_insert (n, "url", "FIXME: https://help.gnome.org/users/FIXME/stable/",
-					     AS_NODE_INSERT_FLAG_NONE,
+			n3 = as_node_insert (n, "url", "<!-- https://help.gnome.org/users/FIXME/stable/ -->",
+					     AS_NODE_INSERT_FLAG_PRE_ESCAPED,
 					     "type", "help", NULL);
 			as_node_set_comment (n3, "FIXME: where on the internet users "
-					     "can find help or empty for none", -1);
+					     "can find help", -1);
 		} else {
-			n3 = as_node_insert (n, "url", "FIXME: http://www.homepage.com/docs/",
-					     AS_NODE_INSERT_FLAG_NONE,
+			n3 = as_node_insert (n, "url", "<!-- http://www.homepage.com/docs/ -->",
+					     AS_NODE_INSERT_FLAG_PRE_ESCAPED,
 					     "type", "help", NULL);
 			as_node_set_comment (n3, "FIXME: where to report bugs for "
-					     "the application, or empty for none", -1);
+					     "the application", -1);
 		}
 	}
 
-	/* add <updatecontact> */
+	/* fix old <update_contact> name */
 	n2 = as_node_find (n, "updatecontact");
+	if (n2 != NULL)
+		as_node_set_name (n2, "update_contact");
+
+	/* add <update_contact> */
+	n2 = as_node_find (n, "updatecontact");
+	if (n2 != NULL)
+		as_node_set_name (n2, "update_contact");
+	n2 = as_node_find (n, "update_contact");
 	if (n2 == NULL) {
 		action_required = TRUE;
-		n3 = as_node_insert (n, "updatecontact", "FIXME: upstream_contact_at_email.com",
-				     AS_NODE_INSERT_FLAG_NONE, NULL);
+		n3 = as_node_insert (n, "update_contact", "<!-- upstream-contact_at_email.com -->",
+				     AS_NODE_INSERT_FLAG_PRE_ESCAPED, NULL);
 		as_node_set_comment (n3, "FIXME: this is optional, but recommended", -1);
 	}
 
@@ -435,9 +448,10 @@ as_util_convert_appdata (GFile *file_input,
 			as_node_set_data (n3, tmp, -1, AS_NODE_INSERT_FLAG_NONE);
 			as_node_set_data (n2, NULL, -1, AS_NODE_INSERT_FLAG_NONE);
 			action_required = TRUE;
-			as_node_insert (n2, "caption", "FIXME: Describe this "
-					"screenshot in less than ~10 words",
-					flags_translate, NULL);
+			as_node_insert (n2, "caption", "<!-- Describe this "
+					"screenshot in less than ~10 words -->",
+					flags_translate | AS_NODE_INSERT_FLAG_PRE_ESCAPED,
+					NULL);
 		}
 	}
 
@@ -479,6 +493,7 @@ as_util_convert_appstream (GFile *file_input,
 	as_store_set_api_version (store, new_version);
 	if (!as_store_to_file (store, file_output,
 				AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE |
+				AS_NODE_TO_XML_FLAG_FORMAT_INDENT |
 				AS_NODE_TO_XML_FLAG_ADD_HEADER,
 				NULL, error))
 		return FALSE;
@@ -536,10 +551,12 @@ as_util_convert (AsUtilPrivate *priv, gchar **values, GError **error)
 	}
 
 	/* don't know what to do */
-	g_set_error_literal (error,
-			     AS_ERROR,
-			     AS_ERROR_INVALID_ARGUMENTS,
-			     "Format not recognised");
+	g_set_error (error,
+		     AS_ERROR,
+		     AS_ERROR_INVALID_ARGUMENTS,
+		     "Conversion %s->%s not implemented",
+		     as_app_source_kind_to_string (input_kind),
+		     as_app_source_kind_to_string (output_kind));
 	return FALSE;
 }
 
@@ -549,8 +566,10 @@ as_util_convert (AsUtilPrivate *priv, gchar **values, GError **error)
 static gboolean
 as_util_upgrade (AsUtilPrivate *priv, gchar **values, GError **error)
 {
+	guint i;
+
 	/* check args */
-	if (g_strv_length (values) != 1) {
+	if (g_strv_length (values) < 1) {
 		/* TRANSLATORS: error message */
 		g_set_error_literal (error,
 				     AS_ERROR,
@@ -560,19 +579,32 @@ as_util_upgrade (AsUtilPrivate *priv, gchar **values, GError **error)
 		return FALSE;
 	}
 
-	/* AppData */
-	if (as_app_guess_source_kind (values[0]) == AS_APP_SOURCE_KIND_APPDATA) {
+	/* process each file */
+	for (i = 0; values[i] != NULL; i++) {
 		_cleanup_object_unref_ GFile *file = NULL;
-		file = g_file_new_for_path (values[0]);
-		return as_util_convert_appdata (file, file, 1.0, error);
+		AsAppSourceKind source_kind;
+		source_kind = as_app_guess_source_kind (values[i]);
+		switch (source_kind) {
+		case AS_APP_SOURCE_KIND_APPDATA:
+			file = g_file_new_for_path (values[i]);
+			if (!as_util_convert_appdata (file, file, 0.8, error))
+				return FALSE;
+			break;
+		case AS_APP_SOURCE_KIND_APPSTREAM:
+			file = g_file_new_for_path (values[i]);
+			if (!as_util_convert_appstream (file, file, 0.8, error))
+				return FALSE;
+			break;
+		default:
+			g_set_error (error,
+				     AS_ERROR,
+				     AS_ERROR_INVALID_ARGUMENTS,
+				     "File format '%s' cannot be upgraded",
+				     as_app_source_kind_to_string (source_kind));
+			return FALSE;
+		}
 	}
-
-	/* don't know what to do */
-	g_set_error_literal (error,
-			     AS_ERROR,
-			     AS_ERROR_INVALID_ARGUMENTS,
-			     "Format not recognised");
-	return FALSE;
+	return TRUE;
 }
 
 /**
@@ -633,10 +665,10 @@ as_util_appdata_from_desktop (AsUtilPrivate *priv, gchar **values, GError **erro
 	}
 
 	/* set things that don't belong in the AppData file */
-	as_app_set_icon (app, NULL, -1);
 	g_ptr_array_set_size (as_app_get_keywords (app, NULL), 0);
 	g_ptr_array_set_size (as_app_get_categories (app), 0);
 	g_ptr_array_set_size (as_app_get_mimetypes (app), 0);
+	g_ptr_array_set_size (as_app_get_icons (app), 0);
 
 	/* add urls */
 	as_app_add_url (app, AS_URL_KIND_HOMEPAGE,
@@ -761,222 +793,107 @@ as_util_dump (AsUtilPrivate *priv, gchar **values, GError **error)
 }
 
 /**
- * as_util_install_icons:
+ * as_util_search:
  **/
 static gboolean
-as_util_install_icons (const gchar *filename, const gchar *origin, GError **error)
+as_util_search (AsUtilPrivate *priv, gchar **values, GError **error)
 {
-	const gchar *destdir;
-	const gchar *tmp;
-	gboolean ret = TRUE;
-	gchar buf[PATH_MAX];
-	gsize len;
-	int r;
-	struct archive *arch = NULL;
-	struct archive_entry *entry;
-	_cleanup_free_ gchar *data = NULL;
-	_cleanup_free_ gchar *dir = NULL;
+	GPtrArray *apps;
+	guint i;
+	_cleanup_object_unref_ AsStore *store = NULL;
 
-	destdir = g_getenv ("DESTDIR");
-	dir = g_strdup_printf ("%s/usr/share/app-info/icons/%s",
-			       destdir != NULL ? destdir : "", origin);
-
-	/* load file at once to avoid seeking */
-	ret = g_file_get_contents (filename, &data, &len, error);
-	if (!ret)
-		goto out;
-
-	/* read anything */
-	arch = archive_read_new ();
-	archive_read_support_format_all (arch);
-	archive_read_support_filter_all (arch);
-	r = archive_read_open_memory (arch, data, len);
-	if (r) {
-		ret = FALSE;
-		g_set_error (error,
-			     AS_ERROR,
-			     AS_ERROR_FAILED,
-			     "Cannot open: %s",
-			     archive_error_string (arch));
-		goto out;
-	}
-
-	/* decompress each file */
-	for (;;) {
-		r = archive_read_next_header (arch, &entry);
-		if (r == ARCHIVE_EOF)
-			break;
-		if (r != ARCHIVE_OK) {
-			ret = FALSE;
-			g_set_error (error,
+	/* check args */
+	if (g_strv_length (values) < 1) {
+		g_set_error_literal (error,
 				     AS_ERROR,
-				     AS_ERROR_FAILED,
-				     "Cannot read header: %s",
-				     archive_error_string (arch));
-			goto out;
-		}
-
-		/* no output file */
-		if (archive_entry_pathname (entry) == NULL)
-			continue;
-
-		/* update output path */
-		g_snprintf (buf, PATH_MAX, "%s/%s",
-			    dir, archive_entry_pathname (entry));
-		archive_entry_update_pathname_utf8 (entry, buf);
-
-		/* update hardlinks */
-		tmp = archive_entry_hardlink (entry);
-		if (tmp != NULL) {
-			g_snprintf (buf, PATH_MAX, "%s/%s", dir, tmp);
-			archive_entry_update_hardlink_utf8 (entry, buf);
-		}
-
-		/* update symlinks */
-		tmp = archive_entry_symlink (entry);
-		if (tmp != NULL) {
-			g_snprintf (buf, PATH_MAX, "%s/%s", dir, tmp);
-			archive_entry_update_symlink_utf8 (entry, buf);
-		}
-
-		r = archive_read_extract (arch, entry, 0);
-		if (r != ARCHIVE_OK) {
-			ret = FALSE;
-			g_set_error (error,
-				     AS_ERROR,
-				     AS_ERROR_FAILED,
-				     "Cannot extract: %s",
-				     archive_error_string (arch));
-			goto out;
-		}
-	}
-out:
-	if (arch != NULL) {
-		archive_read_close (arch);
-		archive_read_free (arch);
-	}
-	return ret;
-}
-
-/**
- * as_util_install_xml:
- **/
-static gboolean
-as_util_install_xml (const gchar *filename,
-		     const gchar *origin,
-		     const gchar *dir,
-		     GError **error)
-{
-	const gchar *destdir;
-	gchar *tmp;
-	_cleanup_free_ gchar *basename = NULL;
-	_cleanup_free_ gchar *path_dest = NULL;
-	_cleanup_free_ gchar *path_parent = NULL;
-	_cleanup_object_unref_ GFile *file_dest = NULL;
-	_cleanup_object_unref_ GFile *file_src = NULL;
-
-	/* create directory structure */
-	destdir = g_getenv ("DESTDIR");
-	path_parent = g_strdup_printf ("%s%s", destdir != NULL ? destdir : "", dir);
-	if (g_mkdir_with_parents (path_parent, 0777) != 0) {
-		g_set_error (error,
-			     AS_ERROR,
-			     AS_ERROR_FAILED,
-			     "Failed to create %s", path_parent);
+				     AS_ERROR_INVALID_ARGUMENTS,
+				     "Not enough arguments, "
+				     "expected search terms");
 		return FALSE;
 	}
 
-	/* calculate the new destination */
-	file_src = g_file_new_for_path (filename);
-	basename = g_path_get_basename (filename);
-	if (origin != NULL) {
-		_cleanup_free_ gchar *basename_new = NULL;
-		tmp = g_strstr_len (basename, -1, ".");
-		if (tmp == NULL) {
-			g_set_error (error,
-				     AS_ERROR,
-				     AS_ERROR_FAILED,
-				     "Name of XML file invalid %s",
-				     basename);
-			return FALSE;
-		}
-		basename_new = g_strdup_printf ("%s%s", origin, tmp);
-		/* replace the fedora.xml.gz into %{origin}.xml.gz */
-		path_dest = g_build_filename (path_parent, basename_new, NULL);
-	} else {
-		path_dest = g_build_filename (path_parent, basename, NULL);
-	}
-
-	/* actually copy file */
-	file_dest = g_file_new_for_path (path_dest);
-	if (!g_file_copy (file_src, file_dest, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, error))
+	/* load system database */
+	store = as_store_new ();
+	if (!as_store_load (store, AS_STORE_LOAD_FLAG_APP_INFO_SYSTEM, NULL, error))
 		return FALSE;
-
-	/* fix the origin */
-	if (origin != NULL) {
-		_cleanup_object_unref_ AsStore *store = NULL;
-		store = as_store_new ();
-		if (!as_store_from_file (store, file_dest, NULL, NULL, error))
-			return FALSE;
-		as_store_set_origin (store, origin);
-		if (!as_store_to_file (store, file_dest,
-				       AS_NODE_TO_XML_FLAG_ADD_HEADER |
-				       AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE,
-				       NULL, error))
-			return FALSE;
+	apps = as_store_get_apps (store);
+	for (i = 0; i < apps->len; i++) {
+		AsApp *app;
+		app = g_ptr_array_index (apps, i);
+		if (as_app_search_matches_all (app, values))
+			g_print ("%s\n", as_app_get_id (app));
 	}
 	return TRUE;
 }
 
 /**
- * as_util_install_filename:
+ * as_util_search_token_sort_cb:
+ **/
+static gint
+as_util_search_token_sort_cb (gconstpointer a, gconstpointer b, gpointer user_data)
+{
+	guint *cnt_a;
+	guint *cnt_b;
+	GHashTable *dict = (GHashTable *) user_data;
+	cnt_a = g_hash_table_lookup (dict, (const gchar *) a);
+	cnt_b = g_hash_table_lookup (dict, (const gchar *) b);
+	if (*cnt_a < *cnt_b)
+		return 1;
+	if (*cnt_a > *cnt_b)
+		return -1;
+	return 0;
+}
+
+/**
+ * as_util_show_search_tokens:
  **/
 static gboolean
-as_util_install_filename (const gchar *filename,
-			  const gchar *origin,
-			  GError **error)
+as_util_show_search_tokens (AsUtilPrivate *priv, gchar **values, GError **error)
 {
-	gboolean ret = FALSE;
-	gchar *tmp;
-	_cleanup_free_ gchar *basename = NULL;
+	GPtrArray *apps;
+	GList *l;
+	guint i;
+	guint j;
+	const gchar *tmp;
+	guint *cnt;
+	_cleanup_hashtable_unref_ GHashTable *dict = NULL;
+	_cleanup_object_unref_ AsStore *store = NULL;
+	_cleanup_list_free_ GList *keys = NULL;
 
-	switch (as_app_guess_source_kind (filename)) {
-	case AS_APP_SOURCE_KIND_APPSTREAM:
-		if (g_strstr_len (filename, -1, ".yml.gz") != NULL) {
-			ret = as_util_install_xml (filename, origin,
-						   "/usr/share/app-info/yaml", error);
-		} else {
-			ret = as_util_install_xml (filename, origin,
-						   "/usr/share/app-info/xmls", error);
+	/* load system database */
+	store = as_store_new ();
+	if (!as_store_load (store, AS_STORE_LOAD_FLAG_APP_INFO_SYSTEM, NULL, error))
+		return FALSE;
+	dict = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+	apps = as_store_get_apps (store);
+	for (i = 0; i < apps->len; i++) {
+		AsApp *app;
+		_cleanup_ptrarray_unref_ GPtrArray *tokens = NULL;
+		app = g_ptr_array_index (apps, i);
+		tokens = as_app_get_search_tokens (app);
+		for (j = 0; j < tokens->len; j++) {
+			tmp = g_ptr_array_index (tokens, j);
+			cnt = g_hash_table_lookup (dict, tmp);
+			if (cnt == NULL) {
+				cnt = g_new0 (guint, 1);
+				g_hash_table_insert (dict,
+						     g_strdup (tmp),
+						     cnt);
+			}
+			(*cnt)++;
 		}
-		break;
-	case AS_APP_SOURCE_KIND_APPDATA:
-	case AS_APP_SOURCE_KIND_METAINFO:
-		ret = as_util_install_xml (filename, NULL,
-					   "/usr/share/appdata", error);
-		break;
-	default:
-		/* icons */
-		if (origin != NULL) {
-			ret = as_util_install_icons (filename, origin, error);
-			break;
-		}
-		basename = g_path_get_basename (filename);
-		tmp = g_strstr_len (basename, -1, "-icons.tar.gz");
-		if (tmp != NULL) {
-			*tmp = '\0';
-			ret = as_util_install_icons (filename, basename, error);
-			break;
-		}
-
-		/* unrecognised */
-		g_set_error_literal (error,
-				     AS_ERROR,
-				     AS_ERROR_FAILED,
-				     "No idea how to process files of this type");
-		break;
 	}
-	return ret;
+
+	/* display the keywords sorted */
+	keys = g_hash_table_get_keys (dict);
+	keys = g_list_sort_with_data (keys, as_util_search_token_sort_cb, dict);
+	for (l = keys; l != NULL; l = l->next) {
+		tmp = l->data;
+		cnt = g_hash_table_lookup (dict, tmp);
+		g_print ("%s [%i]\n", tmp, *cnt);
+	}
+
+	return TRUE;
 }
 
 /**
@@ -1000,7 +917,10 @@ as_util_install (AsUtilPrivate *priv, gchar **values, GError **error)
 	/* for each item on the command line, install the xml files and
 	 * explode the icon files */
 	for (i = 0; values[i] != NULL; i++) {
-		if (!as_util_install_filename (values[i], NULL, error))
+		if (!as_utils_install_filename (AS_UTILS_LOCATION_SHARED,
+						values[i], NULL,
+						g_getenv ("DESTDIR"),
+						error))
 			return FALSE;
 	}
 	return TRUE;
@@ -1027,7 +947,10 @@ as_util_install_origin (AsUtilPrivate *priv, gchar **values, GError **error)
 	/* for each item on the command line, install the xml files and
 	 * explode the icon files */
 	for (i = 1; values[i] != NULL; i++) {
-		if (!as_util_install_filename (values[i], values[0], error))
+		if (!as_utils_install_filename (AS_UTILS_LOCATION_CACHE,
+						values[i], values[0],
+						g_getenv ("DESTDIR"),
+						error))
 			return FALSE;
 	}
 	return TRUE;
@@ -1192,6 +1115,8 @@ as_util_status_html_write_app (AsApp *app, GString *html, AsUtilDistro distro)
 		g_string_append (classes, "kde ");
 	if (g_strcmp0 (as_app_get_project_group (app), "XFCE") == 0)
 		g_string_append (classes, "xfce ");
+	if (as_app_get_id_kind (app) == AS_ID_KIND_ADDON)
+		g_string_append (classes, "addon ");
 	if (classes->len > 0)
 		g_string_truncate (classes, classes->len - 1);
 
@@ -1440,6 +1365,8 @@ as_util_status_html_write_exec_summary (GPtrArray *apps,
 			if (g_strcmp0 (as_app_get_project_group (app),
 				       project_groups[j]) != 0)
 				continue;
+			if (as_app_get_id_kind (app) == AS_ID_KIND_ADDON)
+				continue;
 			total += 1;
 			if (as_app_get_screenshots(app)->len > 0 ||
 			    as_app_get_description (app, "C") != NULL)
@@ -1455,6 +1382,32 @@ as_util_status_html_write_exec_summary (GPtrArray *apps,
 					total, perc);
 	}
 
+	/* specific kudos */
+	total = 0;
+	for (i = 0; i < apps->len; i++) {
+		app = g_ptr_array_index (apps, i);
+		if (as_app_get_id_kind (app) != AS_ID_KIND_DESKTOP)
+			continue;
+		total++;
+	}
+	for (j = 1; j < AS_KUDO_KIND_LAST; j++) {
+		cnt = 0;
+		for (i = 0; i < apps->len; i++) {
+			app = g_ptr_array_index (apps, i);
+			if (!as_app_has_kudo_kind (app, j))
+				continue;
+			cnt += 1;
+		}
+		perc = 0;
+		if (total > 0)
+			perc = 100.f * (gdouble) cnt / (gdouble) total;
+		g_string_append_printf (html, "<tr><td class=\"alt\">"
+					"<i>%s</i></td><td>%i</td>"
+					"<td class=\"thin\">%.1f%%</td></tr>\n",
+					as_kudo_kind_to_string (j),
+					cnt, perc);
+	}
+
 	/* addons with MetaInfo */
 	cnt = 0;
 	for (i = 0; i < apps->len; i++) {
@@ -1494,7 +1447,7 @@ as_util_status_html_write_javascript (GString *html)
 	"function\n"
 	"statusReset(class_name, val)\n"
 	"{\n"
-	"	var class_names = [ 'description', 'screenshots', 'keywords', 'gnome', 'kde', 'xfce' ];\n"
+	"	var class_names = [ 'description', 'screenshots', 'keywords', 'gnome', 'kde', 'xfce', 'addon' ];\n"
 	"	for (var i = 0; i < class_names.length; i++) {\n"
 	"		document.getElementById('button-' + class_names[i] + '-with').disabled = false;\n"
 	"		document.getElementById('button-' + class_names[i] + '-without').disabled = false;\n"
@@ -1624,6 +1577,12 @@ as_util_status_html_write_filter_section (GString *html)
 	"<td><button id=\"button-xfce-without\" onclick=\"statusHideFilter('xfce', true);\">&#x2715;</button></td>\n"
 	"</tr>\n"
 	"<tr>\n"
+	"<tr>\n"
+	"<td class=\"alt\">Hide Addons</td>\n"
+	"<td><button id=\"button-addon-with\" onclick=\"statusHideFilter('addon', false);\">&#10003;</button></td>\n"
+	"<td><button id=\"button-addon-without\" onclick=\"statusHideFilter('addon', true);\">&#x2715;</button></td>\n"
+	"</tr>\n"
+	"<tr>\n"
 	"<td colspan=\"3\"><button id=\"button-reset\" onclick=\"statusReset('description');\">Reset Filters</button></td>\n"
 	"</tr>\n"
 	"</table>\n");
@@ -1714,6 +1673,309 @@ as_util_status_html (AsUtilPrivate *priv, gchar **values, GError **error)
 	return TRUE;
 }
 
+typedef enum {
+	AS_UTIL_PKG_STATE_OK, /* in order of badness */
+	AS_UTIL_PKG_STATE_INFO,
+	AS_UTIL_PKG_STATE_WARN,
+	AS_UTIL_PKG_STATE_FAIL
+} AsUtilPkgState;
+
+/**
+ * as_util_matrix_html_write_item:
+ */
+static void
+as_util_matrix_html_write_item (AsUtilPkgState *state_app,
+				    AsUtilPkgState state,
+				    GString *html,
+				    const gchar *comment)
+{
+	g_string_append (html, "<td>");
+
+	/* ab-use acronym for the mouse-over explaination */
+	if (comment != NULL)
+		g_string_append_printf (html, "<acronym title=\"%s\">", comment);
+
+	switch (state) {
+	case AS_UTIL_PKG_STATE_OK:
+		g_string_append (html, "OK");
+		break;
+	case AS_UTIL_PKG_STATE_INFO:
+		g_string_append (html, "Info");
+		break;
+	case AS_UTIL_PKG_STATE_WARN:
+		g_string_append (html, "Warning");
+		break;
+	case AS_UTIL_PKG_STATE_FAIL:
+		g_string_append (html, "Failed");
+		break;
+	default:
+		break;
+	}
+	if (comment != NULL)
+		g_string_append (html, "</acronym>");
+
+	g_string_append (html, "</td>\n");
+
+	/* update the global state */
+	if (state_app != NULL) {
+		if (state > *state_app)
+			*state_app = state;
+	}
+}
+
+/**
+ * as_util_matrix_html_write_app:
+ */
+static void
+as_util_matrix_html_write_app (AsApp *app, GString *html, AsUtilDistro distro)
+{
+	AsIcon *ic;
+	AsUtilPkgState state_app = AS_UTIL_PKG_STATE_OK;
+	GPtrArray *arr;
+	_cleanup_string_free_ GString *str = NULL;
+
+	str = g_string_new ("");
+	g_string_append_printf (str, "<td>%s</td>\n", as_app_get_id_filename (app));
+
+	/* pkgname */
+	switch (distro) {
+	case AS_UTIL_DISTRO_FEDORA:
+		g_string_append_printf (str, "<td><a href=\"https://apps.fedoraproject.org/packages/%s\">%s</a></td>\n",
+					as_app_get_pkgname_default (app),
+					as_app_get_pkgname_default (app));
+		break;
+	default:
+		g_string_append_printf (str, "<td>%s</td>\n", as_app_get_pkgname_default (app));
+		break;
+	}
+
+	/* summary */
+	if (as_app_get_comment (app, NULL) == NULL) {
+		as_util_matrix_html_write_item (&state_app,
+						AS_UTIL_PKG_STATE_FAIL,
+						str,
+						"No comment in .desktop or summary in AppData file");
+	} else {
+		as_util_matrix_html_write_item (NULL, AS_UTIL_PKG_STATE_OK, str, NULL);
+	}
+
+	/* description */
+	if (as_app_get_description (app, NULL) == NULL) {
+		as_util_matrix_html_write_item (&state_app,
+						AS_UTIL_PKG_STATE_WARN,
+						str,
+						"No long description in AppData file");
+	} else {
+		as_util_matrix_html_write_item (NULL, AS_UTIL_PKG_STATE_OK, str, NULL);
+	}
+
+	/* screenshots */
+	arr = as_app_get_screenshots (app);
+	if (arr ==  NULL || arr->len == 0) {
+		as_util_matrix_html_write_item (&state_app,
+						AS_UTIL_PKG_STATE_WARN,
+						str,
+						"No screenshots in AppData file");
+	} else {
+		as_util_matrix_html_write_item (NULL, AS_UTIL_PKG_STATE_OK, str, NULL);
+	}
+
+	/* icons */
+	ic = as_app_get_icon_default (app);
+	if (ic == NULL) {
+		as_util_matrix_html_write_item (&state_app,
+						AS_UTIL_PKG_STATE_FAIL,
+						str,
+						"No icon");
+	} else {
+		if (!as_app_has_kudo_kind (app, AS_KUDO_KIND_HI_DPI_ICON))
+			as_util_matrix_html_write_item (&state_app,
+							AS_UTIL_PKG_STATE_INFO,
+							str,
+							"No HiDPI icon");
+		else
+			as_util_matrix_html_write_item (NULL, AS_UTIL_PKG_STATE_OK, str, NULL);
+	}
+
+	/* keywords */
+	arr = as_app_get_keywords (app, NULL);
+	if (arr == NULL || arr->len == 0) {
+		as_util_matrix_html_write_item (&state_app,
+						AS_UTIL_PKG_STATE_WARN,
+						str,
+						"No keywords in .desktop file");
+	} else {
+		as_util_matrix_html_write_item (NULL, AS_UTIL_PKG_STATE_OK, str, NULL);
+	}
+
+	/* veto */
+	arr = as_app_get_vetos (app);
+	if (arr == NULL || arr->len == 0) {
+		as_util_matrix_html_write_item (NULL, AS_UTIL_PKG_STATE_OK, str, NULL);
+	} else {
+		_cleanup_free_ gchar *tmp = NULL;
+		tmp = as_util_status_html_join (arr);
+		as_util_matrix_html_write_item (&state_app,
+						AS_UTIL_PKG_STATE_FAIL,
+						str, tmp);
+	}
+
+	/* global state */
+	switch (state_app) {
+	case AS_UTIL_PKG_STATE_OK:
+		g_string_prepend (str, "<tr bgcolor=\"#a7f6a7\">\n");
+		break;
+	case AS_UTIL_PKG_STATE_INFO:
+		g_string_prepend (str, "<tr bgcolor=\"#e0edfb\">\n");
+		break;
+	case AS_UTIL_PKG_STATE_WARN:
+		g_string_prepend (str, "<tr bgcolor=\"#fbfbe0\">\n");
+		break;
+	case AS_UTIL_PKG_STATE_FAIL:
+		g_string_prepend (str, "<tr bgcolor=\"#ffa1a5\">\n");
+		break;
+	default:
+		g_string_prepend (str, "<tr>\n");
+		break;
+	}
+
+	g_string_append (str, "</tr>\n");
+	g_string_append (html, str->str);
+}
+
+/**
+ * as_util_array_sort_by_pkgname_cb:
+ **/
+static gint
+as_util_array_sort_by_pkgname_cb (gconstpointer a, gconstpointer b)
+{
+	AsApp *app1 = *((AsApp **) a);
+	AsApp *app2 = *((AsApp **) b);
+	return g_strcmp0 (as_app_get_pkgname_default (app1),
+			  as_app_get_pkgname_default (app2));
+}
+
+/**
+ * as_util_matrix_html:
+ **/
+static gboolean
+as_util_matrix_html (AsUtilPrivate *priv, gchar **values, GError **error)
+{
+	AsApp *app;
+	AsUtilDistro distro = AS_UTIL_DISTRO_UNKNOWN;
+	GPtrArray *apps = NULL;
+	guint i;
+	_cleanup_object_unref_ AsStore *store = NULL;
+	_cleanup_string_free_ GString *html = NULL;
+
+	/* check args */
+	if (g_strv_length (values) < 2) {
+		g_set_error_literal (error,
+				     AS_ERROR,
+				     AS_ERROR_INVALID_ARGUMENTS,
+				     "Not enough arguments, "
+				     "expected matrix.html filename.xml.gz");
+		return FALSE;
+	}
+
+	/* load file */
+	store = as_store_new ();
+	for (i = 1; values[i] != NULL; i++) {
+		_cleanup_object_unref_ GFile *file = NULL;
+		file = g_file_new_for_path (values[i]);
+		if (!as_store_from_file (store, file, NULL, NULL, error))
+			return FALSE;
+	}
+	apps = as_store_get_apps (store);
+
+	/* order by package name */
+	g_ptr_array_sort (apps, as_util_array_sort_by_pkgname_cb);
+
+	/* detect distro */
+	if (g_strstr_len (values[1], -1, "fedora") != NULL)
+		distro = AS_UTIL_DISTRO_FEDORA;
+
+	/* create header */
+	html = g_string_new ("");
+	g_string_append (html, "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 "
+			       "Transitional//EN\" "
+			       "\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n");
+	g_string_append (html, "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n");
+	g_string_append (html, "<head>\n");
+	g_string_append (html, "<meta http-equiv=\"Content-Type\" content=\"text/html; "
+			       "charset=UTF-8\" />\n");
+	g_string_append (html, "<title>Application Data Matrix</title>\n");
+	as_util_status_html_write_css (html);
+	g_string_append (html, "</head>\n");
+	g_string_append (html, "<body>\n");
+
+	/* write applications */
+	g_string_append (html, "<h1>Packages</h1>\n");
+	g_string_append (html, "<div id=\"apps\">\n");
+	g_string_append (html, "<table>\n");
+
+	/* table header */
+	g_string_append (html, "<tr>\n");
+	g_string_append (html, "<th>Application ID</th>\n");
+	g_string_append (html, "<th>Package Name</th>\n");
+	g_string_append (html, "<th>Summary</th>\n");
+	g_string_append (html, "<th>Description</th>\n");
+	g_string_append (html, "<th>Screenshots</th>\n");
+	g_string_append (html, "<th>Icon</th>\n");
+	g_string_append (html, "<th>Keywords</th>\n");
+	g_string_append (html, "<th>Veto</th>\n");
+	g_string_append (html, "</tr>\n");
+
+	/* apps */
+	for (i = 0; i < apps->len; i++) {
+		app = g_ptr_array_index (apps, i);
+		if (as_app_get_id_kind (app) != AS_ID_KIND_DESKTOP)
+			continue;
+		as_util_matrix_html_write_app (app, html, distro);
+	}
+
+	/* footer */
+	g_string_append (html, "</table>\n");
+	g_string_append (html, "</div>\n");
+	g_string_append (html, "</body>\n");
+	g_string_append (html, "</html>\n");
+
+	/* save file */
+	return g_file_set_contents (values[0], html->str, -1, error);
+}
+
+/**
+ * as_util_status_csv_filter_func:
+ **/
+static gboolean
+as_util_status_csv_filter_func (AsApp *app, gchar **filters)
+{
+	const gchar *tmp;
+	guint i;
+	AsIdKind id_kind = AS_ID_KIND_DESKTOP;
+
+	for (i = 0; filters[i] != NULL; i++) {
+		_cleanup_strv_free_ gchar **split = NULL;
+		split = g_strsplit (filters[i], "=", 2);
+		if (g_strv_length (split) != 2)
+			continue;
+		if (g_strcmp0 (split[0], "id-kind") == 0) {
+			id_kind = as_id_kind_from_string (split[1]);
+			if (as_app_get_id_kind (app) != id_kind)
+				return FALSE;
+			continue;
+		}
+		if (g_strcmp0 (split[0], "metadata") == 0) {
+			tmp = as_app_get_metadata_item (app, split[1]);
+			if (tmp == NULL)
+				return FALSE;
+			continue;
+		}
+		g_warning ("Unknown filter option %s:%s", split[0], split[1]);
+	}
+	return TRUE;
+}
+
 /**
  * as_util_status_csv:
  **/
@@ -1721,8 +1983,8 @@ static gboolean
 as_util_status_csv (AsUtilPrivate *priv, gchar **values, GError **error)
 {
 	AsApp *app;
-	AsIdKind id_kind = AS_ID_KIND_DESKTOP;
 	GPtrArray *apps = NULL;
+	const gchar *tmp;
 	guint i;
 	_cleanup_object_unref_ AsStore *store = NULL;
 	_cleanup_object_unref_ GFile *file = NULL;
@@ -1738,19 +2000,6 @@ as_util_status_csv (AsUtilPrivate *priv, gchar **values, GError **error)
 		return FALSE;
 	}
 
-	/* process filters */
-	for (i = 2; values[i] != NULL; i++) {
-		_cleanup_strv_free_ gchar **split = NULL;
-		split = g_strsplit (values[i], "=", 2);
-		if (g_strv_length (split) != 2)
-			continue;
-		if (g_strcmp0 (split[0], "id-kind") == 0) {
-			id_kind = as_id_kind_from_string (split[1]);
-			continue;
-		}
-		g_warning ("Unknown filter option %s:%s", split[0], split[1]);
-	}
-
 	/* load file */
 	store = as_store_new ();
 	file = g_file_new_for_path (values[0]);
@@ -1763,7 +2012,9 @@ as_util_status_csv (AsUtilPrivate *priv, gchar **values, GError **error)
 	for (i = 0; i < apps->len; i++) {
 		_cleanup_free_ gchar *description = NULL;
 		app = g_ptr_array_index (apps, i);
-		if (as_app_get_id_kind (app) != id_kind)
+
+		/* process filters */
+		if (!as_util_status_csv_filter_func (app, values + 2))
 			continue;
 		g_string_append_printf (data, "%s,", as_app_get_id (app));
 		g_string_append_printf (data, "%s,", as_app_get_pkgname_default (app));
@@ -1774,8 +2025,9 @@ as_util_status_csv (AsUtilPrivate *priv, gchar **values, GError **error)
 			g_strdelimit (description, "\n", '|');
 			g_strdelimit (description, "\"", '\'');
 		}
-		g_string_append_printf (data, "\"%s\",", description);
-		g_string_append_printf (data, "\"%s\",", as_app_get_url_item (app, AS_URL_KIND_HOMEPAGE));
+		g_string_append_printf (data, "\"%s\",", description ? description : "");
+		tmp = as_app_get_url_item (app, AS_URL_KIND_HOMEPAGE);
+		g_string_append_printf (data, "\"%s\",", tmp ? tmp : "");
 		g_string_truncate (data, data->len - 1);
 		g_string_append (data, "\n");
 	}
@@ -1930,13 +2182,14 @@ as_util_validate_file (const gchar *filename,
 	g_print ("%s: ", filename);
 	if (as_app_guess_source_kind (filename) == AS_APP_SOURCE_KIND_APPSTREAM) {
 		gboolean ret;
-		_cleanup_object_unref_ AsStore *store;
-		_cleanup_object_unref_ GFile *file;
+		_cleanup_object_unref_ AsStore *store = NULL;
+		_cleanup_object_unref_ GFile *file = NULL;
 		file = g_file_new_for_path (filename);
 		store = as_store_new ();
 		ret = as_store_from_file (store, file, NULL, NULL, error);
 		if (!ret)
 			return FALSE;
+		flags |= AS_APP_VALIDATE_FLAG_ALL_APPS;
 		probs = as_store_validate (store, flags, error);
 		if (probs == NULL)
 			return FALSE;
@@ -2052,11 +2305,13 @@ as_util_validate_strict (AsUtilPrivate *priv, gchar **values, GError **error)
 static gboolean
 as_util_check_root_app_icon (AsApp *app, GError **error)
 {
+	AsIcon *icon_default;
 	_cleanup_free_ gchar *icon = NULL;
 	_cleanup_object_unref_ GdkPixbuf *pb = NULL;
 
 	/* nothing found */
-	if (as_app_get_icon (app) == NULL) {
+	icon_default = as_app_get_icon_default (app);
+	if (icon_default == NULL) {
 		g_set_error (error,
 			     AS_ERROR,
 			     AS_ERROR_FAILED,
@@ -2066,18 +2321,18 @@ as_util_check_root_app_icon (AsApp *app, GError **error)
 	}
 
 	/* is stock icon */
-	if (as_utils_is_stock_icon_name (as_app_get_icon (app)))
+	if (as_utils_is_stock_icon_name (as_icon_get_name (icon_default)))
 		return TRUE;
 
 	/* can we find it */
 	icon = as_utils_find_icon_filename (g_getenv ("DESTDIR"),
-					    as_app_get_icon (app),
+					    as_icon_get_name (icon_default),
 					    error);
 	if (icon == NULL) {
 		g_prefix_error (error,
 				"%s missing icon %s: ",
 				as_app_get_id (app),
-				as_app_get_icon (app));
+				as_icon_get_name (icon_default));
 		return FALSE;
 	}
 
@@ -2087,7 +2342,7 @@ as_util_check_root_app_icon (AsApp *app, GError **error)
 		g_prefix_error (error,
 				"%s invalid icon %s: ",
 				as_app_get_id (app),
-				as_app_get_icon (app));
+				as_icon_get_name (icon_default));
 		return FALSE;
 	}
 
@@ -2284,6 +2539,18 @@ main (int argc, char *argv[])
 		     /* TRANSLATORS: command description */
 		     _("Dumps the applications in the AppStream metadata"),
 		     as_util_dump);
+	as_util_add (priv->cmd_array,
+		     "search",
+		     NULL,
+		     /* TRANSLATORS: command description */
+		     _("Search for AppStream applications"),
+		     as_util_search);
+	as_util_add (priv->cmd_array,
+		     "show-search-tokens",
+		     NULL,
+		     /* TRANSLATORS: command description */
+		     _("Display application search tokens"),
+		     as_util_show_search_tokens);
 	as_util_add (priv->cmd_array,
 		     "install",
 		     NULL,
@@ -2314,6 +2581,12 @@ main (int argc, char *argv[])
 		     /* TRANSLATORS: command description */
 		     _("Create an CSV status document"),
 		     as_util_status_csv);
+	as_util_add (priv->cmd_array,
+		     "matrix-html",
+		     NULL,
+		     /* TRANSLATORS: command description */
+		     _("Create an HTML matrix page"),
+		     as_util_matrix_html);
 	as_util_add (priv->cmd_array,
 		     "non-package-yaml",
 		     NULL,
