@@ -39,9 +39,9 @@ typedef struct _AsbAppPrivate	AsbAppPrivate;
 struct _AsbAppPrivate
 {
 	GPtrArray	*requires_appdata;
-	GdkPixbuf	*pixbuf;
 	AsbPackage	*pkg;
 	gboolean	 ignore_requires_appdata;
+	gboolean	 hidpi_enabled;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsbApp, asb_app, AS_TYPE_APP)
@@ -58,8 +58,6 @@ asb_app_finalize (GObject *object)
 	AsbAppPrivate *priv = GET_PRIVATE (app);
 
 	g_ptr_array_unref (priv->requires_appdata);
-	if (priv->pixbuf != NULL)
-		g_object_unref (priv->pixbuf);
 	if (priv->pkg != NULL)
 		g_object_unref (priv->pkg);
 
@@ -91,31 +89,6 @@ asb_app_class_init (AsbAppClass *klass)
 	object_class->finalize = asb_app_finalize;
 }
 
-/**
- * asb_app_to_xml:
- * @app: A #AsbApp
- *
- * Converts the application to it's XML representation.
- *
- * Returns: allocated string
- *
- * Since: 0.1.0
- **/
-gchar *
-asb_app_to_xml (AsbApp *app)
-{
-	GString *str;
-	_cleanup_object_unref_ AsStore *store;
-
-	store = as_store_new ();
-	as_store_set_api_version (store, 1.0f);
-	as_store_add_app (store, AS_APP (app));
-	str = as_store_to_xml (store,
-			       AS_NODE_TO_XML_FLAG_FORMAT_INDENT |
-			       AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
-	return g_string_free (str, FALSE);
-}
-
 /**
  * asb_app_add_requires_appdata:
  * @app: A #AsbApp
@@ -163,24 +136,6 @@ asb_app_set_requires_appdata (AsbApp *app, gboolean requires_appdata)
 	}
 }
 
-/**
- * asb_app_set_pixbuf:
- * @app: A #AsbApp
- * @pixbuf: a #GdkPixbuf
- *
- * Sets the icon for the application.
- *
- * Since: 0.1.0
- **/
-void
-asb_app_set_pixbuf (AsbApp *app, GdkPixbuf *pixbuf)
-{
-	AsbAppPrivate *priv = GET_PRIVATE (app);
-	if (priv->pixbuf != NULL)
-		g_object_ref (priv->pixbuf);
-	priv->pixbuf = g_object_ref (pixbuf);
-}
-
 /**
  * asb_app_get_requires_appdata:
  * @app: A #AsbApp
@@ -226,7 +181,7 @@ asb_app_save_resources_image (AsbApp *app,
 	const gchar *output_dir;
 	gboolean ret = TRUE;
 	_cleanup_free_ gchar *filename = NULL;
-	_cleanup_free_ gchar *size_str;
+	_cleanup_free_ gchar *size_str = NULL;
 
 	/* treat source images differently */
 	if (as_image_get_kind (image) == AS_IMAGE_KIND_SOURCE) {
@@ -239,6 +194,8 @@ asb_app_save_resources_image (AsbApp *app,
 
 	/* does screenshot already exist */
 	output_dir = asb_package_get_config (asb_app_get_package (app), "ScreenshotDir");
+	if (output_dir == NULL)
+		return TRUE;
 	filename = g_build_filename (output_dir,
 				     size_str,
 				     as_image_get_basename (image),
@@ -291,9 +248,26 @@ asb_app_save_resources_screenshot (AsbApp *app,
 	return TRUE;
 }
 
+/**
+ * asb_app_set_hidpi_enabled:
+ * @app: A #AsbApp
+ * @hidpi_enabled: if HiDPI mode should be enabled
+ *
+ * Sets the HiDPI mode for the application.
+ *
+ * Since: 0.3.1
+ **/
+void
+asb_app_set_hidpi_enabled (AsbApp *app, gboolean hidpi_enabled)
+{
+	AsbAppPrivate *priv = GET_PRIVATE (app);
+	priv->hidpi_enabled = hidpi_enabled;
+}
+
 /**
  * asb_app_save_resources:
  * @app: A #AsbApp
+ * @save_flags: #AsbAppSaveFlags, e.g. %ASB_APP_SAVE_FLAG_SCREENSHOTS
  * @error: A #GError or %NULL
  *
  * Saves to disk any resources set for the application.
@@ -303,24 +277,48 @@ asb_app_save_resources_screenshot (AsbApp *app,
  * Since: 0.1.0
  **/
 gboolean
-asb_app_save_resources (AsbApp *app, GError **error)
+asb_app_save_resources (AsbApp *app, AsbAppSaveFlags save_flags, GError **error)
 {
 	AsbAppPrivate *priv = GET_PRIVATE (app);
+	AsIcon *icon;
 	AsScreenshot *ss;
+	GdkPixbuf *pixbuf;
+	GPtrArray *icons = NULL;
+	GPtrArray *screenshots = NULL;
 	guint i;
-	GPtrArray *screenshots;
 
 	/* any non-stock icon set */
-	if (priv->pixbuf != NULL) {
+	if (save_flags & ASB_APP_SAVE_FLAG_ICONS)
+		icons = as_app_get_icons (AS_APP (app));
+	for (i = 0; icons != NULL && i < icons->len; i++) {
 		const gchar *tmpdir;
 		_cleanup_free_ gchar *filename = NULL;
+		_cleanup_free_ gchar *size_str = NULL;
 
+		/* don't save some types of icons */
+		icon = g_ptr_array_index (icons, i);
+		if (as_icon_get_kind (icon) == AS_ICON_KIND_STOCK ||
+		    as_icon_get_kind (icon) == AS_ICON_KIND_EMBEDDED ||
+		    as_icon_get_kind (icon) == AS_ICON_KIND_LOCAL ||
+		    as_icon_get_kind (icon) == AS_ICON_KIND_REMOTE)
+			continue;
+
+		/* save to disk */
 		tmpdir = asb_package_get_config (priv->pkg, "TempDir");
 		filename = g_build_filename (tmpdir,
 					     "icons",
-					     as_app_get_icon (AS_APP (app)),
+					     as_icon_get_name (icon),
 					     NULL);
-		if (!gdk_pixbuf_save (priv->pixbuf, filename, "png", error, NULL))
+		pixbuf = as_icon_get_pixbuf (icon);
+		if (pixbuf == NULL) {
+			g_set_error (error,
+				     AS_APP_ERROR,
+				     AS_APP_ERROR_FAILED,
+				     "No pixbuf for %s",
+				     as_icon_get_name (icon));
+			return FALSE;
+		}
+		if (!gdk_pixbuf_save (pixbuf, filename, "png", error, NULL))
 			return FALSE;
 
 		/* set new AppStream compatible icon name */
@@ -330,8 +328,9 @@ asb_app_save_resources (AsbApp *app, GError **error)
 	}
 
 	/* save any screenshots */
-	screenshots = as_app_get_screenshots (AS_APP (app));
-	for (i = 0; i < screenshots->len; i++) {
+	if (save_flags & ASB_APP_SAVE_FLAG_SCREENSHOTS)
+		screenshots = as_app_get_screenshots (AS_APP (app));
+	for (i = 0; screenshots != NULL && i < screenshots->len; i++) {
 		ss = g_ptr_array_index (screenshots, i);
 		if (!asb_app_save_resources_screenshot (app, ss, error))
 			return FALSE;
@@ -339,6 +338,47 @@ asb_app_save_resources (AsbApp *app, GError **error)
 	return TRUE;
 }
 
+/**
+ * asb_app_save_thumbnail:
+ **/
+static gboolean
+asb_app_save_thumbnail (AsScreenshot *ss, AsImage *im_src,
+			guint width, guint height, guint scale,
+			const gchar *mirror_uri,
+			GError **error)
+{
+	_cleanup_free_ gchar *size_str = NULL;
+	_cleanup_free_ gchar *url_tmp = NULL;
+	_cleanup_object_unref_ AsImage *im_tmp = NULL;
+	_cleanup_object_unref_ GdkPixbuf *pixbuf = NULL;
+
+	/* only save the HiDPI screenshot if it's not padded */
+	if (scale > 1) {
+		if (width * scale > as_image_get_width (im_src) ||
+		    height * scale > as_image_get_height (im_src))
+			return TRUE;
+	}
+	size_str = g_strdup_printf ("%ix%i", width, height);
+	url_tmp = g_build_filename (mirror_uri,
+				    size_str,
+				    as_image_get_basename (im_src),
+				    NULL);
+	pixbuf = as_image_save_pixbuf (im_src,
+				       width * scale,
+				       height * scale,
+				       AS_IMAGE_SAVE_FLAG_PAD_16_9 |
+				       AS_IMAGE_SAVE_FLAG_SHARPEN);
+	im_tmp = as_image_new ();
+	as_image_set_width (im_tmp, width * scale);
+	as_image_set_height (im_tmp, height * scale);
+	as_image_set_url (im_tmp, url_tmp, -1);
+	as_image_set_pixbuf (im_tmp, pixbuf);
+	as_image_set_kind (im_tmp, AS_IMAGE_KIND_THUMBNAIL);
+	as_image_set_basename (im_tmp, as_image_get_basename (im_src));
+	as_screenshot_add_image (ss, im_tmp);
+	return TRUE;
+}
+
 /**
  * asb_app_add_screenshot_source:
  * @app: A #AsbApp
@@ -354,6 +394,7 @@ asb_app_save_resources (AsbApp *app, GError **error)
 gboolean
 asb_app_add_screenshot_source (AsbApp *app, const gchar *filename, GError **error)
 {
+	AsbAppPrivate *priv = GET_PRIVATE (app);
 	AsImageAlphaFlags alpha_flags;
 	gboolean is_default;
 	const gchar *mirror_uri;
@@ -361,7 +402,7 @@ asb_app_add_screenshot_source (AsbApp *app, const gchar *filename, GError **erro
 	_cleanup_free_ gchar *basename = NULL;
 	_cleanup_free_ gchar *filename_no_path = NULL;
 	_cleanup_free_ gchar *url_src = NULL;
-	_cleanup_object_unref_ AsImage *im_src;
+	_cleanup_object_unref_ AsImage *im_src = NULL;
 	_cleanup_object_unref_ AsScreenshot *ss = NULL;
 	guint sizes[] = { AS_IMAGE_NORMAL_WIDTH,    AS_IMAGE_NORMAL_HEIGHT,
 			  AS_IMAGE_THUMBNAIL_WIDTH, AS_IMAGE_THUMBNAIL_HEIGHT,
@@ -382,6 +423,20 @@ asb_app_add_screenshot_source (AsbApp *app, const gchar *filename, GError **erro
 				 filename_no_path);
 	}
 
+	/* check screenshot is reasonable in size */
+	if (as_image_get_width (im_src) * 2 < AS_IMAGE_NORMAL_WIDTH ||
+	    as_image_get_height (im_src) * 2 < AS_IMAGE_NORMAL_HEIGHT) {
+		filename_no_path = g_path_get_basename (filename);
+		g_set_error (error,
+			     AS_APP_ERROR,
+			     AS_APP_ERROR_FAILED,
+			     "%s is too small to be used: %ix%i",
+			     filename_no_path,
+			     as_image_get_width (im_src),
+			     as_image_get_height (im_src));
+		return FALSE;
+	}
+
 	/* check the image is not padded */
 	alpha_flags = as_image_get_alpha_flags (im_src);
 	if ((alpha_flags & AS_IMAGE_ALPHA_FLAG_TOP) > 0||
@@ -420,30 +475,22 @@ asb_app_add_screenshot_source (AsbApp *app, const gchar *filename, GError **erro
 	as_screenshot_add_image (ss, im_src);
 	if (as_app_get_id_kind (AS_APP (app)) != AS_ID_KIND_FONT) {
 		for (i = 0; sizes[i] != 0; i += 2) {
-			_cleanup_free_ gchar *size_str;
-			_cleanup_free_ gchar *url_tmp;
-			_cleanup_object_unref_ AsImage *im_tmp;
-			_cleanup_object_unref_ GdkPixbuf *pixbuf;
 
-			size_str = g_strdup_printf ("%ix%i",
-						    sizes[i],
-						    sizes[i+1]);
-			url_tmp = g_build_filename (mirror_uri,
-						    size_str,
-						    basename,
-						    NULL);
-			pixbuf = as_image_save_pixbuf (im_src,
-						       sizes[i],
-						       sizes[i+1],
-						       AS_IMAGE_SAVE_FLAG_PAD_16_9);
-			im_tmp = as_image_new ();
-			as_image_set_width (im_tmp, sizes[i]);
-			as_image_set_height (im_tmp, sizes[i+1]);
-			as_image_set_url (im_tmp, url_tmp, -1);
-			as_image_set_pixbuf (im_tmp, pixbuf);
-			as_image_set_kind (im_tmp, AS_IMAGE_KIND_THUMBNAIL);
-			as_image_set_basename (im_tmp, basename);
-			as_screenshot_add_image (ss, im_tmp);
+			/* save LoDPI */
+			if (!asb_app_save_thumbnail (ss, im_src,
+						     sizes[i], sizes[i+1],
+						     1, mirror_uri, error))
+				return FALSE;
+
+			/* save HiDPI version */
+			if (priv->hidpi_enabled) {
+				if (!asb_app_save_thumbnail (ss, im_src,
+							     sizes[i],
+							     sizes[i+1],
+							     2, mirror_uri,
+							     error))
+					return FALSE;
+			}
 		}
 	}
 	as_app_add_screenshot (AS_APP (app), ss);
