@@ -45,6 +45,9 @@
 #ifdef HAVE_RPM
 #include "asb-package-rpm.h"
 #endif
+#ifdef HAVE_ALPM
+#include "asb-package-alpm.h"
+#endif
 
 #include "asb-package-deb.h"
 
@@ -61,8 +64,11 @@ struct _AsbContextPrivate
 	AsbPanel		*panel;
 	AsbPluginLoader		*plugin_loader;
 	gboolean		 add_cache_id;
+	gboolean		 hidpi_enabled;
+	gboolean		 embedded_icons;
 	gboolean		 no_net;
 	guint			 max_threads;
+	guint			 min_icon_size;
 	gdouble			 api_version;
 	gchar			*old_metadata;
 	gchar			*extra_appstream;
@@ -75,6 +81,7 @@ struct _AsbContextPrivate
 	gchar			*temp_dir;
 	gchar			*output_dir;
 	gchar			*basename;
+	gchar			*origin;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsbContext, asb_context, G_TYPE_OBJECT)
@@ -156,6 +163,38 @@ asb_context_set_add_cache_id (AsbContext *ctx, gboolean add_cache_id)
 	priv->add_cache_id = add_cache_id;
 }
 
+/**
+ * asb_context_set_hidpi_enabled:
+ * @ctx: A #AsbContext
+ * @hidpi_enabled: boolean
+ *
+ * Sets if the high DPI icons should be included in the metadata.
+ *
+ * Since: 0.3.1
+ **/
+void
+asb_context_set_hidpi_enabled (AsbContext *ctx, gboolean hidpi_enabled)
+{
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	priv->hidpi_enabled = hidpi_enabled;
+}
+
+/**
+ * asb_context_set_embedded_icons:
+ * @ctx: A #AsbContext
+ * @embedded_icons: boolean
+ *
+ * Sets if the icons should be Base64 embedded in the metadata.
+ *
+ * Since: 0.3.1
+ **/
+void
+asb_context_set_embedded_icons (AsbContext *ctx, gboolean embedded_icons)
+{
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	priv->embedded_icons = embedded_icons;
+}
+
 /**
  * asb_context_set_max_threads:
  * @ctx: A #AsbContext
@@ -172,6 +211,39 @@ asb_context_set_max_threads (AsbContext *ctx, guint max_threads)
 	priv->max_threads = max_threads;
 }
 
+/**
+ * asb_context_set_min_icon_size:
+ * @ctx: A #AsbContext
+ * @min_icon_size: integer
+ *
+ * Sets the smallest icon size in pixels supported.
+ *
+ * Since: 0.3.1
+ **/
+void
+asb_context_set_min_icon_size (AsbContext *ctx, guint min_icon_size)
+{
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	priv->min_icon_size = min_icon_size;
+}
+
+/**
+ * asb_context_get_min_icon_size:
+ * @ctx: A #AsbContext
+ *
+ * Gets the minimum icon size in pixels.
+ *
+ * Returns: size
+ *
+ * Since: 0.3.1
+ **/
+guint
+asb_context_get_min_icon_size (AsbContext *ctx)
+{
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	return priv->min_icon_size;
+}
+
 /**
  * asb_context_set_old_metadata:
  * @ctx: A #AsbContext
@@ -285,7 +357,7 @@ void
 asb_context_set_screenshot_dir (AsbContext *ctx, const gchar *screenshot_dir)
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
-	priv->screenshot_dir = asb_context_realpath (screenshot_dir);
+	priv->screenshot_dir = g_strdup (screenshot_dir);
 }
 
 /**
@@ -339,7 +411,7 @@ asb_context_set_output_dir (AsbContext *ctx, const gchar *output_dir)
 /**
  * asb_context_set_basename:
  * @ctx: A #AsbContext
- * @basename: AppStream basename, e.g. "fedora-21"
+ * @basename: AppStream file basename, e.g. "appstream"
  *
  * Sets the basename for the two metadata files.
  *
@@ -352,6 +424,22 @@ asb_context_set_basename (AsbContext *ctx, const gchar *basename)
 	priv->basename = g_strdup (basename);
 }
 
+/**
+ * asb_context_set_origin:
+ * @ctx: A #AsbContext
+ * @origin: AppStream origin, e.g. "fedora-21"
+ *
+ * Sets the origin for the two metadata files.
+ *
+ * Since: 0.3.4
+ **/
+void
+asb_context_set_origin (AsbContext *ctx, const gchar *origin)
+{
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	priv->origin = g_strdup (origin);
+}
+
 /**
  * asb_context_get_add_cache_id:
  * @ctx: A #AsbContext
@@ -369,6 +457,40 @@ asb_context_get_add_cache_id (AsbContext *ctx)
 	return priv->add_cache_id;
 }
 
+/**
+ * asb_context_get_hidpi_enabled:
+ * @ctx: A #AsbContext
+ *
+ * Gets if the HiDPI support should be used.
+ *
+ * Returns: boolean
+ *
+ * Since: 0.3.1
+ **/
+gboolean
+asb_context_get_hidpi_enabled (AsbContext *ctx)
+{
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	return priv->hidpi_enabled;
+}
+
+/**
+ * asb_context_get_embedded_icons:
+ * @ctx: A #AsbContext
+ *
+ * Gets if the icons should be embedded in the XML.
+ *
+ * Returns: boolean
+ *
+ * Since: 0.3.1
+ **/
+gboolean
+asb_context_get_embedded_icons (AsbContext *ctx)
+{
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	return priv->embedded_icons;
+}
+
 /**
  * asb_context_get_no_net:
  * @ctx: A #AsbContext
@@ -482,6 +604,10 @@ asb_context_add_filename (AsbContext *ctx, const gchar *filename, GError **error
 #if HAVE_RPM
 	if (g_str_has_suffix (filename, ".rpm"))
 		pkg = asb_package_rpm_new ();
+#endif
+#if HAVE_ALPM
+	if (g_str_has_suffix (filename, ".pkg.tar.xz"))
+		pkg = asb_package_alpm_new ();
 #endif
 	if (g_str_has_suffix (filename, ".deb"))
 		pkg = asb_package_deb_new ();
@@ -548,6 +674,7 @@ asb_context_load_extra_screenshots (AsbContext *ctx, AsApp *app, GError **error)
 
 	/* create a new AsbApp and add all the extra screenshots */
 	app_build = asb_app_new (pkg, as_app_get_id (app));
+	asb_app_set_hidpi_enabled (app_build, priv->hidpi_enabled);
 	as_app_subsume_full (AS_APP (app_build), app,
 			     AS_APP_SUBSUME_FLAG_NO_OVERWRITE);
 	dir = g_dir_open (path, 0, error);
@@ -563,7 +690,7 @@ asb_context_load_extra_screenshots (AsbContext *ctx, AsApp *app, GError **error)
 			     AS_APP_SUBSUME_FLAG_NO_OVERWRITE);
 
 	/* save all the screenshots to disk */
-	return asb_app_save_resources (app_build, error);
+	return asb_app_save_resources (app_build, ASB_APP_SAVE_FLAG_SCREENSHOTS, error);
 }
 
 /**
@@ -584,19 +711,18 @@ asb_context_setup (AsbContext *ctx, GError **error)
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	GList *l;
 	guint i;
-	const gchar *ss_sizes[] = { "112x63",
-				    "624x351",
-				    "752x423",
-				    "source",
-				    NULL };
+	guint sizes[] = { AS_IMAGE_NORMAL_WIDTH,    AS_IMAGE_NORMAL_HEIGHT,
+			  AS_IMAGE_THUMBNAIL_WIDTH, AS_IMAGE_THUMBNAIL_HEIGHT,
+			  AS_IMAGE_LARGE_WIDTH,     AS_IMAGE_LARGE_HEIGHT,
+			  0 };
 	_cleanup_free_ gchar *icons_dir = NULL;
 
 	/* required stuff set */
-	if (priv->basename == NULL) {
+	if (priv->origin == NULL) {
 		g_set_error_literal (error,
 				     ASB_PLUGIN_ERROR,
 				     ASB_PLUGIN_ERROR_FAILED,
-				     "basename not set!");
+				     "origin not set!");
 		return FALSE;
 	}
 	if (priv->output_dir == NULL) {
@@ -628,18 +754,52 @@ asb_context_setup (AsbContext *ctx, GError **error)
 		return FALSE;
 	if (!asb_utils_ensure_exists (priv->cache_dir, error))
 		return FALSE;
+	if (priv->log_dir != NULL) {
+		if (!asb_utils_ensure_exists (priv->log_dir, error))
+			return FALSE;
+	}
 
 	/* icons is nuked; we can re-decompress from the -icons.tar.gz */
 	icons_dir = g_build_filename (priv->temp_dir, "icons", NULL);
 	if (!asb_utils_ensure_exists (icons_dir, error))
 		return FALSE;
+	if (priv->hidpi_enabled) {
+		_cleanup_free_ gchar *icons_dir_hidpi = NULL;
+		_cleanup_free_ gchar *icons_dir_lodpi = NULL;
+		icons_dir_lodpi = g_build_filename (icons_dir, "64x64", NULL);
+		if (!asb_utils_ensure_exists (icons_dir_lodpi, error))
+			return FALSE;
+		icons_dir_hidpi = g_build_filename (icons_dir, "128x128", NULL);
+		if (!asb_utils_ensure_exists (icons_dir_hidpi, error))
+			return FALSE;
+	}
 
 	/* create all the screenshot sizes */
 	if (priv->screenshot_dir != NULL) {
-		for (i = 0; ss_sizes[i] != NULL; i++) {
+		_cleanup_free_ gchar *ss_src = NULL;
+		ss_src = g_build_filename (priv->screenshot_dir,
+					   "source", NULL);
+		if (!asb_utils_ensure_exists (ss_src, error))
+			return FALSE;
+		for (i = 0; sizes[i] != 0; i += 2) {
+			_cleanup_free_ gchar *size_str = NULL;
 			_cleanup_free_ gchar *ss_dir = NULL;
+			size_str = g_strdup_printf ("%ix%i",
+						    sizes[i],
+						    sizes[i+1]);
 			ss_dir = g_build_filename (priv->screenshot_dir,
-						   ss_sizes[i], NULL);
+						   size_str, NULL);
+			if (!asb_utils_ensure_exists (ss_dir, error))
+				return FALSE;
+		}
+		for (i = 0; sizes[i] != 0 && priv->hidpi_enabled; i += 2) {
+			_cleanup_free_ gchar *size_str = NULL;
+			_cleanup_free_ gchar *ss_dir = NULL;
+			size_str = g_strdup_printf ("%ix%i",
+						    sizes[i] * 2,
+						    sizes[i+1] * 2);
+			ss_dir = g_build_filename (priv->screenshot_dir,
+						   size_str, NULL);
 			if (!asb_utils_ensure_exists (ss_dir, error))
 				return FALSE;
 		}
@@ -769,17 +929,17 @@ asb_task_process_func (gpointer data, gpointer user_data)
 static gboolean
 asb_context_write_icons (AsbContext *ctx,
 			 const gchar *temp_dir,
-			 const gchar *output_dir,
-			 const gchar *basename,
 			 GError **error)
 {
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	_cleanup_free_ gchar *filename = NULL;
 	_cleanup_free_ gchar *icons_dir = NULL;
 
 	icons_dir = g_build_filename (temp_dir, "icons", NULL);
 	if (!g_file_test (icons_dir, G_FILE_TEST_EXISTS))
 		return TRUE;
-	filename = g_strdup_printf ("%s/%s-icons.tar.gz", output_dir, basename);
+	filename = g_strdup_printf ("%s/%s-icons.tar.gz",
+				    priv->output_dir, priv->basename);
 	g_print ("Writing %s...\n", filename);
 	return asb_utils_write_archive_dir (filename, icons_dir, error);
 }
@@ -788,17 +948,14 @@ asb_context_write_icons (AsbContext *ctx,
  * asb_context_write_xml:
  **/
 static gboolean
-asb_context_write_xml (AsbContext *ctx,
-		       const gchar *output_dir,
-		       const gchar *basename,
-		       GError **error)
+asb_context_write_xml (AsbContext *ctx, GError **error)
 {
 	AsApp *app;
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	GList *l;
 	_cleanup_free_ gchar *filename = NULL;
-	_cleanup_object_unref_ AsStore *store;
-	_cleanup_object_unref_ GFile *file;
+	_cleanup_object_unref_ AsStore *store = NULL;
+	_cleanup_object_unref_ GFile *file = NULL;
 
 	/* convert any vetod applications into dummy components */
 	for (l = priv->apps; l != NULL; l = l->next) {
@@ -830,11 +987,13 @@ asb_context_write_xml (AsbContext *ctx,
 		}
 
 	}
-	filename = g_strdup_printf ("%s/%s.xml.gz", output_dir, basename);
+	filename = g_strdup_printf ("%s/%s.xml.gz",
+				    priv->output_dir,
+				    priv->basename);
 	file = g_file_new_for_path (filename);
 
 	g_print ("Writing %s...\n", filename);
-	as_store_set_origin (store, basename);
+	as_store_set_origin (store, priv->origin);
 	as_store_set_api_version (store, priv->api_version);
 	if (priv->add_cache_id) {
 		_cleanup_free_ gchar *builder_id = asb_utils_get_builder_id ();
@@ -848,6 +1007,57 @@ asb_context_write_xml (AsbContext *ctx,
 				 NULL, error);
 }
 
+/**
+ * asb_context_convert_icons:
+ **/
+static gboolean
+asb_context_convert_icons (AsbContext *ctx, GError **error)
+{
+	AsApp *app;
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	GList *l;
+
+	/* not enabled */
+	if (!asb_context_get_embedded_icons (ctx))
+		return TRUE;
+
+	/* convert each one before saving resources */
+	for (l = priv->apps; l != NULL; l = l->next) {
+		app = AS_APP (l->data);
+		if (as_app_get_vetos(app)->len > 0)
+			continue;
+		if (!as_app_convert_icons (app, AS_ICON_KIND_EMBEDDED, error))
+			return FALSE;
+	}
+	return TRUE;
+}
+
+/**
+ * asb_context_save_resources:
+ **/
+static gboolean
+asb_context_save_resources (AsbContext *ctx, GError **error)
+{
+	AsApp *app;
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	GList *l;
+
+	for (l = priv->apps; l != NULL; l = l->next) {
+		app = AS_APP (l->data);
+
+		/* save icon */
+		if (as_app_get_vetos(app)->len > 0)
+			continue;
+		if (!ASB_IS_APP (app))
+			continue;
+		if (!asb_app_save_resources (ASB_APP (app),
+					     ASB_APP_SAVE_FLAG_ICONS,
+					     error))
+			return FALSE;
+	}
+	return TRUE;
+}
+
 /**
  * asb_context_detect_pkgname_dups:
  **/
@@ -881,6 +1091,63 @@ asb_context_detect_pkgname_dups (AsbContext *ctx, GError **error)
 	return TRUE;
 }
 
+/**
+ * asb_context_write_app_xml:
+ **/
+static void
+asb_context_write_app_xml (AsbContext *ctx)
+{
+	AsbApp *app;
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	GList *l;
+
+	/* log the XML in the log file */
+	for (l = priv->apps; l != NULL; l = l->next) {
+		_cleanup_string_free_ GString *xml = NULL;
+		_cleanup_object_unref_ AsStore *store = NULL;
+
+		/* we have an open log file? */
+		if (!ASB_IS_APP (l->data))
+			continue;
+
+		/* just log raw XML */
+		app = ASB_APP (l->data);
+		store = as_store_new ();
+		as_store_set_api_version (store, 1.0f);
+		as_store_add_app (store, AS_APP (app));
+		xml = as_store_to_xml (store,
+				       AS_NODE_TO_XML_FLAG_FORMAT_INDENT |
+				       AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
+		asb_package_log (asb_app_get_package (app),
+				 ASB_PACKAGE_LOG_LEVEL_NONE, "%s", xml->str);
+	}
+}
+
+/**
+ * asb_context_detect_missing_data:
+ **/
+static gboolean
+asb_context_detect_missing_data (AsbContext *ctx, GError **error)
+{
+	AsApp *app;
+	AsbContextPrivate *priv = GET_PRIVATE (ctx);
+	GList *l;
+
+	/* look for the thing that an addon extends */
+	for (l = priv->apps; l != NULL; l = l->next) {
+		app = AS_APP (l->data);
+		if (as_app_get_name (AS_APP (app), "C") == NULL)
+			as_app_add_veto (AS_APP (app), "No 'Name' in desktop or <name> in AppData");
+		if (as_app_get_comment (AS_APP (app), "C") == NULL)
+			as_app_add_veto (AS_APP (app), "No 'Comment' in desktop or <summary> in AppData");
+		if (as_app_get_id_kind (AS_APP (app)) != AS_ID_KIND_ADDON) {
+			if (as_app_get_icon_default (AS_APP (app)) == NULL)
+				as_app_add_veto (AS_APP (app), "Has no Icon");
+		}
+	}
+	return TRUE;
+}
+
 /**
  * asb_context_detect_missing_parents:
  **/
@@ -922,6 +1189,9 @@ asb_context_detect_missing_parents (AsbContext *ctx, GError **error)
 			continue;
 		tmp = g_ptr_array_index (as_app_get_extends(app), 0);
 		found = g_hash_table_lookup (hash, tmp);
+		if (found != NULL)
+			continue;
+		found = as_store_get_app_by_id (priv->store_old, tmp);
 		if (found != NULL)
 			continue;
 
@@ -938,17 +1208,14 @@ asb_context_detect_missing_parents (AsbContext *ctx, GError **error)
  * asb_context_write_xml_fail:
  **/
 static gboolean
-asb_context_write_xml_fail (AsbContext *ctx,
-			     const gchar *output_dir,
-			     const gchar *basename,
-			     GError **error)
+asb_context_write_xml_fail (AsbContext *ctx, GError **error)
 {
 	AsApp *app;
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	GList *l;
 	_cleanup_free_ gchar *basename_failed = NULL;
 	_cleanup_free_ gchar *filename = NULL;
-	_cleanup_object_unref_ GFile *file;
+	_cleanup_object_unref_ GFile *file = NULL;
 
 	for (l = priv->apps; l != NULL; l = l->next) {
 		app = AS_APP (l->data);
@@ -963,11 +1230,12 @@ asb_context_write_xml_fail (AsbContext *ctx,
 			continue;
 		as_store_add_app (priv->store_failed, app);
 	}
-	filename = g_strdup_printf ("%s/%s-failed.xml.gz", output_dir, basename);
+	filename = g_strdup_printf ("%s/%s-failed.xml.gz",
+				    priv->output_dir, priv->basename);
 	file = g_file_new_for_path (filename);
 
 	g_print ("Writing %s...\n", filename);
-	basename_failed = g_strdup_printf ("%s-failed", basename);
+	basename_failed = g_strdup_printf ("%s-failed", priv->origin);
 	as_store_set_origin (priv->store_failed, basename_failed);
 	as_store_set_api_version (priv->store_failed, priv->api_version);
 	if (priv->add_cache_id) {
@@ -986,10 +1254,7 @@ asb_context_write_xml_fail (AsbContext *ctx,
  * asb_context_write_xml_ignore:
  **/
 static gboolean
-asb_context_write_xml_ignore (AsbContext *ctx,
-			      const gchar *output_dir,
-			      const gchar *basename,
-			      GError **error)
+asb_context_write_xml_ignore (AsbContext *ctx, GError **error)
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	_cleanup_free_ gchar *basename_cache = NULL;
@@ -1001,11 +1266,12 @@ asb_context_write_xml_ignore (AsbContext *ctx,
 		return TRUE;
 
 	/* the store is already populated */
-	filename = g_strdup_printf ("%s/%s-ignore.xml.gz", output_dir, basename);
+	filename = g_strdup_printf ("%s/%s-ignore.xml.gz",
+				    priv->output_dir, priv->basename);
 	file = g_file_new_for_path (filename);
 
 	g_print ("Writing %s...\n", filename);
-	basename_cache = g_strdup_printf ("%s-ignore", basename);
+	basename_cache = g_strdup_printf ("%s-ignore", priv->origin);
 	as_store_set_origin (priv->store_ignore, basename_cache);
 	as_store_set_api_version (priv->store_ignore, priv->api_version);
 	if (priv->add_cache_id) {
@@ -1031,7 +1297,7 @@ asb_context_disable_older_pkgs (AsbContext *ctx)
 	AsbPackage *pkg;
 	const gchar *key;
 	guint i;
-	_cleanup_hashtable_unref_ GHashTable *newest;
+	_cleanup_hashtable_unref_ GHashTable *newest = NULL;
 
 	newest = g_hash_table_new_full (g_str_hash, g_str_equal,
 					g_free, (GDestroyNotify) g_object_unref);
@@ -1088,6 +1354,7 @@ asb_context_disable_multiarch_pkgs (AsbContext *ctx)
 /**
  * asb_context_process:
  * @ctx: A #AsbContext
+ * @flags: Some #AsbContextProcessFlags, e.g. %AS_CONTEXT_PARSE_FLAG_IGNORE_MISSING_PARENTS
  * @error: A #GError or %NULL
  *
  * Processes all the packages that have been added to the context.
@@ -1097,13 +1364,13 @@ asb_context_disable_multiarch_pkgs (AsbContext *ctx)
  * Since: 0.1.0
  **/
 gboolean
-asb_context_process (AsbContext *ctx, GError **error)
+asb_context_process (AsbContext *ctx, AsbContextProcessFlags flags, GError **error)
 {
 	AsbContextPrivate *priv = GET_PRIVATE (ctx);
 	AsbPackage *pkg;
 	AsbTask *task;
 	GThreadPool *pool;
-	gboolean ret = FALSE;
+	gboolean ret;
 	guint i;
 	_cleanup_ptrarray_unref_ GPtrArray *tasks = NULL;
 
@@ -1118,7 +1385,7 @@ asb_context_process (AsbContext *ctx, GError **error)
 				  TRUE,
 				  error);
 	if (pool == NULL)
-		goto out;
+		return FALSE;
 
 	/* add each package */
 	g_print ("Processing packages...\n");
@@ -1154,9 +1421,8 @@ asb_context_process (AsbContext *ctx, GError **error)
 		g_ptr_array_add (tasks, task);
 
 		/* add task to pool */
-		ret = g_thread_pool_push (pool, task, error);
-		if (!ret)
-			goto out;
+		if (!g_thread_pool_push (pool, task, error))
+			return FALSE;
 	}
 
 	/* wait for them to finish */
@@ -1167,40 +1433,53 @@ asb_context_process (AsbContext *ctx, GError **error)
 	asb_plugin_loader_merge (priv->plugin_loader, priv->apps);
 
 	/* print any warnings */
-	ret = asb_context_detect_missing_parents (ctx, error);
-	if (!ret)
-		goto out;
+	if ((flags & AS_CONTEXT_PARSE_FLAG_IGNORE_MISSING_INFO) == 0) {
+		if (!asb_context_detect_missing_data (ctx, error))
+			return FALSE;
+	}
+	if ((flags & AS_CONTEXT_PARSE_FLAG_IGNORE_MISSING_PARENTS) == 0) {
+		if (!asb_context_detect_missing_parents (ctx, error))
+			return FALSE;
+	}
+	if (!asb_context_detect_pkgname_dups (ctx, error))
+		return FALSE;
+	if (!asb_context_convert_icons (ctx, error))
+		return FALSE;
+	if (!asb_context_save_resources (ctx, error))
+		return FALSE;
+
+	/* write the application XML to the log file */
+	asb_context_write_app_xml (ctx);
 
 	/* write XML file */
-	ret = asb_context_write_xml (ctx, priv->output_dir, priv->basename, error);
+	ret = asb_context_write_xml (ctx, error);
 	if (!ret)
-		goto out;
+		return FALSE;
 
 	/* write XML file */
-	ret = asb_context_write_xml_fail (ctx, priv->output_dir, priv->basename, error);
+	ret = asb_context_write_xml_fail (ctx, error);
 	if (!ret)
-		goto out;
+		return FALSE;
 
 	/* write XML file */
-	ret = asb_context_write_xml_ignore (ctx, priv->output_dir, priv->basename, error);
+	ret = asb_context_write_xml_ignore (ctx, error);
 	if (!ret)
-		goto out;
+		return FALSE;
 
 	/* write icons archive */
 	ret = asb_context_write_icons (ctx,
 				       priv->temp_dir,
-				       priv->output_dir,
-				       priv->basename,
 				       error);
 	if (!ret)
-		goto out;
+		return FALSE;
 
-	/* print any warnings */
-	ret = asb_context_detect_pkgname_dups (ctx, error);
-	if (!ret)
-		goto out;
-out:
-	return ret;
+	/* ensure all packages are flushed */
+	for (i = 0; i < priv->packages->len; i++) {
+		pkg = g_ptr_array_index (priv->packages, i);
+		if (!asb_package_log_flush (pkg, error))
+			return FALSE;
+	}
+	return TRUE;
 }
 
 /**
@@ -1369,6 +1648,7 @@ asb_context_finalize (GObject *object)
 	g_free (priv->temp_dir);
 	g_free (priv->output_dir);
 	g_free (priv->basename);
+	g_free (priv->origin);
 
 	G_OBJECT_CLASS (asb_context_parent_class)->finalize (object);
 }
@@ -1389,6 +1669,7 @@ asb_context_init (AsbContext *ctx)
 	priv->store_ignore = as_store_new ();
 	priv->store_old = as_store_new ();
 	priv->max_threads = 1;
+	priv->min_icon_size = 32;
 }
 
 /**
