@@ -284,6 +284,23 @@ asb_package_rpm_ensure_url (AsbPackage *pkg, GError **error)
 	return TRUE;
 }
 
+/**
+ * asb_package_rpm_ensure_vcs:
+ **/
+static gboolean
+asb_package_rpm_ensure_vcs (AsbPackage *pkg, GError **error)
+{
+	AsbPackageRpm *pkg_rpm = ASB_PACKAGE_RPM (pkg);
+	AsbPackageRpmPrivate *priv = GET_PRIVATE (pkg_rpm);
+	rpmtd td;
+
+	td = rpmtdNew ();
+	headerGet (priv->h, RPMTAG_VCS, td, HEADERGET_MINMEM);
+	asb_package_set_vcs (pkg, rpmtdGetString (td));
+	rpmtdFree (td);
+	return TRUE;
+}
+
 /**
  * asb_package_rpm_ensure_license:
  **/
@@ -312,9 +329,9 @@ asb_package_rpm_add_release (AsbPackage *pkg,
 {
 	AsRelease *release;
 	const gchar *version;
-	_cleanup_free_ gchar *name_dup;
 	gchar *tmp;
 	gchar *vr;
+	_cleanup_free_ gchar *name_dup = NULL;
 
 	/* get last string chunk */
 	name_dup = g_strchomp (g_strdup (name));
@@ -615,6 +632,10 @@ asb_package_rpm_ensure (AsbPackage *pkg,
 		if (!asb_package_rpm_ensure_source (pkg, error))
 			return FALSE;
 	}
+	if ((flags & ASB_PACKAGE_ENSURE_VCS) > 0) {
+		if (!asb_package_rpm_ensure_vcs (pkg, error))
+			return FALSE;
+	}
 	return TRUE;
 }
 
