@@ -53,14 +53,17 @@ struct _AsbPackagePrivate
 	gchar		*nevra;
 	gchar		*evr;
 	gchar		*license;
+	gchar		*vcs;
 	gchar		*source_nevra;
 	gchar		*source_pkgname;
+	gsize		 log_written_len;
 	GString		*log;
 	GHashTable	*configs;
 	GTimer		*timer;
 	gdouble		 last_log;
 	GPtrArray	*releases;
 	GHashTable	*releases_hash;
+	GMutex		 mutex_log;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsbPackage, asb_package, G_TYPE_OBJECT)
@@ -76,6 +79,7 @@ asb_package_finalize (GObject *object)
 	AsbPackage *pkg = ASB_PACKAGE (object);
 	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
 
+	g_mutex_clear (&priv->mutex_log);
 	g_strfreev (priv->filelist);
 	g_strfreev (priv->deps);
 	g_free (priv->filename);
@@ -89,6 +93,7 @@ asb_package_finalize (GObject *object)
 	g_free (priv->nevra);
 	g_free (priv->evr);
 	g_free (priv->license);
+	g_free (priv->vcs);
 	g_free (priv->source_nevra);
 	g_free (priv->source_pkgname);
 	g_string_free (priv->log, TRUE);
@@ -115,6 +120,7 @@ asb_package_init (AsbPackage *pkg)
 	priv->releases = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	priv->releases_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
 						     g_free, (GDestroyNotify) g_object_unref);
+	g_mutex_init (&priv->mutex_log);
 }
 
 /**
@@ -184,7 +190,9 @@ asb_package_log (AsbPackage *pkg,
 	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
 	va_list args;
 	gdouble now;
-	_cleanup_free_ gchar *tmp;
+	_cleanup_free_ gchar *tmp = NULL;
+
+	g_mutex_lock (&priv->mutex_log);
 
 	va_start (args, fmt);
 	tmp = g_strdup_vprintf (fmt, args);
@@ -215,6 +223,7 @@ asb_package_log (AsbPackage *pkg,
 		g_string_append_printf (priv->log, "%s\n", tmp);
 		break;
 	}
+	g_mutex_unlock (&priv->mutex_log);
 }
 
 /**
@@ -233,15 +242,24 @@ asb_package_log_flush (AsbPackage *pkg, GError **error)
 {
 	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
 	_cleanup_free_ gchar *logfile = NULL;
+	_cleanup_free_ gchar *logdir_char = NULL;
+
+	/* needs no update */
+	if (priv->log_written_len == priv->log->len)
+		return TRUE;
 
 	/* don't write if unset */
 	if (asb_package_get_config (pkg, "LogDir") == NULL)
 		return TRUE;
 
 	/* overwrite old log */
-	logfile = g_strdup_printf ("%s/%s.log",
-				   asb_package_get_config (pkg, "LogDir"),
-				   asb_package_get_name (pkg));
+	logdir_char = g_strdup_printf ("%s/%c",
+				       asb_package_get_config (pkg, "LogDir"),
+				       g_ascii_tolower (priv->name[0]));
+	if (!asb_utils_ensure_exists (logdir_char, error))
+		return FALSE;
+	priv->log_written_len = priv->log->len;
+	logfile = g_strdup_printf ("%s/%s.log", logdir_char, priv->name);
 	return g_file_set_contents (logfile, priv->log->str, -1, error);
 }
 
@@ -347,6 +365,23 @@ asb_package_get_license (AsbPackage *pkg)
 	return priv->license;
 }
 
+/**
+ * asb_package_get_vcs:
+ * @pkg: A #AsbPackage
+ *
+ * Gets the package version control system.
+ *
+ * Returns: utf8 string
+ *
+ * Since: 0.3.4
+ **/
+const gchar *
+asb_package_get_vcs (AsbPackage *pkg)
+{
+	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
+	return priv->vcs;
+}
+
 /**
  * asb_package_get_source:
  * @pkg: A #AsbPackage
@@ -533,6 +568,23 @@ asb_package_set_license (AsbPackage *pkg, const gchar *license)
 	priv->license = g_strdup (license);
 }
 
+/**
+ * asb_package_set_vcs:
+ * @pkg: A #AsbPackage
+ * @vcs: vcs string
+ *
+ * Sets the package version control system.
+ *
+ * Since: 0.3.4
+ **/
+void
+asb_package_set_vcs (AsbPackage *pkg, const gchar *vcs)
+{
+	AsbPackagePrivate *priv = GET_PRIVATE (pkg);
+	g_free (priv->vcs);
+	priv->vcs = g_strdup (vcs);
+}
+
 /**
  * asb_package_set_source:
  * @pkg: A #AsbPackage
@@ -761,6 +813,8 @@ asb_package_ensure (AsbPackage *pkg,
 		flags &= ~ASB_PACKAGE_ENSURE_NEVRA;
 	if (priv->license != NULL)
 		flags &= ~ASB_PACKAGE_ENSURE_LICENSE;
+	if (priv->vcs != NULL)
+		flags &= ~ASB_PACKAGE_ENSURE_VCS;
 	if (priv->url != NULL)
 		flags &= ~ASB_PACKAGE_ENSURE_URL;
 	if (priv->source_pkgname != NULL)
