@@ -169,6 +169,7 @@ asb_plugin_loader_process_app (AsbPluginLoader *plugin_loader,
 	AsbPluginLoaderPrivate *priv = GET_PRIVATE (plugin_loader);
 	AsbPlugin *plugin;
 	AsbPluginProcessAppFunc plugin_func = NULL;
+	GError *error_local = NULL;
 	gboolean ret;
 	guint i;
 
@@ -184,8 +185,20 @@ asb_plugin_loader_process_app (AsbPluginLoader *plugin_loader,
 				 ASB_PACKAGE_LOG_LEVEL_DEBUG,
 				 "Running asb_plugin_process_app() from %s",
 				 plugin->name);
-		if (!plugin_func (plugin, pkg, app, tmpdir, error))
-			return FALSE;
+		if (!plugin_func (plugin, pkg, app, tmpdir, &error_local)) {
+			if (g_error_matches (error_local,
+					     ASB_PLUGIN_ERROR,
+					     ASB_PLUGIN_ERROR_IGNORE)) {
+				asb_package_log (pkg,
+						 ASB_PACKAGE_LOG_LEVEL_WARNING,
+						 "Ignoring: %s",
+						 error_local->message);
+				g_clear_error (&error_local);
+			} else {
+				g_propagate_error (error, error_local);
+				return FALSE;
+			}
+		}
 	}
 	return TRUE;
 }
@@ -263,7 +276,7 @@ asb_plugin_loader_merge (AsbPluginLoader *plugin_loader, GList *apps)
 	const gchar *tmp;
 	gboolean ret;
 	guint i;
-	_cleanup_hashtable_unref_ GHashTable *hash;
+	_cleanup_hashtable_unref_ GHashTable *hash = NULL;
 
 	/* run each plugin */
 	for (i = 0; i < priv->plugins->len; i++) {
@@ -296,6 +309,8 @@ asb_plugin_loader_merge (AsbPluginLoader *plugin_loader, GList *apps)
 		if (!ASB_IS_APP (l->data))
 			continue;
 		app = ASB_APP (l->data);
+		if (as_app_get_vetos(AS_APP(app))->len > 0)
+			continue;
 		key = as_app_get_id (AS_APP (app));
 		found = g_hash_table_lookup (hash, key);
 		if (found == NULL) {
@@ -384,7 +399,7 @@ asb_plugin_loader_setup (AsbPluginLoader *plugin_loader, GError **error)
 	AsbPluginLoaderPrivate *priv = GET_PRIVATE (plugin_loader);
 	const gchar *filename_tmp;
 	const gchar *location = "./plugins/.libs/";
-	_cleanup_dir_close_ GDir *dir;
+	_cleanup_dir_close_ GDir *dir = NULL;
 
 	/* search system-wide if not found locally */
 	if (!g_file_test (location, G_FILE_TEST_EXISTS))
