@@ -23,6 +23,7 @@
 
 #include <glib.h>
 #include <stdlib.h>
+#include <locale.h>
 
 #include "as-cleanup.h"
 
@@ -53,6 +54,32 @@ asb_test_get_filename (const gchar *filename)
 	return g_strdup (full_tmp);
 }
 
+/**
+ * asb_test_compare_lines:
+ **/
+static gboolean
+asb_test_compare_lines (const gchar *txt1, const gchar *txt2, GError **error)
+{
+	_cleanup_free_ gchar *output = NULL;
+
+	/* exactly the same */
+	if (g_strcmp0 (txt1, txt2) == 0)
+		return TRUE;
+
+	/* save temp files and diff them */
+	if (!g_file_set_contents ("/tmp/a", txt1, -1, error))
+		return FALSE;
+	if (!g_file_set_contents ("/tmp/b", txt2, -1, error))
+		return FALSE;
+	if (!g_spawn_command_line_sync ("diff -urNp /tmp/b /tmp/a",
+					&output, NULL, NULL, error))
+		return FALSE;
+
+	/* just output the diff */
+	g_set_error_literal (error, 1, 0, output);
+	return FALSE;
+}
+
 #ifdef HAVE_RPM
 static void
 asb_test_package_rpm_func (void)
@@ -195,7 +222,7 @@ asb_test_plugin_loader_func (void)
 	g_assert (ret);
 
 	/* get the list of globs */
-	globs = asb_plugin_loader_get_globs (loader);
+	globs = asb_plugin_loader_get_globs (loader);\
 	g_assert_cmpint (globs->len, ==, 22);
 	g_assert_cmpstr (asb_glob_value_search (globs, "/usr/share/applications/gimp.desktop"), ==, "");
 	g_assert_cmpstr (asb_glob_value_search (globs, "/srv/dave.txt"), ==, NULL);
@@ -232,11 +259,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	GError *error = NULL;
 	const gchar *expected_xml;
 	gboolean ret;
-	_cleanup_free_ gchar *filename1 = NULL;
-	_cleanup_free_ gchar *filename2 = NULL;
-	_cleanup_free_ gchar *filename3 = NULL;
-	_cleanup_free_ gchar *filename4 = NULL;
-	_cleanup_free_ gchar *filename5 = NULL;
+	guint i;
 	_cleanup_object_unref_ AsbContext *ctx = NULL;
 	_cleanup_object_unref_ AsStore *store_failed = NULL;
 	_cleanup_object_unref_ AsStore *store_ignore = NULL;
@@ -247,6 +270,16 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	_cleanup_string_free_ GString *xml = NULL;
 	_cleanup_string_free_ GString *xml_failed = NULL;
 	_cleanup_string_free_ GString *xml_ignore = NULL;
+	const gchar *filenames[] = {
+		"test-0.1-1.fc21.noarch.rpm",		/* a console app */
+		"app-1-1.fc21.x86_64.rpm",		/* a GUI app */
+		"app-extra-1-1.fc21.noarch.rpm",	/* addons for a GUI app */
+		"app-console-1-1.fc21.noarch.rpm",	/* app with no icon */
+		"app-1-1.fc21.i686.rpm",		/* GUI multiarch app */
+		"composite-1-1.fc21.x86_64.rpm",	/* multiple GUI apps */
+		"font-1-1.fc21.noarch.rpm",		/* font */
+		"font-serif-1-1.fc21.noarch.rpm",	/* font that extends */
+		NULL};
 
 	/* set up the context */
 	ctx = asb_context_new ();
@@ -255,7 +288,9 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	asb_context_set_api_version (ctx, 0.8);
 	asb_context_set_add_cache_id (ctx, TRUE);
 	asb_context_set_no_net (ctx, TRUE);
-	asb_context_set_basename (ctx, "asb-self-test");
+	asb_context_set_hidpi_enabled (ctx, TRUE);
+	asb_context_set_basename (ctx, "appstream");
+	asb_context_set_origin (ctx, "asb-self-test");
 	asb_context_set_cache_dir (ctx, "/tmp/asbuilder/cache");
 	asb_context_set_output_dir (ctx, "/tmp/asbuilder/output");
 	asb_context_set_temp_dir (ctx, "/tmp/asbuilder/temp");
@@ -279,46 +314,23 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	g_assert_no_error (error);
 	g_assert (ret);
 
-	/* add a console application */
-	filename1 = asb_test_get_filename ("test-0.1-1.fc21.noarch.rpm");
-	g_assert (filename1 != NULL);
-	ret = asb_context_add_filename (ctx, filename1, &error);
-	g_assert_no_error (error);
-	g_assert (ret);
-
-	/* add another GUI application */
-	filename2 = asb_test_get_filename ("app-1-1.fc21.x86_64.rpm");
-	g_assert (filename2 != NULL);
-	ret = asb_context_add_filename (ctx, filename2, &error);
-	g_assert_no_error (error);
-	g_assert (ret);
-
-	/* add addons for the GUI application */
-	filename3 = asb_test_get_filename ("app-extra-1-1.fc21.noarch.rpm");
-	g_assert (filename3 != NULL);
-	ret = asb_context_add_filename (ctx, filename3, &error);
-	g_assert_no_error (error);
-	g_assert (ret);
-
-	/* add application with no icon */
-	filename4 = asb_test_get_filename ("app-console-1-1.fc21.noarch.rpm");
-	g_assert (filename4 != NULL);
-	ret = asb_context_add_filename (ctx, filename4, &error);
-	g_assert_no_error (error);
-	g_assert (ret);
-
-	/* the same GUI application in the different architecture */
-	filename5 = asb_test_get_filename ("app-1-1.fc21.i686.rpm");
-	g_assert (filename5 != NULL);
-	ret = asb_context_add_filename (ctx, filename5, &error);
-	g_assert_no_error (error);
-	g_assert (ret);
+	/* add packages */
+	for (i = 0; filenames[i] != NULL; i++) {
+		_cleanup_free_ gchar *filename = NULL;
+		filename = asb_test_get_filename (filenames[i]);
+		if (filename == NULL)
+			g_warning ("%s not found", filenames[i]);
+		g_assert (filename != NULL);
+		ret = asb_context_add_filename (ctx, filename, &error);
+		g_assert_no_error (error);
+		g_assert (ret);
+	}
 
 	/* verify queue size */
 	switch (mode) {
 	case ASB_TEST_CONTEXT_MODE_NO_CACHE:
 	case ASB_TEST_CONTEXT_MODE_WITH_OLD_CACHE:
-		g_assert_cmpint (asb_context_get_packages(ctx)->len, ==, 5);
+		g_assert_cmpint (asb_context_get_packages(ctx)->len, ==, 8);
 		break;
 	default:
 		/* no packages should need extracting */
@@ -327,23 +339,23 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	}
 
 	/* run the plugins */
-	ret = asb_context_process (ctx, &error);
+	ret = asb_context_process (ctx, AS_CONTEXT_PARSE_FLAG_NONE, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
 
 	/* check files created */
-	g_assert (g_file_test ("/tmp/asbuilder/output/asb-self-test.xml.gz", G_FILE_TEST_EXISTS));
-	g_assert (g_file_test ("/tmp/asbuilder/output/asb-self-test-failed.xml.gz", G_FILE_TEST_EXISTS));
-	g_assert (g_file_test ("/tmp/asbuilder/output/asb-self-test-ignore.xml.gz", G_FILE_TEST_EXISTS));
-	g_assert (g_file_test ("/tmp/asbuilder/output/asb-self-test-icons.tar.gz", G_FILE_TEST_EXISTS));
+	g_assert (g_file_test ("/tmp/asbuilder/output/appstream.xml.gz", G_FILE_TEST_EXISTS));
+	g_assert (g_file_test ("/tmp/asbuilder/output/appstream-failed.xml.gz", G_FILE_TEST_EXISTS));
+	g_assert (g_file_test ("/tmp/asbuilder/output/appstream-ignore.xml.gz", G_FILE_TEST_EXISTS));
+	g_assert (g_file_test ("/tmp/asbuilder/output/appstream-icons.tar.gz", G_FILE_TEST_EXISTS));
 
 	/* load AppStream metadata */
-	file = g_file_new_for_path ("/tmp/asbuilder/output/asb-self-test.xml.gz");
+	file = g_file_new_for_path ("/tmp/asbuilder/output/appstream.xml.gz");
 	store = as_store_new ();
 	ret = as_store_from_file (store, file, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
-	g_assert_cmpint (as_store_get_size (store), ==, 2);
+	g_assert_cmpint (as_store_get_size (store), ==, 5);
 	app = as_store_get_app_by_pkgname (store, "app");
 	g_assert (app != NULL);
 	app = as_store_get_app_by_id (store, "app.desktop");
@@ -353,6 +365,36 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	xml = as_store_to_xml (store, AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
 	expected_xml =
 		"<components version=\"0.8\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test\">\n"
+		"<component type=\"font\">\n"
+		"<id>Liberation</id>\n"
+		"<pkgname>font</pkgname>\n"
+		"<pkgname>font-serif</pkgname>\n"
+		"<name>Liberation</name>\n"
+		"<summary>Open source versions of several commecial fonts</summary>\n"
+		"<description><p>The Liberation Fonts are intended to be replacements for Times New Roman, Arial, and Courier New.</p></description>\n"
+		"<icon height=\"64\" width=\"64\" type=\"cached\">LiberationSerif.png</icon>\n"
+		"<project_license>GPL-2.0+</project_license>\n"
+		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
+		"<screenshots>\n"
+		"<screenshot type=\"default\">\n"
+		"<caption>Liberation Serif – Regular</caption>\n"
+		"<image type=\"source\" height=\"48\" width=\"640\"/>\n"
+		"</screenshot>\n"
+		"<screenshot priority=\"-32\">\n"
+		"<caption>Liberation Serif – Bold</caption>\n"
+		"<image type=\"source\" height=\"48\" width=\"640\"/>\n"
+		"</screenshot>\n"
+		"</screenshots>\n"
+		"<releases>\n"
+		"<release version=\"1\" timestamp=\"1407844800\"/>\n"
+		"</releases>\n"
+		"<languages>\n"
+		"<lang>en</lang>\n"
+		"</languages>\n"
+		"<metadata>\n"
+		"<value key=\"X-CacheID\">font-1-1.fc21.noarch.rpm</value>\n"
+		"</metadata>\n"
+		"</component>\n"
 		"<component type=\"addon\">\n"
 		"<id>app-extra</id>\n"
 		"<pkgname>app-extra</pkgname>\n"
@@ -379,7 +421,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<name>App</name>\n"
 		"<summary>A test application</summary>\n"
 		"<description><p>Long description goes here.</p></description>\n"
-		"<icon type=\"cached\">app.png</icon>\n"
+		"<icon height=\"64\" width=\"64\" type=\"cached\">app.png</icon>\n"
 		"<categories>\n"
 		"<category>Profiling</category>\n"
 		"<category>System</category>\n"
@@ -408,21 +450,64 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<lang percentage=\"33\">ru</lang>\n"
 		"</languages>\n"
 		"<metadata>\n"
+		"<value key=\"PaddedIcon\">48x48</value>\n"
 		"<value key=\"X-CacheID\">app-1-1.fc21.x86_64.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
+		"<component type=\"desktop\">\n"
+		"<id>valid1.desktop</id>\n"
+		"<pkgname>composite</pkgname>\n"
+		"<name>Frobnicator</name>\n"
+		"<summary>Frobnicator</summary>\n"
+		"<icon type=\"stock\">computer</icon>\n"
+		"<categories>\n"
+		"<category>Profiling</category>\n"
+		"</categories>\n"
+		"<kudos>\n"
+		"<kudo>HiDpiIcon</kudo>\n"
+		"</kudos>\n"
+		"<project_license>GPL-2.0+</project_license>\n"
+		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
+		"<releases>\n"
+		"<release version=\"1\" timestamp=\"1407844800\"/>\n"
+		"</releases>\n"
+		"<metadata>\n"
+		"<value key=\"X-CacheID\">composite-1-1.fc21.x86_64.rpm</value>\n"
+		"</metadata>\n"
+		"</component>\n"
+		"<component type=\"desktop\">\n"
+		"<id>valid2.desktop</id>\n"
+		"<pkgname>composite</pkgname>\n"
+		"<name>Frobnicator Example</name>\n"
+		"<summary>Frobnicator Example Program</summary>\n"
+		"<icon type=\"stock\">computer</icon>\n"
+		"<categories>\n"
+		"<category>Profiling</category>\n"
+		"</categories>\n"
+		"<kudos>\n"
+		"<kudo>HiDpiIcon</kudo>\n"
+		"</kudos>\n"
+		"<project_license>GPL-2.0+</project_license>\n"
+		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
+		"<releases>\n"
+		"<release version=\"1\" timestamp=\"1407844800\"/>\n"
+		"</releases>\n"
+		"<metadata>\n"
+		"<value key=\"X-CacheID\">composite-1-1.fc21.x86_64.rpm</value>\n"
+		"</metadata>\n"
+		"</component>\n"
 		"</components>\n";
-	if (g_strcmp0 (xml->str, expected_xml) != 0)
-		g_warning ("Expected:\n%s\nGot:\n%s", expected_xml, xml->str);
-	g_assert_cmpstr (xml->str, ==, expected_xml);
+	ret = asb_test_compare_lines (xml->str, expected_xml, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
 
 	/* load failed metadata */
-	file_failed = g_file_new_for_path ("/tmp/asbuilder/output/asb-self-test-failed.xml.gz");
+	file_failed = g_file_new_for_path ("/tmp/asbuilder/output/appstream-failed.xml.gz");
 	store_failed = as_store_new ();
 	ret = as_store_from_file (store_failed, file_failed, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (ret);
-	g_assert_cmpint (as_store_get_size (store_failed), ==, 2);
+	g_assert_cmpint (as_store_get_size (store_failed), ==, 4);
 	app = as_store_get_app_by_id (store_failed, "console1.desktop");
 	g_assert (app != NULL);
 	app = as_store_get_app_by_id (store_failed, "console2.desktop");
@@ -432,13 +517,83 @@ asb_test_context_test_func (AsbTestContextMode mode)
 	xml_failed = as_store_to_xml (store_failed, AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
 	expected_xml =
 		"<components version=\"0.8\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test-failed\">\n"
+		"<component type=\"font\">\n"
+		"<id>LiberationSerif</id>\n"
+		"<pkgname>font-serif</pkgname>\n"
+		"<source_pkgname>font</source_pkgname>\n"
+		"<name>Liberation Serif</name>\n"
+		"<summary>A Bold font from Liberation Serif</summary>\n"
+		"<icon height=\"64\" width=\"64\" type=\"cached\">LiberationSerif.png</icon>\n"
+		"<categories>\n"
+		"<category>Addons</category>\n"
+		"<category>Fonts</category>\n"
+		"</categories>\n"
+		"<vetos>\n"
+		"<veto>LiberationSerif was merged into Liberation</veto>\n"
+		"</vetos>\n"
+		"<project_license>GPL-2.0+</project_license>\n"
+		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
+		"<extends>Liberation</extends>\n"
+		"<screenshots>\n"
+		"<screenshot type=\"default\">\n"
+		"<caption>Liberation Serif – Regular</caption>\n"
+		"<image type=\"source\" height=\"48\" width=\"640\"/>\n"
+		"</screenshot>\n"
+		"<screenshot priority=\"-32\">\n"
+		"<caption>Liberation Serif – Bold</caption>\n"
+		"<image type=\"source\" height=\"48\" width=\"640\"/>\n"
+		"</screenshot>\n"
+		"</screenshots>\n"
+		"<releases>\n"
+		"<release version=\"1\" timestamp=\"1407844800\"/>\n"
+		"</releases>\n"
+		"<languages>\n"
+		"<lang>en</lang>\n"
+		"</languages>\n"
+		"<metadata>\n"
+		"<value key=\"X-CacheID\">font-serif-1-1.fc21.noarch.rpm</value>\n"
+		"</metadata>\n"
+		"</component>\n"
+		"<component type=\"addon\">\n"
+		"<id>app-core</id>\n"
+		"<pkgname>app</pkgname>\n"
+		"<name>Core</name>\n"
+		"<summary>Addons for core functionality</summary>\n"
+		"<kudos>\n"
+		"<kudo>AppMenu</kudo>\n"
+		"<kudo>ModernToolkit</kudo>\n"
+		"<kudo>Notifications</kudo>\n"
+		"<kudo>SearchProvider</kudo>\n"
+		"<kudo>UserDocs</kudo>\n"
+		"</kudos>\n"
+		"<vetos>\n"
+		"<veto>partially absorbing app-core into app.desktop</veto>\n"
+		"</vetos>\n"
+		"<project_license>GPL-2.0+</project_license>\n"
+		"<url type=\"homepage\">http://people.freedesktop.org/</url>\n"
+		"<extends>app.desktop</extends>\n"
+		"<releases>\n"
+		"<release version=\"1\" timestamp=\"1407844800\"/>\n"
+		"</releases>\n"
+		"<provides>\n"
+		"<dbus type=\"session\">org.freedesktop.AppStream</dbus>\n"
+		"</provides>\n"
+		"<languages>\n"
+		"<lang percentage=\"100\">en_GB</lang>\n"
+		"<lang percentage=\"33\">ru</lang>\n"
+		"</languages>\n"
+		"<metadata>\n"
+		"<value key=\"X-CacheID\">app-1-1.fc21.x86_64.rpm</value>\n"
+		"<value key=\"X-Merge-With-Parent\">app.desktop</value>\n"
+		"</metadata>\n"
+		"</component>\n"
 		"<component type=\"desktop\">\n"
 		"<id>console1.desktop</id>\n"
 		"<pkgname>app-console</pkgname>\n"
 		"<source_pkgname>app</source_pkgname>\n"
 		"<name>Console1</name>\n"
 		"<summary>A console1 test application</summary>\n"
-		"<icon type=\"cached\">console1.png</icon>\n"
+		"<icon height=\"64\" width=\"64\" type=\"cached\">console1.png</icon>\n"
 		"<categories>\n"
 		"<category>ConsoleOnly</category>\n"
 		"</categories>\n"
@@ -455,6 +610,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<lang percentage=\"33\">ru</lang>\n"
 		"</languages>\n"
 		"<metadata>\n"
+		"<value key=\"PaddedIcon\">48x48</value>\n"
 		"<value key=\"X-CacheID\">app-console-1-1.fc21.noarch.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
@@ -464,7 +620,7 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<source_pkgname>app</source_pkgname>\n"
 		"<name>Console2</name>\n"
 		"<summary>A console2 test application</summary>\n"
-		"<icon type=\"cached\">console2.png</icon>\n"
+		"<icon height=\"64\" width=\"64\" type=\"cached\">console2.png</icon>\n"
 		"<categories>\n"
 		"<category>ConsoleOnly</category>\n"
 		"</categories>\n"
@@ -481,16 +637,17 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"<lang percentage=\"33\">ru</lang>\n"
 		"</languages>\n"
 		"<metadata>\n"
+		"<value key=\"PaddedIcon\">48x48</value>\n"
 		"<value key=\"X-CacheID\">app-console-1-1.fc21.noarch.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
 		"</components>\n";
-	if (g_strcmp0 (xml_failed->str, expected_xml) != 0)
-		g_warning ("Expected:\n%s\nGot:\n%s", expected_xml, xml_failed->str);
-	g_assert_cmpstr (xml_failed->str, ==, expected_xml);
+	ret = asb_test_compare_lines (xml_failed->str, expected_xml, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
 
 	/* load ignored metadata */
-	file_ignore = g_file_new_for_path ("/tmp/asbuilder/output/asb-self-test-ignore.xml.gz");
+	file_ignore = g_file_new_for_path ("/tmp/asbuilder/output/appstream-ignore.xml.gz");
 	store_ignore = as_store_new ();
 	ret = as_store_from_file (store_ignore, file_ignore, NULL, NULL, &error);
 	g_assert_no_error (error);
@@ -513,25 +670,41 @@ asb_test_context_test_func (AsbTestContextMode mode)
 		"</metadata>\n"
 		"</component>\n"
 		"<component>\n"
+		"<id>font-serif.noarch</id>\n"
+		"<metadata>\n"
+		"<value key=\"X-CacheID\">font-serif-1-1.fc21.noarch.rpm</value>\n"
+		"</metadata>\n"
+		"</component>\n"
+		"<component>\n"
 		"<id>test.noarch</id>\n"
 		"<metadata>\n"
 		"<value key=\"X-CacheID\">test-0.1-1.fc21.noarch.rpm</value>\n"
 		"</metadata>\n"
 		"</component>\n"
 		"</components>\n";
-	if (g_strcmp0 (xml_ignore->str, expected_xml) != 0)
-		g_warning ("Expected:\n%s\nGot:\n%s", expected_xml, xml_ignore->str);
-	g_assert_cmpstr (xml_ignore->str, ==, expected_xml);
+	ret = asb_test_compare_lines (xml_ignore->str, expected_xml, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
 
+	/* check icon dir */
+	g_assert (g_file_test ("/tmp/asbuilder/temp/icons/64x64/app.png", G_FILE_TEST_EXISTS));
+	g_assert (!g_file_test ("/tmp/asbuilder/temp/icons/app.png", G_FILE_TEST_EXISTS));
+	g_assert (!g_file_test ("/tmp/asbuilder/temp/icons/128x128/app.png", G_FILE_TEST_EXISTS));
 }
 #endif
 
 static void
 asb_test_context_nocache_func (void)
 {
-#ifdef HAVE_RPM
 	GError *error = NULL;
 	gboolean ret;
+
+	/* remove icons */
+	ret = asb_utils_rmtree ("/tmp/asbuilder/temp/icons", &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+#ifdef HAVE_RPM
 	ret = asb_utils_rmtree ("/tmp/asbuilder/output", &error);
 	g_assert_no_error (error);
 	g_assert (ret);
@@ -578,9 +751,92 @@ asb_test_context_oldcache_func (void)
 #endif
 }
 
+static void
+asb_test_context_extra_appstream_func (void)
+{
+	GError *error = NULL;
+	gboolean ret;
+	AsApp *app;
+	const gchar *expected_xml;
+	_cleanup_free_ gchar *extra_appdata = NULL;
+	_cleanup_object_unref_ AsbContext *ctx = NULL;
+	_cleanup_object_unref_ AsStore *store = NULL;
+	_cleanup_object_unref_ GFile *file = NULL;
+	_cleanup_string_free_ GString *xml = NULL;
+
+	ctx = asb_context_new ();
+	extra_appdata = asb_test_get_filename ("extra-appdata");
+	g_assert (extra_appdata != NULL);
+	asb_context_set_extra_appstream (ctx, extra_appdata);
+	asb_context_set_max_threads (ctx, 1);
+	asb_context_set_api_version (ctx, 0.8);
+	asb_context_set_add_cache_id (ctx, TRUE);
+	asb_context_set_no_net (ctx, TRUE);
+	asb_context_set_hidpi_enabled (ctx, TRUE);
+	asb_context_set_basename (ctx, "appstream");
+	asb_context_set_origin (ctx, "asb-self-test");
+	asb_context_set_cache_dir (ctx, "/tmp/asbuilder/cache");
+	asb_context_set_output_dir (ctx, "/tmp/asbuilder/output");
+	asb_context_set_temp_dir (ctx, "/tmp/asbuilder/temp");
+	ret = asb_context_setup (ctx, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* run the plugins */
+	ret = asb_context_process (ctx, AS_CONTEXT_PARSE_FLAG_NONE, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* check files created */
+	g_assert (g_file_test ("/tmp/asbuilder/output/appstream.xml.gz", G_FILE_TEST_EXISTS));
+
+	/* load AppStream metadata */
+	file = g_file_new_for_path ("/tmp/asbuilder/output/appstream.xml.gz");
+	store = as_store_new ();
+	ret = as_store_from_file (store, file, NULL, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (as_store_get_size (store), ==, 2);
+	app = as_store_get_app_by_id (store, "epiphany-test.desktop");
+	g_assert (app != NULL);
+	app = as_store_get_app_by_id (store, "epiphany-local.desktop");
+	g_assert (app != NULL);
+
+	/* check it matches what we expect */
+	xml = as_store_to_xml (store, AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE);
+	expected_xml =
+		"<components version=\"0.8\" builder_id=\"appstream-glib:4\" origin=\"asb-self-test\">\n"
+		"<component type=\"webapp\">\n"
+		"<id>epiphany-local.desktop</id>\n"
+		"<name>Local</name>\n"
+		"<summary>My local webapp</summary>\n"
+		"<description><p>This is awesome</p></description>\n"
+		"<icon type=\"local\">/usr/share/icons/hicolor/256x256/apps/fedora-logo-icon.png</icon>\n"
+		"<url type=\"homepage\">http://www.hughski.com/</url>\n"
+		"</component>\n"
+		"<component type=\"webapp\">\n"
+		"<id>epiphany-test.desktop</id>\n"
+		"<name>Test</name>\n"
+		"<summary>Please use my awesome webapp</summary>\n"
+		"<description><p>This could be awesome</p></description>\n"
+		"<icon type=\"remote\">http://www.hughski.com/img/logo.png</icon>\n"
+		"<url type=\"homepage\">http://www.hughski.com/</url>\n"
+		"</component>\n"
+		"</components>\n";
+	ret = asb_test_compare_lines (xml->str, expected_xml, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* remove temp space */
+	ret = asb_utils_rmtree ("/tmp/asbuilder", &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+}
+
 int
 main (int argc, char **argv)
 {
+	setlocale (LC_ALL, "");
 	g_test_init (&argc, &argv, NULL);
 
 	/* only critical and error are fatal */
@@ -594,6 +850,7 @@ main (int argc, char **argv)
 	g_test_add_func ("/AppStreamBuilder/context{no-cache}", asb_test_context_nocache_func);
 	g_test_add_func ("/AppStreamBuilder/context{cache}", asb_test_context_cache_func);
 	g_test_add_func ("/AppStreamBuilder/context{old-cache}", asb_test_context_oldcache_func);
+	g_test_add_func ("/AppStreamBuilder/context{extra-appstream}", asb_test_context_extra_appstream_func);
 #ifdef HAVE_RPM
 	g_test_add_func ("/AppStreamBuilder/package{rpm}", asb_test_package_rpm_func);
 #endif
