@@ -141,9 +141,9 @@ asb_task_explode_extra_packages (AsbTask *task, GError **error)
 	const gchar *tmp;
 	gchar **deps;
 	guint i;
-	_cleanup_hashtable_unref_ GHashTable *hash;
-	_cleanup_ptrarray_unref_ GPtrArray *array;
-	_cleanup_ptrarray_unref_ GPtrArray *icon_themes;
+	_cleanup_hashtable_unref_ GHashTable *hash = NULL;
+	_cleanup_ptrarray_unref_ GPtrArray *array = NULL;
+	_cleanup_ptrarray_unref_ GPtrArray *icon_themes = NULL;
 
 	/* anything the package requires */
 	hash = g_hash_table_new (g_str_hash, g_str_equal);
@@ -299,20 +299,26 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 	/* run plugins */
 	asb_panel_set_status (priv->panel, "Examining");
 	for (i = 0; i < priv->plugins_to_run->len; i++) {
+		GList *apps_tmp = NULL;
 		plugin = g_ptr_array_index (priv->plugins_to_run, i);
 		asb_package_log (priv->pkg,
 				 ASB_PACKAGE_LOG_LEVEL_DEBUG,
 				 "Processing %s with %s",
 				 basename,
 				 plugin->name);
-		apps = asb_plugin_process (plugin, priv->pkg, priv->tmpdir, &error);
-		if (apps == NULL) {
+		apps_tmp = asb_plugin_process (plugin, priv->pkg, priv->tmpdir, &error);
+		if (apps_tmp == NULL) {
 			asb_package_log (priv->pkg,
 					 ASB_PACKAGE_LOG_LEVEL_WARNING,
 					 "Failed to run process '%s': %s",
 					 plugin->name, error->message);
 			g_clear_error (&error);
 		}
+		for (l = apps_tmp; l != NULL; l = l->next) {
+			app = ASB_APP (l->data);
+			asb_plugin_add_app (&apps, AS_APP (app));
+		}
+		g_list_free_full (apps_tmp, g_object_unref);
 	}
 	if (apps == NULL)
 		goto skip;
@@ -335,6 +341,7 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 		if (!asb_package_ensure (priv->pkg,
 					 ASB_PACKAGE_ENSURE_LICENSE |
 					 ASB_PACKAGE_ENSURE_RELEASES |
+					 ASB_PACKAGE_ENSURE_VCS |
 					 ASB_PACKAGE_ENSURE_URL,
 					 error_not_used))
 			return FALSE;
@@ -380,18 +387,6 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 			goto skip;
 		}
 
-		/* don't include components that have no name or comment */
-		if (as_app_get_name (AS_APP (app), "C") == NULL)
-			as_app_add_veto (AS_APP (app), "No 'Name' in desktop or <name> in AppData");
-		if (as_app_get_comment (AS_APP (app), "C") == NULL)
-			as_app_add_veto (AS_APP (app), "No 'Comment' in desktop or <summary> in AppData");
-
-		/* don't include apps that have no icon */
-		if (as_app_get_id_kind (AS_APP (app)) != AS_ID_KIND_ADDON) {
-			if (as_app_get_icon (AS_APP (app)) == NULL)
-				as_app_add_veto (AS_APP (app), "Has no Icon");
-		}
-
 		/* veto apps that *still* require appdata */
 		array = asb_app_get_requires_appdata (app);
 		for (i = 0; i < array->len; i++) {
@@ -403,19 +398,6 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 			as_app_add_veto (AS_APP (app), "Required AppData: %s", tmp);
 		}
 
-		/* save icon and screenshots */
-		if (array->len == 0) {
-			ret = asb_app_save_resources (app, &error);
-			if (!ret) {
-				asb_package_log (priv->pkg,
-						 ASB_PACKAGE_LOG_LEVEL_WARNING,
-						 "Failed to save resources: %s",
-						 error->message);
-				g_clear_error (&error);
-				goto skip;
-			}
-		}
-
 		/* set cache-id in case we want to use the metadata directly */
 		if (asb_context_get_add_cache_id (priv->ctx)) {
 			cache_id = asb_utils_get_cache_id_for_filename (priv->filename);
@@ -425,14 +407,24 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 			g_free (cache_id);
 		}
 
+		/* set the VCS information into the metadata */
+		if (asb_package_get_vcs (priv->pkg) != NULL) {
+			as_app_add_metadata (AS_APP (app),
+					     "VersionControlSystem",
+					     asb_package_get_vcs (priv->pkg), -1);
+		}
+
+		/* save any screenshots early */
+		if (array->len == 0) {
+			if (!asb_app_save_resources (ASB_APP (app),
+						     ASB_APP_SAVE_FLAG_SCREENSHOTS,
+						     error_not_used))
+				return FALSE;
+		}
+
 		/* all okay */
 		asb_context_add_app (priv->ctx, app);
 		nr_added++;
-
-		/* log the XML in the log file */
-		tmp = asb_app_to_xml (app);
-		asb_package_log (priv->pkg, ASB_PACKAGE_LOG_LEVEL_NONE, "%s", tmp);
-		g_free (tmp);
 	}
 skip:
 	/* add a dummy element to the AppStream metadata so that we don't keep
