@@ -157,7 +157,7 @@ asb_utils_ensure_exists_and_empty (const gchar *directory, GError **error)
 
 	/* find each */
 	while ((filename = g_dir_read_name (dir))) {
-		_cleanup_free_ gchar *src;
+		_cleanup_free_ gchar *src = NULL;
 		src = g_build_filename (directory, filename, NULL);
 		if (g_file_test (src, G_FILE_TEST_IS_DIR)) {
 			if (!asb_utils_rmtree (src, error))
@@ -175,52 +175,106 @@ asb_utils_ensure_exists_and_empty (const gchar *directory, GError **error)
 	return TRUE;
 }
 
+/**
+ * asb_utils_count_directories_deep:
+ **/
+static const guint
+asb_utils_count_directories_deep (const gchar *path)
+{
+	guint cnt = 0;
+	guint i;
+
+	for (i = 0; path[i] != '\0'; i++) {
+		if (path[i] != '/')
+			continue;
+		cnt++;
+	}
+	return cnt;
+}
+
+/**
+ * asb_utils_get_back_to_root:
+ **/
+static gchar *
+asb_utils_get_back_to_root (guint levels)
+{
+	GString *str;
+	guint i;
+
+	str = g_string_new ("");
+	for (i = 0; i < levels; i++)
+		g_string_append (str, "../");
+	return g_string_free (str, FALSE);
+}
+
+/**
+ * asb_utils_sanitise_path:
+ *
+ * Converts various formats into an absolute path.
+ **/
+static gchar *
+asb_utils_sanitise_path (const gchar *path)
+{
+	/* /usr/share/README -> /usr/share/README */
+	if (path[0] == '/')
+		return g_strdup (path);
+
+	/* ./usr/share/README -> /usr/share/README */
+	if (path[0] == '.')
+		return g_strdup (path + 1);
+
+	/* usr/share/README -> /usr/share/README */
+	return g_strconcat ("/", path, NULL);
+}
+
 /**
  * asb_utils_explode_file:
  **/
 static gboolean
-asb_utils_explode_file (struct archive_entry *entry,
-			const gchar *dir,
-			GPtrArray *glob)
+asb_utils_explode_file (struct archive_entry *entry, const gchar *dir)
 {
 	const gchar *tmp;
-	gchar buf[PATH_MAX];
+	guint symlink_depth;
+	_cleanup_free_ gchar *back_up = NULL;
 	_cleanup_free_ gchar *path = NULL;
+	_cleanup_free_ gchar *buf = NULL;
 
 	/* no output file */
 	if (archive_entry_pathname (entry) == NULL)
 		return FALSE;
 
-	/* do we have to decompress this file */
-	tmp = archive_entry_pathname (entry);
-	if (glob != NULL) {
-		if (tmp[0] == '/') {
-			path = g_strdup (tmp);
-		} else if (tmp[0] == '.') {
-			path = g_strdup (tmp + 1);
-		} else {
-			path = g_strconcat ("/", tmp, NULL);
-		}
-		if (asb_glob_value_search (glob, path) == NULL)
-			return FALSE;
-	}
-
 	/* update output path */
-	g_snprintf (buf, PATH_MAX, "%s/%s", dir, tmp);
+	tmp = archive_entry_pathname (entry);
+	path = asb_utils_sanitise_path (tmp);
+	buf = g_build_filename (dir, path, NULL);
 	archive_entry_update_pathname_utf8 (entry, buf);
 
 	/* update hardlinks */
 	tmp = archive_entry_hardlink (entry);
 	if (tmp != NULL) {
-		g_snprintf (buf, PATH_MAX, "%s/%s", dir, tmp);
-		archive_entry_update_hardlink_utf8 (entry, buf);
+		_cleanup_free_ gchar *buf_link = NULL;
+		_cleanup_free_ gchar *path_link = NULL;
+		path_link = asb_utils_sanitise_path (tmp);
+		buf_link = g_build_filename (dir, path_link, NULL);
+		if (!g_file_test (buf_link, G_FILE_TEST_EXISTS)) {
+			g_warning ("%s does not exist, cannot hardlink", tmp);
+			return FALSE;
+		}
+		archive_entry_update_hardlink_utf8 (entry, buf_link);
 	}
 
 	/* update symlinks */
 	tmp = archive_entry_symlink (entry);
 	if (tmp != NULL) {
-		g_snprintf (buf, PATH_MAX, "%s/%s", dir, tmp);
-		archive_entry_update_symlink_utf8 (entry, buf);
+		_cleanup_free_ gchar *buf_link = NULL;
+		_cleanup_free_ gchar *path_link = NULL;
+		path_link = asb_utils_sanitise_path (tmp);
+		symlink_depth = asb_utils_count_directories_deep (path) - 1;
+		back_up = asb_utils_get_back_to_root (symlink_depth);
+		if (tmp[0] == '/')
+			tmp++;
+		buf_link = g_build_filename (back_up, tmp, NULL);
+		archive_entry_update_symlink_utf8 (entry, buf_link);
 	}
 	return TRUE;
 }
@@ -244,20 +298,82 @@ asb_utils_explode (const gchar *filename,
 		   GPtrArray *glob,
 		   GError **error)
 {
+	const gchar *tmp;
 	gboolean ret = TRUE;
 	gboolean valid;
 	gsize len;
 	int r;
 	struct archive *arch = NULL;
+	struct archive *arch_preview = NULL;
 	struct archive_entry *entry;
 	_cleanup_free_ gchar *data = NULL;
+	_cleanup_hashtable_unref_ GHashTable *matches = NULL;
 
 	/* load file at once to avoid seeking */
 	ret = g_file_get_contents (filename, &data, &len, error);
 	if (!ret)
 		goto out;
 
-	/* read anything */
+	/* populate a hash with all the files, symlinks and hardlinks that
+	 * actually need decompressing */
+	arch_preview = archive_read_new ();
+	archive_read_support_format_all (arch_preview);
+	archive_read_support_filter_all (arch_preview);
+	r = archive_read_open_memory (arch_preview, data, len);
+	if (r) {
+		ret = FALSE;
+		g_set_error (error,
+			     ASB_PLUGIN_ERROR,
+			     ASB_PLUGIN_ERROR_FAILED,
+			     "Cannot open: %s",
+			     archive_error_string (arch_preview));
+		goto out;
+	}
+	matches = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	for (;;) {
+		_cleanup_free_ gchar *path = NULL;
+		r = archive_read_next_header (arch_preview, &entry);
+		if (r == ARCHIVE_EOF)
+			break;
+		if (r != ARCHIVE_OK) {
+			ret = FALSE;
+			g_set_error (error,
+				     ASB_PLUGIN_ERROR,
+				     ASB_PLUGIN_ERROR_FAILED,
+				     "Cannot read header: %s",
+				     archive_error_string (arch_preview));
+			goto out;
+		}
+
+		/* get the destination filename */
+		tmp = archive_entry_pathname (entry);
+		if (tmp == NULL)
+			continue;
+		path = asb_utils_sanitise_path (tmp);
+		if (glob != NULL) {
+			if (asb_glob_value_search (glob, path) == NULL)
+				continue;
+		}
+		g_hash_table_insert (matches, g_strdup (path), GINT_TO_POINTER (1));
+
+		/* add hardlink */
+		tmp = archive_entry_hardlink (entry);
+		if (tmp != NULL) {
+			g_hash_table_insert (matches,
+					     asb_utils_sanitise_path (tmp),
+					     GINT_TO_POINTER (1));
+		}
+
+		/* add symlink */
+		tmp = archive_entry_symlink (entry);
+		if (tmp != NULL) {
+			g_hash_table_insert (matches,
+					     asb_utils_sanitise_path (tmp),
+					     GINT_TO_POINTER (1));
+		}
+	}
+
+	/* decompress anything matching either glob */
 	arch = archive_read_new ();
 	archive_read_support_format_all (arch);
 	archive_read_support_filter_all (arch);
@@ -271,9 +387,8 @@ asb_utils_explode (const gchar *filename,
 			     archive_error_string (arch));
 		goto out;
 	}
-
-	/* decompress each file */
 	for (;;) {
+		_cleanup_free_ gchar *path = NULL;
 		r = archive_read_next_header (arch, &entry);
 		if (r == ARCHIVE_EOF)
 			break;
@@ -288,7 +403,11 @@ asb_utils_explode (const gchar *filename,
 		}
 
 		/* only extract if valid */
-		valid = asb_utils_explode_file (entry, dir, glob);
+		tmp = archive_entry_pathname (entry);
+		path = asb_utils_sanitise_path (tmp);
+		if (g_hash_table_lookup (matches, path) == NULL)
+			continue;
+		valid = asb_utils_explode_file (entry, dir);
 		if (!valid)
 			continue;
 		r = archive_read_extract (arch, entry, 0);
@@ -303,6 +422,10 @@ asb_utils_explode (const gchar *filename,
 		}
 	}
 out:
+	if (arch_preview != NULL) {
+		archive_read_close (arch_preview);
+		archive_read_free (arch_preview);
+	}
 	if (arch != NULL) {
 		archive_read_close (arch);
 		archive_read_free (arch);
@@ -315,6 +438,7 @@ out:
  **/
 static gboolean
 asb_utils_write_archive (const gchar *filename,
+			 const gchar *path_orig,
 			 GPtrArray *files,
 			 GError **error)
 {
@@ -331,18 +455,19 @@ asb_utils_write_archive (const gchar *filename,
 	archive_write_set_format_pax_restricted (a);
 	archive_write_open_filename (a, filename);
 	for (i = 0; i < files->len; i++) {
-		_cleanup_free_ gchar *basename;
 		_cleanup_free_ gchar *data = NULL;
+		_cleanup_free_ gchar *filename_full = NULL;
+
 		tmp = g_ptr_array_index (files, i);
-		stat (tmp, &st);
+		filename_full = g_build_filename (path_orig, tmp, NULL);
+		stat (filename_full, &st);
 		entry = archive_entry_new ();
-		basename = g_path_get_basename (tmp);
-		archive_entry_set_pathname (entry, basename);
+		archive_entry_set_pathname (entry, tmp);
 		archive_entry_set_size (entry, st.st_size);
 		archive_entry_set_filetype (entry, AE_IFREG);
 		archive_entry_set_perm (entry, 0644);
 		archive_write_header (a, entry);
-		ret = g_file_get_contents (tmp, &data, &len, error);
+		ret = g_file_get_contents (filename_full, &data, &len, error);
 		if (!ret)
 			goto out;
 		archive_write_data (a, data, len);
@@ -354,6 +479,38 @@ out:
 	return ret;
 }
 
+/**
+ * asb_utils_add_files_recursive:
+ **/
+static gboolean
+asb_utils_add_files_recursive (GPtrArray *files,
+			       const gchar *path_orig,
+			       const gchar *path,
+			       GError **error)
+{
+	const gchar *path_trailing;
+	const gchar *tmp;
+	guint path_orig_len;
+	_cleanup_dir_close_ GDir *dir = NULL;
+
+	dir = g_dir_open (path, 0, error);
+	if (dir == NULL)
+		return FALSE;
+	path_orig_len = strlen (path_orig);
+	while ((tmp = g_dir_read_name (dir)) != NULL) {
+		_cleanup_free_ gchar *path_new = NULL;
+		path_new = g_build_filename (path, tmp, NULL);
+		if (g_file_test (path_new, G_FILE_TEST_IS_DIR)) {
+			if (!asb_utils_add_files_recursive (files, path_orig, path_new, error))
+				return FALSE;
+		} else {
+			path_trailing = path_new + path_orig_len + 1;
+			g_ptr_array_add (files, g_strdup (path_trailing));
+		}
+	}
+	return TRUE;
+}
+
 /**
  * asb_utils_write_archive_dir:
  * @filename: archive filename
@@ -371,20 +528,17 @@ asb_utils_write_archive_dir (const gchar *filename,
 			     const gchar *directory,
 			     GError **error)
 {
-	const gchar *tmp;
-	_cleanup_dir_close_ GDir *dir = NULL;
 	_cleanup_ptrarray_unref_ GPtrArray *files = NULL;
 
 	/* add all files in the directory to the archive */
-	dir = g_dir_open (directory, 0, error);
-	if (dir == NULL)
-		return FALSE;
 	files = g_ptr_array_new_with_free_func (g_free);
-	while ((tmp = g_dir_read_name (dir)) != NULL)
-		g_ptr_array_add (files, g_build_filename (directory, tmp, NULL));
+	if (!asb_utils_add_files_recursive (files, directory, directory, error))
+		return FALSE;
+	if (files->len == 0)
+		return TRUE;
 
 	/* write tar file */
-	return asb_utils_write_archive (filename, files, error);
+	return asb_utils_write_archive (filename, directory, files, error);
 }
 
 /**
@@ -559,6 +713,9 @@ asb_glob_value_search (GPtrArray *array, const gchar *search)
 	const AsbGlobValue *tmp;
 	guint i;
 
+	g_return_val_if_fail (array != NULL, NULL);
+	g_return_val_if_fail (search != NULL, NULL);
+
 	/* invalid */
 	if (search == NULL)
 		return NULL;
