@@ -51,7 +51,7 @@ asb_plugin_initialize (AsbPlugin *plugin)
 	plugin->priv->filenames = g_ptr_array_new_with_free_func (g_free);
 	g_mutex_init (&plugin->priv->filenames_mutex);
 	plugin->priv->session = soup_session_new_with_options (SOUP_SESSION_USER_AGENT, "createrepo_as",
-							       SOUP_SESSION_TIMEOUT, 5000,
+							       SOUP_SESSION_TIMEOUT, 10,
 							       NULL);
 	soup_session_add_feature_by_type (plugin->priv->session,
 					  SOUP_TYPE_PROXY_RESOLVER_DEFAULT);
@@ -101,7 +101,7 @@ asb_plugin_appdata_add_path (AsbPlugin *plugin, const gchar *path, GError **erro
 	if (dir == NULL)
 		return FALSE;
 	while ((tmp = g_dir_read_name (dir)) != NULL) {
-		_cleanup_free_ gchar *filename;
+		_cleanup_free_ gchar *filename = NULL;
 		filename = g_build_filename (path, tmp, NULL);
 		if (g_file_test (filename, G_FILE_TEST_IS_DIR)) {
 			if (!asb_plugin_appdata_add_path (plugin, filename, error))
@@ -187,8 +187,8 @@ asb_plugin_appdata_load_url (AsbPlugin *plugin,
 	gboolean ret = TRUE;
 	SoupStatus status;
 	SoupURI *uri = NULL;
-	_cleanup_free_ gchar *basename;
-	_cleanup_free_ gchar *cache_filename;
+	_cleanup_free_ gchar *basename = NULL;
+	_cleanup_free_ gchar *cache_filename = NULL;
 	_cleanup_object_unref_ SoupMessage *msg = NULL;
 
 	/* download to cache if not already added */
@@ -268,7 +268,7 @@ asb_plugin_process_filename (AsbPlugin *plugin,
 	GList *l;
 	GList *list;
 	guint i;
-	_cleanup_object_unref_ AsApp *appdata;
+	_cleanup_object_unref_ AsApp *appdata = NULL;
 	_cleanup_ptrarray_unref_ GPtrArray *problems = NULL;
 
 	/* validate */
@@ -502,10 +502,11 @@ asb_plugin_process_app (AsbPlugin *plugin,
 			const gchar *tmpdir,
 			GError **error)
 {
+	GError *error_local = NULL;
 	const gchar *kind_str;
 	const gchar *tmp;
-	_cleanup_free_ gchar *appdata_basename;
-	_cleanup_free_ gchar *appdata_filename;
+	_cleanup_free_ gchar *appdata_basename = NULL;
+	_cleanup_free_ gchar *appdata_filename = NULL;
 	_cleanup_free_ gchar *appdata_filename_extra = NULL;
 
 	/* get possible sources */
@@ -545,10 +546,20 @@ asb_plugin_process_app (AsbPlugin *plugin,
 
 	/* any installed appdata file */
 	if (g_file_test (appdata_filename, G_FILE_TEST_EXISTS)) {
-		return asb_plugin_process_filename (plugin,
-						    app,
-						    appdata_filename,
-						    error);
+		/* be understanding if upstream gets the AppData file
+		 * wrong -- just fall back to the desktop file data */
+		if (!asb_plugin_process_filename (plugin,
+						  app,
+						  appdata_filename,
+						  &error_local)) {
+			error_local->code = ASB_PLUGIN_ERROR_IGNORE;
+			g_propagate_error (error, error_local);
+			g_prefix_error (error,
+					"AppData file '%s' invalid: ",
+					appdata_filename);
+			return FALSE;
+		}
+		return TRUE;
 	}
 
 	/* we're going to require this soon */
