@@ -81,6 +81,8 @@ static GdkPixbuf *
 asb_app_load_icon (AsbApp *app,
 		   const gchar *filename,
 		   const gchar *logfn,
+		   guint icon_size,
+		   guint min_icon_size,
 		   GError **error)
 {
 	GdkPixbuf *pixbuf = NULL;
@@ -94,8 +96,8 @@ asb_app_load_icon (AsbApp *app,
 	/* open file in native size */
 	if (g_str_has_suffix (filename, ".svg")) {
 		pixbuf_src = gdk_pixbuf_new_from_file_at_scale (filename,
-								AS_APP_ICON_DEFAULT_WIDTH,
-								AS_APP_ICON_DEFAULT_HEIGHT,
+								icon_size,
+								icon_size,
 								TRUE, error);
 	} else {
 		pixbuf_src = gdk_pixbuf_new_from_file (filename, error);
@@ -104,8 +106,8 @@ asb_app_load_icon (AsbApp *app,
 		return NULL;
 
 	/* check size */
-	if (gdk_pixbuf_get_width (pixbuf_src) < AS_APP_ICON_MIN_WIDTH ||
-	    gdk_pixbuf_get_height (pixbuf_src) < AS_APP_ICON_MIN_HEIGHT) {
+	if (gdk_pixbuf_get_width (pixbuf_src) < (gint) min_icon_size &&
+	    gdk_pixbuf_get_height (pixbuf_src) < (gint) min_icon_size) {
 		g_set_error (error,
 			     ASB_PLUGIN_ERROR,
 			     ASB_PLUGIN_ERROR_FAILED,
@@ -127,38 +129,37 @@ asb_app_load_icon (AsbApp *app,
 	/* don't do anything to an icon with the perfect size */
 	pixbuf_width = gdk_pixbuf_get_width (pixbuf_src);
 	pixbuf_height = gdk_pixbuf_get_height (pixbuf_src);
-	if (pixbuf_width == AS_APP_ICON_DEFAULT_WIDTH &&
-	    pixbuf_height == AS_APP_ICON_DEFAULT_HEIGHT)
+	if (pixbuf_width == icon_size && pixbuf_height == icon_size)
 		return g_object_ref (pixbuf_src);
 
 	/* never scale up, just pad */
-	if (pixbuf_width < AS_APP_ICON_DEFAULT_WIDTH &&
-	    pixbuf_height < AS_APP_ICON_DEFAULT_HEIGHT) {
+	if (pixbuf_width < icon_size && pixbuf_height < icon_size) {
+		_cleanup_free_ gchar *size_str = NULL;
+		size_str = g_strdup_printf ("%ix%i",
+					    pixbuf_width,
+					    pixbuf_height);
+		as_app_add_metadata (AS_APP (app), "PaddedIcon",
+				     size_str, -1);
 		asb_package_log (asb_app_get_package (app),
 				 ASB_PACKAGE_LOG_LEVEL_INFO,
-				 "icon %s padded to %ix%i as size %ix%i",
-				 logfn,
-				 AS_APP_ICON_DEFAULT_WIDTH,
-				 AS_APP_ICON_DEFAULT_HEIGHT,
-				 pixbuf_width, pixbuf_height);
+				 "icon %s padded to %ix%i as size %s",
+				 logfn, icon_size, icon_size, size_str);
 		pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-					 AS_APP_ICON_DEFAULT_WIDTH,
-					 AS_APP_ICON_DEFAULT_HEIGHT);
+					 icon_size, icon_size);
 		gdk_pixbuf_fill (pixbuf, 0x00000000);
 		gdk_pixbuf_copy_area (pixbuf_src,
 				      0, 0, /* of src */
 				      pixbuf_width, pixbuf_height,
 				      pixbuf,
-				      (AS_APP_ICON_DEFAULT_WIDTH - pixbuf_width) / 2,
-				      (AS_APP_ICON_DEFAULT_HEIGHT - pixbuf_height) / 2);
+				      (icon_size - pixbuf_width) / 2,
+				      (icon_size - pixbuf_height) / 2);
 		return pixbuf;
 	}
 
 	/* is the aspect ratio perfectly square */
 	if (pixbuf_width == pixbuf_height) {
 		pixbuf = gdk_pixbuf_scale_simple (pixbuf_src,
-						  AS_APP_ICON_DEFAULT_WIDTH,
-						  AS_APP_ICON_DEFAULT_HEIGHT,
+						  icon_size, icon_size,
 						  GDK_INTERP_HYPER);
 		as_pixbuf_sharpen (pixbuf, 1, -0.5);
 		return pixbuf;
@@ -166,15 +167,14 @@ asb_app_load_icon (AsbApp *app,
 
 	/* create new square pixbuf with alpha padding */
 	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-				 AS_APP_ICON_DEFAULT_WIDTH,
-				 AS_APP_ICON_DEFAULT_HEIGHT);
+				 icon_size, icon_size);
 	gdk_pixbuf_fill (pixbuf, 0x00000000);
 	if (pixbuf_width > pixbuf_height) {
-		tmp_width = AS_APP_ICON_DEFAULT_WIDTH;
-		tmp_height = AS_APP_ICON_DEFAULT_HEIGHT * pixbuf_height / pixbuf_width;
+		tmp_width = icon_size;
+		tmp_height = icon_size * pixbuf_height / pixbuf_width;
 	} else {
-		tmp_width = AS_APP_ICON_DEFAULT_WIDTH * pixbuf_width / pixbuf_height;
-		tmp_height = AS_APP_ICON_DEFAULT_HEIGHT;
+		tmp_width = icon_size * pixbuf_width / pixbuf_height;
+		tmp_height = icon_size;
 	}
 	pixbuf_tmp = gdk_pixbuf_scale_simple (pixbuf_src, tmp_width, tmp_height,
 					      GDK_INTERP_HYPER);
@@ -183,29 +183,121 @@ asb_app_load_icon (AsbApp *app,
 			      0, 0, /* of src */
 			      tmp_width, tmp_height,
 			      pixbuf,
-			      (AS_APP_ICON_DEFAULT_WIDTH - tmp_width) / 2,
-			      (AS_APP_ICON_DEFAULT_HEIGHT - tmp_height) / 2);
+			      (icon_size - tmp_width) / 2,
+			      (icon_size - tmp_height) / 2);
 	return pixbuf;
 }
 
 /**
- * asb_app_find_icon:
+ * asb_plugin_desktop_add_icons:
  */
-static GdkPixbuf *
-asb_app_find_icon (AsbApp *app,
-		   const gchar *tmpdir,
-		   const gchar *something,
-		   GError **error)
+static gboolean
+asb_plugin_desktop_add_icons (AsbPlugin *plugin,
+			      AsbApp *app,
+			      const gchar *tmpdir,
+			      const gchar *key,
+			      GError **error)
 {
+	guint min_icon_size;
+	_cleanup_free_ gchar *fn_hidpi = NULL;
 	_cleanup_free_ gchar *fn = NULL;
+	_cleanup_free_ gchar *name_hidpi = NULL;
+	_cleanup_free_ gchar *name = NULL;
+	_cleanup_object_unref_ AsIcon *icon_hidpi = NULL;
+	_cleanup_object_unref_ AsIcon *icon = NULL;
+	_cleanup_object_unref_ GdkPixbuf *pixbuf_hidpi = NULL;
+	_cleanup_object_unref_ GdkPixbuf *pixbuf = NULL;
 
-	/* find file */
-	fn = as_utils_find_icon_filename (tmpdir, something, error);
-	if (fn == NULL)
-		return NULL;
+	/* find 64x64 icon */
+	fn = as_utils_find_icon_filename_full (tmpdir, key,
+					       AS_UTILS_FIND_ICON_NONE,
+					       error);
+	if (fn == NULL) {
+		g_prefix_error (error, "Failed to find icon: ");
+		return FALSE;
+	}
+
+	/* is icon in a unsupported format */
+	if (g_str_has_suffix (fn, ".xpm")) {
+		g_set_error (error,
+			     ASB_PLUGIN_ERROR,
+			     ASB_PLUGIN_ERROR_NOT_SUPPORTED,
+			     "Uses XPM icon: %s", key);
+		return FALSE;
+	}
+	if (g_str_has_suffix (fn, ".gif")) {
+		g_set_error (error,
+			     ASB_PLUGIN_ERROR,
+			     ASB_PLUGIN_ERROR_NOT_SUPPORTED,
+			     "Uses GIF icon: %s", key);
+		return FALSE;
+	}
+	if (g_str_has_suffix (fn, ".ico")) {
+		g_set_error (error,
+			     ASB_PLUGIN_ERROR,
+			     ASB_PLUGIN_ERROR_NOT_SUPPORTED,
+			     "Uses ICO icon: %s", key);
+		return FALSE;
+	}
 
 	/* load the icon */
-	return asb_app_load_icon (app, fn, fn + strlen (tmpdir), error);
+	min_icon_size = asb_context_get_min_icon_size (plugin->ctx);
+	pixbuf = asb_app_load_icon (app, fn, fn + strlen (tmpdir),
+				    64, min_icon_size, error);
+	if (pixbuf == NULL) {
+		g_prefix_error (error, "Failed to load icon: ");
+		return FALSE;
+	}
+
+	/* save in target directory */
+	if (asb_context_get_hidpi_enabled (plugin->ctx)) {
+		name = g_strdup_printf ("%ix%i/%s.png",
+					64, 64,
+					as_app_get_id_filename (AS_APP (app)));
+	} else {
+		name = g_strdup_printf ("%s.png",
+					as_app_get_id_filename (AS_APP (app)));
+	}
+	icon = as_icon_new ();
+	as_icon_set_pixbuf (icon, pixbuf);
+	as_icon_set_name (icon, name, -1);
+	as_icon_set_kind (icon, AS_ICON_KIND_CACHED);
+	as_icon_set_prefix (icon, as_app_get_icon_path (AS_APP (app)));
+	as_app_add_icon (AS_APP (app), icon);
+
+	/* is HiDPI disabled */
+	if (!asb_context_get_hidpi_enabled (plugin->ctx))
+		return TRUE;
+
+	/* try to get a HiDPI icon */
+	fn_hidpi = as_utils_find_icon_filename_full (tmpdir, key,
+						     AS_UTILS_FIND_ICON_HI_DPI,
+						     NULL);
+	if (fn_hidpi == NULL)
+		return TRUE;
+
+	/* load the HiDPI icon */
+	pixbuf_hidpi = asb_app_load_icon (app, fn_hidpi,
+					  fn_hidpi + strlen (tmpdir),
+					  128, 128, NULL);
+	if (pixbuf_hidpi == NULL)
+		return TRUE;
+	if (gdk_pixbuf_get_width (pixbuf_hidpi) <= gdk_pixbuf_get_width (pixbuf) ||
+	    gdk_pixbuf_get_height (pixbuf_hidpi) <= gdk_pixbuf_get_height (pixbuf))
+		return TRUE;
+	as_app_add_kudo_kind (AS_APP (app), AS_KUDO_KIND_HI_DPI_ICON);
+
+	/* save icon */
+	name_hidpi = g_strdup_printf ("%ix%i/%s.png",
+				      128, 128,
+				      as_app_get_id_filename (AS_APP (app)));
+	icon_hidpi = as_icon_new ();
+	as_icon_set_pixbuf (icon_hidpi, pixbuf_hidpi);
+	as_icon_set_name (icon_hidpi, name_hidpi, -1);
+	as_icon_set_kind (icon_hidpi, AS_ICON_KIND_CACHED);
+	as_icon_set_prefix (icon_hidpi, as_app_get_icon_path (AS_APP (app)));
+	as_app_add_icon (AS_APP (app), icon_hidpi);
+	return TRUE;
 }
 
 /**
@@ -219,17 +311,17 @@ asb_plugin_process_filename (AsbPlugin *plugin,
 			     const gchar *tmpdir,
 			     GError **error)
 {
-	const gchar *key;
+	AsIcon *icon;
 	gboolean ret;
 	_cleanup_free_ gchar *app_id = NULL;
 	_cleanup_free_ gchar *full_filename = NULL;
-	_cleanup_free_ gchar *icon_filename = NULL;
 	_cleanup_object_unref_ AsbApp *app = NULL;
 	_cleanup_object_unref_ GdkPixbuf *pixbuf = NULL;
 
 	/* create app */
 	app_id = g_path_get_basename (filename);
 	app = asb_app_new (pkg, app_id);
+	asb_app_set_hidpi_enabled (app, asb_context_get_hidpi_enabled (plugin->ctx));
 	full_filename = g_build_filename (tmpdir, filename, NULL);
 	ret = as_app_parse_file (AS_APP (app),
 				 full_filename,
@@ -240,7 +332,7 @@ asb_plugin_process_filename (AsbPlugin *plugin,
 
 	/* NoDisplay apps are never included */
 	if (as_app_get_metadata_item (AS_APP (app), "NoDisplay") != NULL)
-		as_app_add_veto (AS_APP (app), "NoDisplay=true");
+		asb_app_add_requires_appdata (app, "NoDisplay=true");
 
 	/* Settings or DesktopSettings requires AppData */
 	if (as_app_has_category (AS_APP (app), "Settings"))
@@ -249,35 +341,25 @@ asb_plugin_process_filename (AsbPlugin *plugin,
 		asb_app_add_requires_appdata (app, "Category=DesktopSettings");
 
 	/* is the icon a stock-icon-name? */
-	key = as_app_get_icon (AS_APP (app));
-	if (key != NULL) {
-		if (as_app_get_icon_kind (AS_APP (app)) == AS_ICON_KIND_STOCK) {
+	icon = as_app_get_icon_default (AS_APP (app));
+	if (icon != NULL) {
+		_cleanup_free_ gchar *key = NULL;
+		key = g_strdup (as_icon_get_name (icon));
+		if (as_icon_get_kind (icon) == AS_ICON_KIND_STOCK) {
 			asb_package_log (pkg,
 					 ASB_PACKAGE_LOG_LEVEL_DEBUG,
 					 "using stock icon %s", key);
 		} else {
 			_cleanup_error_free_ GError *error_local = NULL;
-
-			/* is icon XPM or GIF */
-			if (g_str_has_suffix (key, ".xpm"))
-				as_app_add_veto (AS_APP (app), "Uses XPM icon: %s", key);
-			else if (g_str_has_suffix (key, ".gif"))
-				as_app_add_veto (AS_APP (app), "Uses GIF icon: %s", key);
-			else if (g_str_has_suffix (key, ".ico"))
-				as_app_add_veto (AS_APP (app), "Uses ICO icon: %s", key);
-
-			/* find icon */
-			pixbuf = asb_app_find_icon (app, tmpdir, key, &error_local);
-			if (pixbuf == NULL) {
-				as_app_add_veto (AS_APP (app), "Failed to find icon: %s",
+			g_ptr_array_set_size (as_app_get_icons (AS_APP (app)), 0);
+			ret = asb_plugin_desktop_add_icons (plugin,
+							    app,
+							    tmpdir,
+							    key,
+							    &error_local);
+			if (!ret) {
+				as_app_add_veto (AS_APP (app), "%s",
 						 error_local->message);
-			} else {
-				/* save in target directory */
-				icon_filename = g_strdup_printf ("%s.png",
-								 as_app_get_id_filename (AS_APP (app)));
-				as_app_set_icon (AS_APP (app), icon_filename, -1);
-				as_app_set_icon_kind (AS_APP (app), AS_ICON_KIND_CACHED);
-				asb_app_set_pixbuf (app, pixbuf);
 			}
 		}
 	}
