@@ -54,7 +54,7 @@ ai_app_validate_add (GPtrArray *problems,
 	AsProblem *problem;
 	guint i;
 	va_list args;
-	_cleanup_free_ gchar *str;
+	_cleanup_free_ gchar *str = NULL;
 
 	va_start (args, fmt);
 	str = g_strdup_vprintf (fmt, args);
@@ -186,28 +186,29 @@ as_app_validate_description_li (const gchar *text, AsAppValidateHelper *helper)
 	if (str_len < length_li_min) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<li> is too short");
+				     "<li> is too short [%s]", text);
 	}
 	if (str_len > length_li_max) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<li> is too long");
+				     "<li> is too long [%s]", text);
 	}
 	if (ai_app_validate_fullstop_ending (text)) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<li> cannot end in '.'");
+				     "<li> cannot end in '.' [%s]", text);
 	}
 	if (as_app_validate_has_hyperlink (text)) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<li> cannot contain a hyperlink");
+				     "<li> cannot contain a hyperlink [%s]",
+				     text);
 	}
 	if (require_sentence_case &&
 	    !as_app_validate_has_first_word_capital (helper, text)) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<li> requires sentence case");
+				     "<li> requires sentence case [%s]", text);
 	}
 }
 
@@ -241,7 +242,7 @@ as_app_validate_description_para (const gchar *text, AsAppValidateHelper *helper
 	if (helper->previous_para_was_short) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<p> is too short [p]");
+				     "<p> is too short [%s]", text);
 	}
 	helper->previous_para_was_short = FALSE;
 
@@ -254,7 +255,7 @@ as_app_validate_description_para (const gchar *text, AsAppValidateHelper *helper
 	if (str_len > length_para_max) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<p> is too long");
+				     "<p> is too long [%s]", text);
 	}
 	if (g_str_has_prefix (text, "This application")) {
 		ai_app_validate_add (helper->probs,
@@ -264,20 +265,21 @@ as_app_validate_description_para (const gchar *text, AsAppValidateHelper *helper
 	if (as_app_validate_has_hyperlink (text)) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<p> cannot contain a hyperlink");
+				     "<p> cannot contain a hyperlink [%s]",
+				     text);
 	}
 	if (require_sentence_case &&
 	    !as_app_validate_has_first_word_capital (helper, text)) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<p> requires sentence case");
+				     "<p> requires sentence case [%s]", text);
 	}
 	if (text[str_len - 1] != '.' &&
 	    text[str_len - 1] != '!' &&
 	    text[str_len - 1] != ':') {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<p> does not end in '.|:|!'");
+				     "<p> does not end in '.|:|!' [%s]", text);
 	}
 	helper->number_paragraphs++;
 	helper->para_chars_before_list += str_len;
@@ -300,7 +302,8 @@ as_app_validate_description_list (const gchar *text, AsAppValidateHelper *helper
 	if (helper->number_paragraphs < 1) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<ul> cannot start a description");
+				     "<ul> cannot start a description [%s]",
+				     text);
 	}
 	if (helper->para_chars_before_list != 0 &&
 	    helper->para_chars_before_list < (guint) length_para_before_list) {
@@ -327,7 +330,7 @@ as_app_validate_description (const gchar *xml,
 {
 	GNode *l;
 	GNode *l2;
-	_cleanup_node_unref_ GNode *node;
+	_cleanup_node_unref_ GNode *node = NULL;
 
 	/* parse xml */
 	node = as_node_from_xml (xml, -1,
@@ -459,7 +462,7 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	if (base_uri == NULL) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_URL_NOT_FOUND,
-				     "<screenshot> url '%s' not valid", url);
+				     "<screenshot> url not valid [%s]", url);
 		return FALSE;
 	}
 	msg = soup_message_new_from_uri (SOUP_METHOD_GET, base_uri);
@@ -473,7 +476,7 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	if (status_code != SOUP_STATUS_OK) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_URL_NOT_FOUND,
-				     "<screenshot> url '%s' not found", url);
+				     "<screenshot> url not found [%s]", url);
 		return FALSE;
 	}
 
@@ -481,7 +484,8 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	if (msg->response_body->length == 0) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_FILE_INVALID,
-				     "<screenshot> url '%s' is a zero length file", url);
+				     "<screenshot> url is a zero length file [%s]",
+				     url);
 		return FALSE;
 	}
 
@@ -492,7 +496,8 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	if (stream == NULL) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_URL_NOT_FOUND,
-				     "<screenshot> failed to load data from '%s'", url);
+				     "<screenshot> failed to load data [%s]",
+				     url);
 		return FALSE;
 	}
 
@@ -501,7 +506,7 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	if (pixbuf == NULL) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_FILE_INVALID,
-				     "<screenshot> failed to load '%s'",
+				     "<screenshot> failed to load [%s]",
 				     url);
 		return FALSE;
 	}
@@ -513,7 +518,8 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	    as_image_get_width (im) != screenshot_width) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
-				     "<screenshot> width did not match specified");
+				     "<screenshot> width did not match specified [%s]",
+				     url);
 	}
 
 	/* check height matches */
@@ -521,29 +527,34 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	    as_image_get_height (im) != screenshot_height) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
-				     "<screenshot> height did not match specified");
+				     "<screenshot> height did not match specified [%s]",
+				     url);
 	}
 
 	/* check size is reasonable */
 	if (screenshot_width < ss_size_width_min) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
-				     "<screenshot> width was too small");
+				     "<screenshot> width too small [%s]",
+				     url);
 	}
 	if (screenshot_height < ss_size_height_min) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
-				     "<screenshot> height was too small");
+				     "<screenshot> height too small [%s]",
+				     url);
 	}
 	if (screenshot_width > ss_size_width_max) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
-				     "<screenshot> width was too large");
+				     "<screenshot> width too large [%s]",
+				     url);
 	}
 	if (screenshot_height > ss_size_height_max) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
-				     "<screenshot> height was too large");
+				     "<screenshot> height too large [%s]",
+				     url);
 	}
 
 	/* check padding */
@@ -553,13 +564,15 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	    (alpha_flags & AS_IMAGE_ALPHA_FLAG_BOTTOM) > 0) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<image> has vertical alpha padding");
+				     "<image> has vertical padding [%s]",
+				     url);
 	}
 	if ((alpha_flags & AS_IMAGE_ALPHA_FLAG_LEFT) > 0||
 	    (alpha_flags & AS_IMAGE_ALPHA_FLAG_RIGHT) > 0) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<image> has horizontal alpha padding");
+				     "<image> has horizontal padding [%s]",
+				     url);
 	}
 
 	/* check aspect ratio */
@@ -570,7 +583,8 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 				 screenshot_aspect, desired_aspect);
 			ai_app_validate_add (helper->probs,
 					     AS_PROBLEM_KIND_ASPECT_RATIO_INCORRECT,
-					     "<screenshot> aspect ratio was not 16:9");
+					     "<screenshot> aspect ratio not 16:9 [%s]",
+					     url);
 		}
 	}
 	return TRUE;
@@ -647,32 +661,92 @@ as_app_validate_screenshot (AsScreenshot *ss, AsAppValidateHelper *helper)
 		if (str_len < length_caption_min) {
 			ai_app_validate_add (helper->probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<caption> is too short");
+					     "<caption> is too short [%s]", tmp);
 		}
 		if (str_len > length_caption_max) {
 			ai_app_validate_add (helper->probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<caption> is too long");
+					     "<caption> is too long [%s]", tmp);
 		}
 		if (ai_app_validate_fullstop_ending (tmp)) {
 			ai_app_validate_add (helper->probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<caption> cannot end in '.'");
+					     "<caption> cannot end in '.' [%s]",
+					     tmp);
 		}
 		if (as_app_validate_has_hyperlink (tmp)) {
 			ai_app_validate_add (helper->probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<caption> cannot contain a hyperlink");
+					     "<caption> cannot contain a hyperlink [%s]",
+					     tmp);
 		}
 		if (require_sentence_case &&
 		    !as_app_validate_has_first_word_capital (helper, tmp)) {
 			ai_app_validate_add (helper->probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<caption> requires sentence case");
+					     "<caption> requires sentence case [%s]",
+					     tmp);
 		}
 	}
 }
 
+/**
+ * as_app_validate_icons:
+ **/
+static void
+as_app_validate_icons (AsApp *app, AsAppValidateHelper *helper)
+{
+	AsIcon *icon;
+	AsIconKind icon_kind;
+	const gchar *icon_name;
+
+	/* just check the default icon */
+	icon = as_app_get_icon_default (app);
+	if (icon == NULL)
+		return;
+
+	/* check the content is correct */
+	icon_kind = as_icon_get_kind (icon);
+	icon_name = as_icon_get_name (icon);
+	switch (icon_kind) {
+	case AS_ICON_KIND_STOCK:
+		if (!as_utils_is_stock_icon_name (icon_name)) {
+			ai_app_validate_add (helper->probs,
+					     AS_PROBLEM_KIND_TAG_INVALID,
+					     "stock icon is not valid [%s]",
+					     icon_name);
+		}
+		break;
+	case AS_ICON_KIND_LOCAL:
+		if (!g_str_has_prefix (icon_name, "/")) {
+			ai_app_validate_add (helper->probs,
+					     AS_PROBLEM_KIND_TAG_INVALID,
+					     "local icon is not a filename [%s]",
+					     icon_name);
+		}
+		break;
+	case AS_ICON_KIND_CACHED:
+		if (g_str_has_prefix (icon_name, "/")) {
+			ai_app_validate_add (helper->probs,
+					     AS_PROBLEM_KIND_TAG_INVALID,
+					     "cached icon is a filename [%s]",
+					     icon_name);
+		}
+		break;
+	case AS_ICON_KIND_REMOTE:
+		if (!g_str_has_prefix (icon_name, "http://") &&
+		    !g_str_has_prefix (icon_name, "https://")) {
+			ai_app_validate_add (helper->probs,
+					     AS_PROBLEM_KIND_TAG_INVALID,
+					     "remote icon is not a url [%s]",
+					     icon_name);
+		}
+		break;
+	default:
+		break;
+	}
+}
+
 /**
  * as_app_validate_screenshots:
  **/
@@ -762,7 +836,7 @@ as_app_validate_release (AsRelease *release, AsAppValidateHelper *helper, GError
 				     AS_PROBLEM_KIND_ATTRIBUTE_MISSING,
 				     "<release> has no timestamp");
 	}
-	if (timestamp > 20120101 && timestamp < 20151231) {
+	if (timestamp > 20120101 && timestamp < 20251231) {
 		ai_app_validate_add (helper->probs,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
 				     "<release> timestamp should be a UNIX time");
@@ -779,7 +853,8 @@ as_app_validate_release (AsRelease *release, AsAppValidateHelper *helper, GError
 			ai_app_validate_add (helper->probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<release> description should be "
-					     "prose and not contain hyperlinks");
+					     "prose and not contain hyperlinks [%s]",
+					     tmp);
 		}
 		if (!as_app_validate_description (tmp,
 						  helper,
@@ -885,8 +960,16 @@ as_app_validate_is_content_license (const gchar *license)
 			continue;
 		if (g_strcmp0 (tokens[i], "@CC-BY-3.0") == 0)
 			continue;
+		if (g_strcmp0 (tokens[i], "@CC-BY-4.0") == 0)
+			continue;
 		if (g_strcmp0 (tokens[i], "@CC-BY-SA-3.0") == 0)
 			continue;
+		if (g_strcmp0 (tokens[i], "@CC-BY-SA-4.0") == 0)
+			continue;
+		if (g_strcmp0 (tokens[i], "@GFDL-1.1") == 0)
+			continue;
+		if (g_strcmp0 (tokens[i], "@GFDL-1.2") == 0)
+			continue;
 		if (g_strcmp0 (tokens[i], "@GFDL-1.3") == 0)
 			continue;
 		if (g_strcmp0 (tokens[i], "&") == 0)
@@ -933,6 +1016,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	gboolean require_translations = FALSE;
 	gboolean require_url = TRUE;
 	gboolean require_content_license = TRUE;
+	gboolean require_name = TRUE;
 	gboolean validate_license = TRUE;
 	gboolean ret;
 	guint length_name_max = 30;
@@ -955,6 +1039,14 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		number_para_max = 10;
 		number_para_min = 1;
 		require_sentence_case = FALSE;
+		switch (as_app_get_source_kind (app)) {
+		case AS_APP_SOURCE_KIND_METAINFO:
+		case AS_APP_SOURCE_KIND_APPDATA:
+			require_name = FALSE;
+			break;
+		default:
+			break;
+		}
 	}
 
 	/* make the requirements more strict */
@@ -986,15 +1078,10 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	id = as_app_get_id (app);
 	switch (as_app_get_id_kind (app)) {
 	case AS_ID_KIND_DESKTOP:
+	case AS_ID_KIND_WEB_APP:
 		if (g_str_has_suffix (id, ".desktop"))
 			ret = TRUE;
 		break;
-	case AS_ID_KIND_FONT:
-		if (g_str_has_suffix (id, ".ttf"))
-			ret = TRUE;
-		else if (g_str_has_suffix (id, ".otf"))
-			ret = TRUE;
-		break;
 	case AS_ID_KIND_INPUT_METHOD:
 		if (g_str_has_suffix (id, ".xml"))
 			ret = TRUE;
@@ -1011,7 +1098,9 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 				     "<id> has invalid type attribute");
 
 		break;
+	case AS_ID_KIND_FONT:
 	case AS_ID_KIND_ADDON:
+	case AS_ID_KIND_SOURCE:
 		/* anything goes */
 		ret = TRUE;
 	default:
@@ -1030,12 +1119,14 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		    !as_app_validate_is_content_license (license)) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_TAG_INVALID,
-					     "<metadata_license> is not valid");
+					     "<metadata_license> is not valid [%s]",
+					     license);
 		} else if (validate_license) {
 			ret = as_app_validate_license (license, &error_local);
 			if (!ret) {
 				g_prefix_error (&error_local,
-						"<metadata_license> is not valid: ");
+						"<metadata_license> is not valid [%s]",
+						license);
 				ai_app_validate_add (probs,
 						     AS_PROBLEM_KIND_TAG_INVALID,
 						     "%s", error_local->message);
@@ -1062,7 +1153,8 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		ret = as_app_validate_license (license, &error_local);
 		if (!ret) {
 			g_prefix_error (&error_local,
-					"<project_license> is not valid: ");
+					"<project_license> is not valid [%s]",
+					license);
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_TAG_INVALID,
 					     "%s", error_local->message);
@@ -1092,13 +1184,14 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 
 	/* extends */
 	if (as_app_get_extends(app)->len == 0 &&
+	    as_app_get_id_kind (app) == AS_ID_KIND_ADDON &&
 	    as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_METAINFO) {
 		ai_app_validate_add (probs,
 				     AS_PROBLEM_KIND_TAG_MISSING,
 				     "<extends> is not present");
 	}
 
-	/* updatecontact */
+	/* update_contact */
 	update_contact = as_app_get_update_contact (app);
 	if (g_strcmp0 (update_contact,
 		       "someone_who_cares@upstream_project.org") == 0) {
@@ -1109,7 +1202,8 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 	if (update_contact != NULL && strlen (update_contact) < 6) {
 		ai_app_validate_add (probs,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
-				     "<update_contact> is too short");
+				     "<update_contact> is too short [%s]",
+				     update_contact);
 	}
 	if (require_contactdetails && update_contact == NULL) {
 		switch (as_app_get_source_kind (app)) {
@@ -1117,7 +1211,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		case AS_APP_SOURCE_KIND_METAINFO:
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_TAG_MISSING,
-					     "<updatecontact> is not present");
+					     "<update_contact> is not present");
 			break;
 		default:
 			break;
@@ -1156,7 +1250,7 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		if (g_strcmp0 (key, "unknown") == 0) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_TAG_INVALID,
-					     "<url> type invalid");
+					     "<url> type invalid [%s]", key);
 		}
 		tmp = g_hash_table_lookup (urls, key);
 		if (tmp == NULL || tmp[0] == '\0')
@@ -1165,13 +1259,17 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		    !g_str_has_prefix (tmp, "https://")) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_TAG_INVALID,
-					     "<url> does not start with 'http://'");
+					     "<url> does not start with 'http://' [%s]",
+					     tmp);
 		}
 	}
 
 	/* screenshots */
 	as_app_validate_screenshots (app, &helper);
 
+	/* icons */
+	as_app_validate_icons (app, &helper);
+
 	/* releases */
 	ret = as_app_validate_releases (app, &helper, error);
 	if (!ret)
@@ -1184,30 +1282,33 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		if (str_len < length_name_min) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<name> is too short");
+					     "<name> is too short [%s]", name);
 		}
 		if (str_len > length_name_max) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<name> is too long");
+					     "<name> is too long [%s]", name);
 		}
 		if (ai_app_validate_fullstop_ending (name)) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<name> cannot end in '.'");
+					     "<name> cannot end in '.' [%s]",
+					     name);
 		}
 		if (as_app_validate_has_hyperlink (name)) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<name> cannot contain a hyperlink");
+					     "<name> cannot contain a hyperlink [%s]",
+					     name);
 		}
 		if (require_sentence_case &&
 		    !as_app_validate_has_first_word_capital (&helper, name)) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<name> requires sentence case");
+					     "<name> requires sentence case [%s]",
+					     name);
 		}
-	} else if (as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_METAINFO) {
+	} else if (require_name) {
 		ai_app_validate_add (probs,
 				     AS_PROBLEM_KIND_TAG_MISSING,
 				     "<name> is not present");
@@ -1220,30 +1321,35 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		if (str_len < length_summary_min) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<summary> is too short");
+					     "<summary> is too short [%s]",
+					     summary);
 		}
 		if (str_len > length_summary_max) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<summary> is too long");
+					     "<summary> is too long [%s]",
+					     summary);
 		}
 		if (ai_app_validate_fullstop_ending (summary)) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<summary> cannot end in '.'");
+					     "<summary> cannot end in '.' [%s]",
+					     summary);
 		}
 		if (as_app_validate_has_hyperlink (summary)) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<summary> cannot contain a hyperlink");
+					     "<summary> cannot contain a hyperlink [%s]",
+					     summary);
 		}
 		if (require_sentence_case &&
 		    !as_app_validate_has_first_word_capital (&helper, summary)) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<summary> requires sentence case");
+					     "<summary> requires sentence case [%s]",
+					     summary);
 		}
-	} else if (as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_METAINFO) {
+	} else if (require_name) {
 		ai_app_validate_add (probs,
 				     AS_PROBLEM_KIND_TAG_MISSING,
 				     "<summary> is not present");
@@ -1299,22 +1405,26 @@ as_app_validate (AsApp *app, AsAppValidateFlags flags, GError **error)
 		if (str_len < length_name_min) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<developer_name> is too short");
+					     "<developer_name> is too short [%s]",
+					     name);
 		}
 		if (str_len > length_name_max) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<developer_name> is too long");
+					     "<developer_name> is too long [%s]",
+					     name);
 		}
 		if (as_app_validate_has_hyperlink (name)) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<developer_name> cannot contain a hyperlink");
+					     "<developer_name> cannot contain a hyperlink [%s]",
+					     name);
 		}
 		if (as_app_validate_has_email (name)) {
 			ai_app_validate_add (probs,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
-					     "<developer_name> cannot contain an email address");
+					     "<developer_name> cannot contain an email address [%s]",
+					     name);
 		}
 	}
 
