@@ -92,50 +92,6 @@ as_id_kind_from_string (const gchar *id_kind)
 	return AS_ID_KIND_UNKNOWN;
 }
 
-/**
- * as_icon_kind_to_string:
- * @icon_kind: the @AsIconKind.
- *
- * Converts the enumerated value to an text representation.
- *
- * Returns: string version of @icon_kind
- *
- * Since: 0.1.0
- **/
-const gchar *
-as_icon_kind_to_string (AsIconKind icon_kind)
-{
-	if (icon_kind == AS_ICON_KIND_CACHED)
-		return "cached";
-	if (icon_kind == AS_ICON_KIND_STOCK)
-		return "stock";
-	if (icon_kind == AS_ICON_KIND_REMOTE)
-		return "remote";
-	return "unknown";
-}
-
-/**
- * as_icon_kind_from_string:
- * @icon_kind: the string.
- *
- * Converts the text representation to an enumerated value.
- *
- * Returns: a #AsIconKind or %AS_ICON_KIND_UNKNOWN for unknown
- *
- * Since: 0.1.0
- **/
-AsIconKind
-as_icon_kind_from_string (const gchar *icon_kind)
-{
-	if (g_strcmp0 (icon_kind, "cached") == 0)
-		return AS_ICON_KIND_CACHED;
-	if (g_strcmp0 (icon_kind, "stock") == 0)
-		return AS_ICON_KIND_STOCK;
-	if (g_strcmp0 (icon_kind, "remote") == 0)
-		return AS_ICON_KIND_REMOTE;
-	return AS_ICON_KIND_UNKNOWN;
-}
-
 /**
  * as_url_kind_to_string:
  * @url_kind: the @AsUrlKind.
@@ -159,6 +115,8 @@ as_url_kind_to_string (AsUrlKind url_kind)
 		return "donation";
 	if (url_kind == AS_URL_KIND_HELP)
 		return "help";
+	if (url_kind == AS_URL_KIND_MISSING)
+		return "missing";
 	return "unknown";
 }
 
@@ -185,6 +143,8 @@ as_url_kind_from_string (const gchar *url_kind)
 		return AS_URL_KIND_DONATION;
 	if (g_strcmp0 (url_kind, "help") == 0)
 		return AS_URL_KIND_HELP;
+	if (g_strcmp0 (url_kind, "missing") == 0)
+		return AS_URL_KIND_MISSING;
 	return AS_URL_KIND_UNKNOWN;
 }
 
@@ -213,6 +173,8 @@ as_kudo_kind_to_string (AsKudoKind kudo_kind)
 		return "Notifications";
 	if (kudo_kind == AS_KUDO_KIND_HIGH_CONTRAST)
 		return "HighContrast";
+	if (kudo_kind == AS_KUDO_KIND_HI_DPI_ICON)
+		return "HiDpiIcon";
 	return NULL;
 }
 
@@ -241,5 +203,7 @@ as_kudo_kind_from_string (const gchar *kudo_kind)
 		return AS_KUDO_KIND_NOTIFICATIONS;
 	if (g_strcmp0 (kudo_kind, "HighContrast") == 0)
 		return AS_KUDO_KIND_HIGH_CONTRAST;
+	if (g_strcmp0 (kudo_kind, "HiDpiIcon") == 0)
+		return AS_KUDO_KIND_HI_DPI_ICON;
 	return AS_KUDO_KIND_UNKNOWN;
 }
