@@ -56,24 +56,6 @@ typedef enum {
 	AS_ID_KIND_LAST
 } AsIdKind;
 
-/**
- * AsIconKind:
- * @AS_ICON_KIND_UNKNOWN:		Type invalid or not known
- * @AS_ICON_KIND_STOCK:			Stock icon or present in the generic icon theme
- * @AS_ICON_KIND_CACHED:		An icon shipped with the AppStream metadata
- * @AS_ICON_KIND_REMOTE:		An icon referenced by a remote URL
- *
- * The icon type.
- **/
-typedef enum {
-	AS_ICON_KIND_UNKNOWN,		/* Since: 0.1.0 */
-	AS_ICON_KIND_STOCK,		/* Since: 0.1.0 */
-	AS_ICON_KIND_CACHED,		/* Since: 0.1.0 */
-	AS_ICON_KIND_REMOTE,		/* Since: 0.1.0 */
-	/*< private >*/
-	AS_ICON_KIND_LAST
-} AsIconKind;
-
 /**
  * AsUrlKind:
  * @AS_URL_KIND_UNKNOWN:		Type invalid or not known
@@ -107,6 +89,7 @@ typedef enum {
  * @AS_KUDO_KIND_MODERN_TOOLKIT:	Uses a modern toolkit like GTK3 or QT5
  * @AS_KUDO_KIND_NOTIFICATIONS:		Registers notifications with KDE or GNOME
  * @AS_KUDO_KIND_HIGH_CONTRAST:		Installs a high contrast icon
+ * @AS_KUDO_KIND_HI_DPI_ICON:		Installs a high DPI icon
  *
  * The kudo type.
  **/
@@ -118,6 +101,7 @@ typedef enum {
 	AS_KUDO_KIND_MODERN_TOOLKIT,	/* Since: 0.2.2 */
 	AS_KUDO_KIND_NOTIFICATIONS,	/* Since: 0.2.2 */
 	AS_KUDO_KIND_HIGH_CONTRAST,	/* Since: 0.3.0 */
+	AS_KUDO_KIND_HI_DPI_ICON,	/* Since: 0.3.1 */
 	/*< private >*/
 	AS_KUDO_KIND_LAST
 } AsKudoKind;
@@ -125,9 +109,6 @@ typedef enum {
 const gchar	*as_id_kind_to_string		(AsIdKind	 id_kind);
 AsIdKind	 as_id_kind_from_string		(const gchar	*id_kind);
 
-const gchar	*as_icon_kind_to_string		(AsIconKind	 icon_kind);
-AsIconKind	 as_icon_kind_from_string	(const gchar	*icon_kind);
-
 const gchar	*as_url_kind_to_string		(AsUrlKind	 url_kind);
 AsUrlKind	 as_url_kind_from_string	(const gchar	*url_kind);
 
