@@ -432,7 +432,9 @@ as_image_node_parse (AsImage *image, GNode *node, GError **error)
 	if (size != G_MAXINT)
 		as_image_set_height (image, size);
 	tmp = as_node_get_attribute (node, "type");
-	if (tmp != NULL)
+	if (tmp == NULL)
+		as_image_set_kind (image, AS_IMAGE_KIND_SOURCE);
+	else
 		as_image_set_kind (image, as_image_kind_from_string (tmp));
 	taken = as_node_take_data (node);
 	if (taken != NULL) {
@@ -545,6 +547,10 @@ as_image_save_pixbuf (AsImage *image,
 	guint pixbuf_width;
 	_cleanup_object_unref_ GdkPixbuf *pixbuf_tmp = NULL;
 
+	/* never set */
+	if (priv->pixbuf == NULL)
+		return NULL;
+
 	/* 0 means 'default' */
 	if (width == 0)
 		width = gdk_pixbuf_get_width (priv->pixbuf);
@@ -558,7 +564,8 @@ as_image_save_pixbuf (AsImage *image,
 		return g_object_ref (priv->pixbuf);
 
 	/* never scale up, just pad */
-	if (pixbuf_width < width && pixbuf_height < height) {
+	if (pixbuf_width < width && pixbuf_height < height &&
+	    (flags & AS_IMAGE_SAVE_FLAG_BLUR) == 0) {
 		pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
 					 TRUE, 8, width, height);
 		gdk_pixbuf_fill (pixbuf, 0x00000000);
@@ -577,7 +584,10 @@ as_image_save_pixbuf (AsImage *image,
 		pixbuf = gdk_pixbuf_scale_simple (priv->pixbuf,
 						  width, height,
 						  GDK_INTERP_HYPER);
-		as_pixbuf_sharpen (pixbuf, 1, -0.5);
+		if ((flags & AS_IMAGE_SAVE_FLAG_SHARPEN) > 0)
+			as_pixbuf_sharpen (pixbuf, 1, -0.5);
+		if ((flags & AS_IMAGE_SAVE_FLAG_BLUR) > 0)
+			as_pixbuf_blur (pixbuf, 5, 3);
 		return pixbuf;
 	}
 
@@ -597,7 +607,10 @@ as_image_save_pixbuf (AsImage *image,
 	pixbuf_tmp = gdk_pixbuf_scale_simple (priv->pixbuf,
 					      tmp_width, tmp_height,
 					      GDK_INTERP_HYPER);
-	as_pixbuf_sharpen (pixbuf_tmp, 1, -0.5);
+	if ((flags & AS_IMAGE_SAVE_FLAG_SHARPEN) > 0)
+		as_pixbuf_sharpen (pixbuf_tmp, 1, -0.5);
+	if ((flags & AS_IMAGE_SAVE_FLAG_BLUR) > 0)
+		as_pixbuf_blur (pixbuf_tmp, 5, 3);
 	gdk_pixbuf_copy_area (pixbuf_tmp,
 			      0, 0, /* of src */
 			      tmp_width, tmp_height,
@@ -630,7 +643,7 @@ as_image_save_filename (AsImage *image,
 		        AsImageSaveFlags flags,
 		        GError **error)
 {
-	_cleanup_object_unref_ GdkPixbuf *pixbuf;
+	_cleanup_object_unref_ GdkPixbuf *pixbuf = NULL;
 
 	/* save source file */
 	pixbuf = as_image_save_pixbuf (image, width, height, flags);
