@@ -364,15 +364,23 @@ as_node_to_xml_string (GString *xml,
 	/* comment */
 	comment = as_node_get_comment (n);
 	if (comment != NULL) {
+		guint i;
+		_cleanup_strv_free_ gchar **split = NULL;
+
 		/* do not put additional spacing for the root node */
 		if (depth_offset < g_node_depth ((GNode *) n) &&
 		    (flags & AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE) > 0)
 			g_string_append (xml, "\n");
 		if ((flags & AS_NODE_TO_XML_FLAG_FORMAT_INDENT) > 0)
 			as_node_add_padding (xml, depth - depth_offset);
-		g_string_append_printf (xml, "<!-- %s -->", comment);
-		if ((flags & AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE) > 0)
-			g_string_append (xml, "\n");
+
+		/* add each comment section */
+		split = g_strsplit (comment, "<&>", -1);
+		for (i = 0; split[i] != NULL; i++) {
+			g_string_append_printf (xml, "<!--%s-->", split[i]);
+			if ((flags & AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE) > 0)
+				g_string_append (xml, "\n");
+		}
 	}
 
 	/* root node */
@@ -597,6 +605,15 @@ as_node_text_cb (GMarkupParseContext *context,
 
 	/* split up into lines and add each with spaces stripped */
 	data = helper->current->data;
+	if (data->cdata != NULL) {
+		g_set_error (error,
+			     AS_NODE_ERROR,
+			     AS_NODE_ERROR_INVALID_MARKUP,
+			     "<%s> already set '%s' and tried to replace with '%s'",
+			     as_tag_data_get_name (data),
+			     data->cdata, text);
+		return;
+	}
 	if ((helper->flags & AS_NODE_FROM_XML_FLAG_LITERAL_TEXT) > 0) {
 		data->cdata = g_strndup (text, text_len);
 	} else {
@@ -615,6 +632,7 @@ as_node_passthrough_cb (GMarkupParseContext *context,
 			GError             **error)
 {
 	AsNodeToXmlHelper *helper = (AsNodeToXmlHelper *) user_data;
+	const gchar *existing;
 	const gchar *tmp;
 	gchar *found;
 	_cleanup_free_ gchar *text = NULL;
@@ -636,10 +654,21 @@ as_node_passthrough_cb (GMarkupParseContext *context,
 	found = g_strrstr (text, "-->");
 	if (found != NULL)
 		*found = '\0';
-	tmp = g_strstrip (text + 4);
+	tmp = text + 4;
+	if ((helper->flags & AS_NODE_FROM_XML_FLAG_LITERAL_TEXT) == 0)
+		tmp = g_strstrip ((gchar *) tmp);
 	if (tmp == NULL || tmp[0] == '\0')
 		return;
-	as_node_add_attribute (helper->current, "@comment-tmp", tmp, -1);
+
+	/* append together comments */
+	existing = as_node_get_attribute (helper->current, "@comment-tmp");
+	if (existing == NULL) {
+		as_node_add_attribute (helper->current, "@comment-tmp", tmp, -1);
+	} else {
+		_cleanup_free_ gchar *join = NULL;
+		join = g_strdup_printf ("%s<&>%s", existing, tmp);
+		as_node_add_attribute (helper->current, "@comment-tmp", join, -1);
+	}
 }
 
 /**
@@ -1222,6 +1251,23 @@ as_node_add_attribute (GNode *node,
 	attr->value = as_strndup (value, value_len);
 }
 
+/**
+ * as_node_add_attribute_as_int: (skip)
+ * @node: a #GNode
+ * @key: the attribute key
+ * @value: new data
+ *
+ * Adds a new attribute to a node.
+ *
+ * Since: 0.3.1
+ **/
+void
+as_node_add_attribute_as_int (GNode *node, const gchar *key, gint value)
+{
+	_cleanup_free_ gchar *tmp = g_strdup_printf ("%i", value);
+	as_node_add_attribute (node, key, tmp, -1);
+}
+
 /**
  * as_node_find: (skip)
  * @root: a root node, or %NULL
@@ -1291,6 +1337,29 @@ as_node_find_with_attribute (GNode *root, const gchar *path,
 	return node;
 }
 
+/**
+ * as_node_insert_line_breaks:
+ **/
+static gchar *
+as_node_insert_line_breaks (const gchar *text, guint break_len)
+{
+	GString *str;
+	guint i;
+	guint new_len;
+
+	/* allocate long enough for the string, plus the extra newlines */
+	new_len = strlen (text) * (break_len + 1) / break_len;
+	str = g_string_new_len (NULL, new_len + 2);
+	g_string_append (str, "\n");
+	g_string_append (str, text);
+
+	/* insert a newline every break length */
+	for (i = break_len + 1; i < str->len; i += break_len + 1)
+		g_string_insert (str, i, "\n");
+	g_string_append (str, "\n");
+	return g_string_free (str, FALSE);
+}
+
 /**
  * as_node_insert: (skip)
  * @parent: a parent #GNode.
@@ -1318,10 +1387,16 @@ as_node_insert (GNode *parent,
 	guint i;
 	va_list args;
 
+	g_return_val_if_fail (name != NULL, NULL);
+
 	data = g_slice_new0 (AsNodeData);
 	as_node_data_set_name (data, name, insert_flags);
-	if (cdata != NULL)
-		data->cdata = g_strdup (cdata);
+	if (cdata != NULL) {
+		if (insert_flags & AS_NODE_INSERT_FLAG_BASE64_ENCODED)
+			data->cdata = as_node_insert_line_breaks (cdata, 76);
+		else
+			data->cdata = g_strdup (cdata);
+	}
 	data->cdata_escaped = insert_flags & AS_NODE_INSERT_FLAG_PRE_ESCAPED;
 
 	/* process the attrs valist */
@@ -1373,6 +1448,8 @@ as_node_insert_localized (GNode *parent,
 	const gchar *value_c;
 	_cleanup_list_free_ GList *list = NULL;
 
+	g_return_if_fail (name != NULL);
+
 	/* add the untranslated value first */
 	value_c = g_hash_table_lookup (localized, "C");
 	if (value_c == NULL)
@@ -1439,6 +1516,8 @@ as_node_insert_hash (GNode *parent,
 	const gchar *value;
 	gboolean swapped = (insert_flags & AS_NODE_INSERT_FLAG_SWAPPED) > 0;
 
+	g_return_if_fail (name != NULL);
+
 	list = g_hash_table_get_keys (hash);
 	list = g_list_sort (list, as_node_list_sort_cb);
 	for (l = list; l != NULL; l = l->next) {
