@@ -47,6 +47,7 @@ struct _AsScreenshotPrivate
 	AsScreenshotKind	 kind;
 	GHashTable		*captions;
 	GPtrArray		*images;
+	gint			 priority;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsScreenshot, as_screenshot, G_TYPE_OBJECT)
@@ -148,6 +149,23 @@ as_screenshot_get_kind (AsScreenshot *screenshot)
 	return priv->kind;
 }
 
+/**
+ * as_screenshot_get_priority:
+ * @screenshot: a #AsScreenshot instance.
+ *
+ * Gets the screenshot priority.
+ *
+ * Returns: a priority value
+ *
+ * Since: 0.3.1
+ **/
+gint
+as_screenshot_get_priority (AsScreenshot *screenshot)
+{
+	AsScreenshotPrivate *priv = GET_PRIVATE (screenshot);
+	return priv->priority;
+}
+
 /**
  * as_screenshot_get_images:
  * @screenshot: a #AsScreenshot instance.
@@ -246,6 +264,22 @@ as_screenshot_get_caption (AsScreenshot *screenshot, const gchar *locale)
 	return as_hash_lookup_by_locale (priv->captions, locale);
 }
 
+/**
+ * as_screenshot_set_priority:
+ * @screenshot: a #AsScreenshot instance.
+ * @priority: the priority value.
+ *
+ * Sets the screenshot priority. Higher numbers are better.
+ *
+ * Since: 0.3.1
+ **/
+void
+as_screenshot_set_priority (AsScreenshot *screenshot, gint priority)
+{
+	AsScreenshotPrivate *priv = GET_PRIVATE (screenshot);
+	priv->priority = priority;
+}
+
 /**
  * as_screenshot_set_kind:
  * @screenshot: a #AsScreenshot instance.
@@ -340,6 +374,8 @@ as_screenshot_node_insert (AsScreenshot *screenshot,
 					  priv->captions,
 					  AS_NODE_INSERT_FLAG_DEDUPE_LANG);
 	}
+	if (api_version >= 0.8 && priv->priority != 0)
+		as_node_add_attribute_as_int (n, "priority", priv->priority);
 	for (i = 0; i < priv->images->len; i++) {
 		image = g_ptr_array_index (priv->images, i);
 		as_image_node_insert (image, n, api_version);
@@ -367,6 +403,7 @@ as_screenshot_node_parse (AsScreenshot *screenshot, GNode *node, GError **error)
 	GNode *c;
 	const gchar *tmp;
 	guint size;
+	gint priority;
 	_cleanup_hashtable_unref_ GHashTable *captions = NULL;
 
 	tmp = as_node_get_attribute (node, "type");
@@ -374,11 +411,14 @@ as_screenshot_node_parse (AsScreenshot *screenshot, GNode *node, GError **error)
 		as_screenshot_set_kind (screenshot,
 					as_screenshot_kind_from_string (tmp));
 	}
+	priority = as_node_get_attribute_as_int (node, "priority");
+	if (priority != G_MAXINT)
+		as_screenshot_set_priority (screenshot, priority);
 
 	/* add captions */
 	captions = as_node_get_localized (node, "caption");
 	if (captions != NULL) {
-		_cleanup_list_free_ GList *keys;
+		_cleanup_list_free_ GList *keys = NULL;
 		keys = g_hash_table_get_keys (captions);
 		for (l = keys; l != NULL; l = l->next) {
 			tmp = l->data;
