@@ -321,8 +321,6 @@ as_store_get_apps_by_metadata (AsStore *store,
  * NOTE: if applications are removed *all* the indexes will be invalid and
  * will have to be re-added.
  *
- * Returns: (transfer none): a #AsApp or %NULL
- *
  * Since: 0.3.0
  **/
 void
@@ -642,6 +640,7 @@ as_store_from_root (AsStore *store,
 				     error_local->message);
 			return FALSE;
 		}
+		as_app_set_origin (app, priv->origin);
 		as_store_add_app (store, app);
 	}
 
@@ -657,12 +656,15 @@ as_store_from_root (AsStore *store,
 static gboolean
 as_store_load_yaml_file (AsStore *store,
 			 GFile *file,
+			 const gchar *icon_root,
 			 GCancellable *cancellable,
 			 GError **error)
 {
+	AsStorePrivate *priv = GET_PRIVATE (store);
 	GNode *app_n;
 	GNode *n;
 	const gchar *tmp;
+	_cleanup_free_ gchar *icon_path = NULL;
 	_cleanup_yaml_unref_ GNode *root = NULL;
 
 	/* load file */
@@ -684,14 +686,27 @@ as_store_load_yaml_file (AsStore *store,
 		}
 	}
 
+	/* if we have an origin either from the YAML or _set_origin() */
+	if (priv->origin != NULL) {
+		if (icon_root == NULL)
+			icon_root = "/usr/share/app-info/icons/";
+		icon_path = g_build_filename (icon_root,
+					      priv->origin,
+					      NULL);
+	}
+
 	/* parse applications */
 	for (app_n = root->children->next; app_n != NULL; app_n = app_n->next) {
 		_cleanup_object_unref_ AsApp *app = NULL;
 		if (app_n->children == NULL)
 			continue;
 		app = as_app_new ();
+		if (icon_path != NULL)
+			as_app_set_icon_path (app, icon_path, -1);
+		as_app_set_source_kind (app, AS_APP_SOURCE_KIND_APPSTREAM);
 		if (!as_app_node_parse_dep11 (app, app_n, error))
 			return FALSE;
+		as_app_set_origin (app, priv->origin);
 		if (as_app_get_id (app) != NULL)
 			as_store_add_app (store, app);
 	}
@@ -733,7 +748,8 @@ as_store_from_file (AsStore *store,
 	/* a DEP-11 file */
 	filename = g_file_get_path (file);
 	if (g_strstr_len (filename, -1, ".yml") != NULL)
-		return as_store_load_yaml_file (store, file, cancellable, error);
+		return as_store_load_yaml_file (store, file, icon_root,
+						cancellable, error);
 
 	/* an AppStream XML file */
 	root = as_node_from_file (file,
@@ -744,8 +760,8 @@ as_store_from_file (AsStore *store,
 		g_set_error (error,
 			     AS_STORE_ERROR,
 			     AS_STORE_ERROR_FAILED,
-			     "Failed to parse file: %s",
-			     error_local->message);
+			     "Failed to parse %s file: %s",
+			     filename, error_local->message);
 		return FALSE;
 	}
 	return as_store_from_root (store, root, icon_root, error);
@@ -863,6 +879,34 @@ as_store_to_xml (AsStore *store, AsNodeToXmlFlags flags)
 	return xml;
 }
 
+/**
+ * as_store_convert_icons:
+ * @store: a #AsStore instance.
+ * @kind: the AsIconKind, e.g. %AS_ICON_KIND_EMBEDDED.
+ * @error: A #GError or %NULL
+ *
+ * Converts all the icons in the store to a specific kind.
+ *
+ * Returns: %TRUE for success
+ *
+ * Since: 0.3.1
+ **/
+gboolean
+as_store_convert_icons (AsStore *store, AsIconKind kind, GError **error)
+{
+	AsStorePrivate *priv = GET_PRIVATE (store);
+	AsApp *app;
+	guint i;
+
+	/* convert application icons */
+	for (i = 0; i < priv->array->len; i++) {
+		app = g_ptr_array_index (priv->array, i);
+		if (!as_app_convert_icons (app, kind, error))
+			return FALSE;
+	}
+	return TRUE;
+}
+
 /**
  * as_store_to_file:
  * @store: a #AsStore instance.
@@ -871,7 +915,7 @@ as_store_to_xml (AsStore *store, AsNodeToXmlFlags flags)
  * @cancellable: A #GCancellable, or %NULL
  * @error: A #GError or %NULL
  *
- * Outputs a compressed XML file of all the applications in the store.
+ * Outputs an optionally compressed XML file of all the applications in the store.
  *
  * Returns: A #GString
  *
@@ -889,6 +933,28 @@ as_store_to_file (AsStore *store,
 	_cleanup_object_unref_ GOutputStream *out = NULL;
 	_cleanup_object_unref_ GZlibCompressor *compressor = NULL;
 	_cleanup_string_free_ GString *xml = NULL;
+	_cleanup_free_ gchar *basename = NULL;
+
+	/* check if compressed */
+	basename = g_file_get_basename (file);
+	if (g_strstr_len (basename, -1, ".gz") == NULL) {
+		xml = as_store_to_xml (store, flags);
+		if (!g_file_replace_contents (file, xml->str, xml->len,
+					      NULL,
+					      FALSE,
+					      G_FILE_CREATE_NONE,
+					      NULL,
+					      cancellable,
+					      &error_local)) {
+			g_set_error (error,
+				     AS_STORE_ERROR,
+				     AS_STORE_ERROR_FAILED,
+				     "Failed to write file: %s",
+				     error_local->message);
+			return FALSE;
+		}
+		return TRUE;
+	}
 
 	/* compress as a gzip file */
 	compressor = g_zlib_compressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP, -1);
@@ -1115,19 +1181,21 @@ as_store_guess_origin_fallback (AsStore *store,
 				GError **error)
 {
 	gchar *tmp;
-	_cleanup_free_ gchar *origin_fallback;
+	_cleanup_free_ gchar *origin_fallback = NULL;
 
 	/* the first component of the file (e.g. "fedora-20.xml.gz)
 	 * is used for the icon directory as we might want to clean up
 	 * the icons manually if they are installed in /var/cache */
 	origin_fallback = g_path_get_basename (filename);
 	tmp = g_strstr_len (origin_fallback, -1, ".xml");
+	if (tmp == NULL)
+		tmp = g_strstr_len (origin_fallback, -1, ".yml");
 	if (tmp == NULL) {
 		g_set_error (error,
 			     AS_STORE_ERROR,
 			     AS_STORE_ERROR_FAILED,
 			     "AppStream metadata name %s not valid, "
-			     "expected .xml[.*]",
+			     "expected .xml[.*] or .yml[.*]",
 			     filename);
 		return FALSE;
 	}
@@ -1228,10 +1296,6 @@ as_store_load_app_info (AsStore *store,
 	_cleanup_free_ gchar *icon_root = NULL;
 	_cleanup_free_ gchar *path_md = NULL;
 
-	/* watch the directory for changes */
-	if (!as_store_monitor_directory (store, path, cancellable, error))
-		return FALSE;
-
 	/* search all files */
 	path_md = g_build_filename (path, format, NULL);
 	if (!g_file_test (path_md, G_FILE_TEST_EXISTS))
@@ -1247,7 +1311,7 @@ as_store_load_app_info (AsStore *store,
 	}
 	icon_root = g_build_filename (path, "icons", NULL);
 	while ((tmp = g_dir_read_name (dir)) != NULL) {
-		_cleanup_free_ gchar *filename_md;
+		_cleanup_free_ gchar *filename_md = NULL;
 		filename_md = g_build_filename (path_md, tmp, NULL);
 		if (!as_store_load_app_info_file (store,
 						  filename_md,
@@ -1256,6 +1320,13 @@ as_store_load_app_info (AsStore *store,
 						  error))
 			return FALSE;
 	}
+
+	/* watch the directories for changes */
+	if (!as_store_monitor_directory (store, path_md, cancellable, error))
+		return FALSE;
+	if (!as_store_monitor_directory (store, icon_root, cancellable, error))
+		return FALSE;
+
 	return TRUE;
 }
 
@@ -1300,10 +1371,14 @@ as_store_load_app_install_file (AsStore *store,
 				const gchar *path_icons,
 				GError **error)
 {
+	AsIcon *icon;
+	GPtrArray *icons;
+	guint i;
 	_cleanup_error_free_ GError *error_local = NULL;
-	_cleanup_object_unref_ AsApp *app;
+	_cleanup_object_unref_ AsApp *app = NULL;
 
 	app = as_app_new ();
+	as_app_set_icon_path (app, path_icons, -1);
 	if (!as_app_parse_file (app,
 				filename,
 				AS_APP_PARSE_FLAG_USE_HEURISTICS,
@@ -1322,9 +1397,14 @@ as_store_load_app_install_file (AsStore *store,
 			     error_local->message);
 		return FALSE;
 	}
-	if (as_app_get_icon_kind (app) == AS_ICON_KIND_UNKNOWN)
-		as_app_set_icon_kind (app, AS_ICON_KIND_CACHED);
-	as_app_set_icon_path (app, path_icons, -1);
+
+	/* convert all the icons */
+	icons = as_app_get_icons (app);
+	for (i = 0; i < icons->len; i++) {
+		icon = g_ptr_array_index (icons, i);
+		if (as_icon_get_kind (icon) == AS_ICON_KIND_UNKNOWN)
+			as_icon_set_kind (icon, AS_ICON_KIND_CACHED);
+	}
 	as_store_add_app_install_screenshot (app);
 	as_store_add_app (store, app);
 
@@ -1441,7 +1521,6 @@ as_store_load_installed (AsStore *store,
 
 		/* set lower priority than AppStream entries */
 		as_app_set_priority (app, -1);
-		as_app_set_icon_kind (app, AS_ICON_KIND_STOCK);
 		as_app_set_state (app, AS_APP_STATE_INSTALLED);
 		as_store_add_app (store, app);
 	}
@@ -1581,7 +1660,7 @@ as_store_validate_add (GPtrArray *problems, AsProblemKind kind, const gchar *fmt
 	AsProblem *problem;
 	guint i;
 	va_list args;
-	_cleanup_free_ gchar *str;
+	_cleanup_free_ gchar *str = NULL;
 
 	va_start (args, fmt);
 	str = g_strdup_vprintf (fmt, args);
@@ -1631,7 +1710,7 @@ as_store_validate (AsStore *store, AsAppValidateFlags flags, GError **error)
 		if ((priv->problems & AS_STORE_PROBLEM_LEGACY_ROOT) == 0) {
 			as_store_validate_add (probs,
 					       AS_PROBLEM_KIND_TAG_INVALID,
-					       "metdata version is v%.1f and "
+					       "metadata version is v%.1f and "
 					       "XML root is not <applications>",
 					       priv->api_version);
 		}
@@ -1639,14 +1718,14 @@ as_store_validate (AsStore *store, AsAppValidateFlags flags, GError **error)
 		if ((priv->problems & AS_STORE_PROBLEM_LEGACY_ROOT) != 0) {
 			as_store_validate_add (probs,
 					       AS_PROBLEM_KIND_TAG_INVALID,
-					       "metdata version is v%.1f and "
+					       "metadata version is v%.1f and "
 					       "XML root is not <components>",
 					       priv->api_version);
 		}
 		if (priv->origin == NULL) {
 			as_store_validate_add (probs,
 					       AS_PROBLEM_KIND_TAG_MISSING,
-					       "metdata version is v%.1f and "
+					       "metadata version is v%.1f and "
 					       "origin attribute is missing",
 					       priv->api_version);
 		}
@@ -1654,19 +1733,23 @@ as_store_validate (AsStore *store, AsAppValidateFlags flags, GError **error)
 
 	/* check each application */
 	for (i = 0; i < priv->array->len; i++) {
+		AsProblem *prob;
+		guint j;
+		_cleanup_ptrarray_unref_ GPtrArray *probs_app = NULL;
+
 		app = g_ptr_array_index (priv->array, i);
 		if (priv->api_version < 0.3) {
 			if (as_app_get_source_pkgname (app) != NULL) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<source_pkgname> only introduced in v0.3",
 						       priv->api_version);
 			}
 			if (as_app_get_priority (app) != 0) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<priority> only introduced in v0.3",
 						       priv->api_version);
 			}
@@ -1675,35 +1758,35 @@ as_store_validate (AsStore *store, AsAppValidateFlags flags, GError **error)
 			if (as_app_get_project_group (app) != NULL) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<project_group> only introduced in v0.4",
 						       priv->api_version);
 			}
 			if (as_app_get_mimetypes(app)->len > 0) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<mimetypes> only introduced in v0.4",
 						       priv->api_version);
 			}
 			if (as_app_get_screenshots(app)->len > 0) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<screenshots> only introduced in v0.4",
 						       priv->api_version);
 			}
 			if (as_app_get_compulsory_for_desktops(app)->len > 0) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<compulsory_for_desktop> only introduced in v0.4",
 						       priv->api_version);
 			}
 			if (g_list_length (as_app_get_languages(app)) > 0) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<languages> only introduced in v0.4",
 						       priv->api_version);
 			}
@@ -1712,7 +1795,7 @@ as_store_validate (AsStore *store, AsAppValidateFlags flags, GError **error)
 			if ((as_app_get_problems (app) & AS_APP_PROBLEM_PREFORMATTED_DESCRIPTION) == 0) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<description> markup "
 						       "was introduced in v0.6",
 						       priv->api_version);
@@ -1720,21 +1803,21 @@ as_store_validate (AsStore *store, AsAppValidateFlags flags, GError **error)
 			if (as_app_get_architectures(app)->len > 0) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<architectures> only introduced in v0.6",
 						       priv->api_version);
 			}
 			if (as_app_get_releases(app)->len > 0) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<releases> only introduced in v0.6",
 						       priv->api_version);
 			}
 			if (as_app_get_provides(app)->len > 0) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<provides> only introduced in v0.6",
 						       priv->api_version);
 			}
@@ -1742,7 +1825,7 @@ as_store_validate (AsStore *store, AsAppValidateFlags flags, GError **error)
 			if ((as_app_get_problems (app) & AS_APP_PROBLEM_PREFORMATTED_DESCRIPTION) != 0) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<description> requiring markup "
 						       "was introduced in v0.6",
 						       priv->api_version);
@@ -1752,25 +1835,57 @@ as_store_validate (AsStore *store, AsAppValidateFlags flags, GError **error)
 			if (as_app_get_id_kind (app) == AS_ID_KIND_ADDON) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "addon kinds only introduced in v0.7",
 						       priv->api_version);
 			}
 			if (as_app_get_developer_name (app, NULL) != NULL) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<developer_name> only introduced in v0.7",
 						       priv->api_version);
 			}
 			if (as_app_get_extends(app)->len > 0) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metdata version is v%.1f and "
+						       "metadata version is v%.1f and "
 						       "<extends> only introduced in v0.7",
 						       priv->api_version);
 			}
 		}
+
+		/* check for translations where there should be none */
+		if ((as_app_get_problems (app) & AS_APP_PROBLEM_TRANSLATED_ID) != 0) {
+			as_store_validate_add (probs,
+					       AS_PROBLEM_KIND_TAG_INVALID,
+					       "<id> values cannot be translated");
+		}
+		if ((as_app_get_problems (app) & AS_APP_PROBLEM_TRANSLATED_LICENSE) != 0) {
+			as_store_validate_add (probs,
+					       AS_PROBLEM_KIND_TAG_INVALID,
+					       "<license> values cannot be translated");
+		}
+		if ((as_app_get_problems (app) & AS_APP_PROBLEM_TRANSLATED_PROJECT_GROUP) != 0) {
+			as_store_validate_add (probs,
+					       AS_PROBLEM_KIND_TAG_INVALID,
+					       "<project_group> values cannot be translated");
+		}
+
+		/* validate each application */
+		if (flags & AS_APP_VALIDATE_FLAG_ALL_APPS) {
+			probs_app = as_app_validate (app, flags, error);
+			if (probs_app == NULL)
+				return NULL;
+			for (j = 0; j < probs_app->len; j++) {
+				prob = g_ptr_array_index (probs_app, j);
+				as_store_validate_add (probs,
+						       as_problem_get_kind (prob),
+						       "%s: %s",
+						       as_app_get_id (app),
+						       as_problem_get_message (prob));
+			}
+		}
 	}
 	return probs;
 }
