@@ -79,6 +79,10 @@ as_tag_from_string_full (const gchar *tag, AsTagFlags flags)
 #endif
 	AsTag etag = AS_TAG_UNKNOWN;
 
+	/* invalid */
+	if (tag == NULL)
+		return AS_TAG_UNKNOWN;
+
 #ifdef HAVE_GPERF
 	/* use a perfect hash */
 	ky = as_tag_from_gperf (tag, strlen (tag));
@@ -105,6 +109,11 @@ as_tag_from_string_full (const gchar *tag, AsTagFlags flags)
 			return AS_TAG_APPLICATIONS;
 		if (g_strcmp0 (tag, "application") == 0)
 			return AS_TAG_APPLICATION;
+		if (g_strcmp0 (tag, "updatecontact") == 0)
+			return AS_TAG_UPDATE_CONTACT;
+		/* fix spelling error */
+		if (g_strcmp0 (tag, "metadata_licence") == 0)
+			return AS_TAG_METADATA_LICENSE;
 	}
 
 	/* translated versions */
@@ -154,7 +163,7 @@ as_tag_to_string (AsTag tag)
 		"project_license",
 		"screenshot",
 		"screenshots",
-		"updatecontact",
+		"update_contact",
 		"image",
 		"compulsory_for_desktop",
 		"priority",
