@@ -35,15 +35,34 @@
 #include <fnmatch.h>
 #include <string.h>
 #include <libsoup/soup.h>
+#include <archive_entry.h>
+#include <archive.h>
 
 #include "as-app.h"
 #include "as-cleanup.h"
 #include "as-enums.h"
 #include "as-node.h"
 #include "as-resources.h"
+#include "as-store.h"
 #include "as-utils.h"
 #include "as-utils-private.h"
 
+/**
+ * as_utils_error_quark:
+ *
+ * Return value: An error quark.
+ *
+ * Since: 0.3.7
+ **/
+GQuark
+as_utils_error_quark (void)
+{
+	static GQuark quark = 0;
+	if (!quark)
+		quark = g_quark_from_static_string ("AsUtilsError");
+	return quark;
+}
+
 /**
  * as_strndup:
  * @text: the text to copy.
@@ -196,7 +215,7 @@ as_hash_lookup_by_locale (GHashTable *hash, const gchar *locale)
 gboolean
 as_utils_is_stock_icon_name (const gchar *name)
 {
-	_cleanup_bytes_unref_ GBytes *data;
+	_cleanup_bytes_unref_ GBytes *data = NULL;
 	_cleanup_free_ gchar *key = NULL;
 
 	/* load the readonly data section and look for the icon name */
@@ -223,7 +242,7 @@ as_utils_is_stock_icon_name (const gchar *name)
 gboolean
 as_utils_is_spdx_license_id (const gchar *license_id)
 {
-	_cleanup_bytes_unref_ GBytes *data;
+	_cleanup_bytes_unref_ GBytes *data = NULL;
 	_cleanup_free_ gchar *key = NULL;
 
 	/* load the readonly data section and look for the icon name */
@@ -250,10 +269,10 @@ as_utils_is_spdx_license_id (const gchar *license_id)
 gboolean
 as_utils_is_blacklisted_id (const gchar *desktop_id)
 {
-	_cleanup_bytes_unref_ GBytes *data;
+	guint i;
+	_cleanup_bytes_unref_ GBytes *data = NULL;
 	_cleanup_free_ gchar *key = NULL;
 	_cleanup_strv_free_ gchar **split = NULL;
-	guint i;
 
 	/* load the readonly data section and look for the icon name */
 	data = g_resource_lookup_data (as_get_resource (),
@@ -283,7 +302,7 @@ as_utils_is_blacklisted_id (const gchar *desktop_id)
 gboolean
 as_utils_is_environment_id (const gchar *environment_id)
 {
-	_cleanup_bytes_unref_ GBytes *data;
+	_cleanup_bytes_unref_ GBytes *data = NULL;
 	_cleanup_free_ gchar *key = NULL;
 
 	/* load the readonly data section and look for the icon name */
@@ -310,7 +329,7 @@ as_utils_is_environment_id (const gchar *environment_id)
 gboolean
 as_utils_is_category_id (const gchar *category_id)
 {
-	_cleanup_bytes_unref_ GBytes *data;
+	_cleanup_bytes_unref_ GBytes *data = NULL;
 	_cleanup_free_ gchar *key = NULL;
 
 	/* load the readonly data section and look for the icon name */
@@ -595,10 +614,10 @@ as_utils_check_url_exists (const gchar *url, guint timeout, GError **error)
 }
 
 /**
- * as_pixbuf_box_blur_private:
+ * as_pixbuf_blur_private:
  **/
 static void
-as_pixbuf_box_blur_private (GdkPixbuf *src, GdkPixbuf *dest, gint radius, guchar *div_kernel_size)
+as_pixbuf_blur_private (GdkPixbuf *src, GdkPixbuf *dest, gint radius, guchar *div_kernel_size)
 {
 	gint width, height, src_rowstride, dest_rowstride, n_channels;
 	guchar *p_src, *p_dest, *c1, *c2;
@@ -708,14 +727,21 @@ as_pixbuf_box_blur_private (GdkPixbuf *src, GdkPixbuf *dest, gint radius, guchar
 }
 
 /**
- * as_pixbuf_box_blur:
+ * as_pixbuf_blur:
+ * @src: the GdkPixbuf.
+ * @radius: the pixel radius for the gaussian blur, typical values are 1..3
+ * @iterations: Amount to blur the image, typical values are 1..5
+ *
+ * Blurs an image. Warning, this method is s..l..o..w... for large images.
+ *
+ * Since: 0.3.2
  **/
-static void
-as_pixbuf_box_blur (GdkPixbuf *src, gint radius, gint iterations)
+void
+as_pixbuf_blur (GdkPixbuf *src, gint radius, gint iterations)
 {
 	gint kernel_size;
 	gint i;
-	_cleanup_free_ guchar *div_kernel_size;
+	_cleanup_free_ guchar *div_kernel_size = NULL;
 	_cleanup_object_unref_ GdkPixbuf *tmp = NULL;
 
 	tmp = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (src),
@@ -729,7 +755,7 @@ as_pixbuf_box_blur (GdkPixbuf *src, gint radius, gint iterations)
 		div_kernel_size[i] = (guchar) (i / kernel_size);
 
 	while (iterations-- > 0)
-		as_pixbuf_box_blur_private (src, tmp, radius, div_kernel_size);
+		as_pixbuf_blur_private (src, tmp, radius, div_kernel_size);
 }
 
 #define interpolate_value(original, reference, distance)		\
@@ -758,7 +784,7 @@ as_pixbuf_sharpen (GdkPixbuf *src, gint radius, gdouble amount)
 	_cleanup_object_unref_ GdkPixbuf *blurred = NULL;
 
 	blurred = gdk_pixbuf_copy (src);
-	as_pixbuf_box_blur (blurred, radius, 3);
+	as_pixbuf_blur (blurred, radius, 3);
 
 	width = gdk_pixbuf_get_width (src);
 	height = gdk_pixbuf_get_height (src);
@@ -790,26 +816,29 @@ as_pixbuf_sharpen (GdkPixbuf *src, gint radius, gdouble amount)
 }
 
 /**
- * as_utils_find_icon_filename:
+ * as_utils_find_icon_filename_full:
  * @destdir: the destdir.
  * @search: the icon search name, e.g. "microphone.svg"
+ * @flags: A #AsUtilsFindIconFlag bitfield
  * @error: A #GError or %NULL
  *
  * Finds an icon filename from a filesystem root.
  *
  * Returns: (transfer full): a newly allocated %NULL terminated string
  *
- * Since: 0.2.5
+ * Since: 0.3.1
  **/
 gchar *
-as_utils_find_icon_filename (const gchar *destdir,
-			     const gchar *search,
-			     GError **error)
+as_utils_find_icon_filename_full (const gchar *destdir,
+				  const gchar *search,
+				  AsUtilsFindIconFlag flags,
+				  GError **error)
 {
 	guint i;
 	guint j;
 	guint k;
 	guint m;
+	const gchar **sizes;
 	const gchar *pixmap_dirs[] = { "pixmaps", "icons", NULL };
 	const gchar *theme_dirs[] = { "hicolor", "oxygen", NULL };
 	const gchar *supported_ext[] = { ".png",
@@ -818,16 +847,20 @@ as_utils_find_icon_filename (const gchar *destdir,
 					 ".xpm",
 					 "",
 					 NULL };
-	const gchar *sizes[] = { "64x64",
-				 "128x128",
-				 "96x96",
-				 "256x256",
-				 "scalable",
-				 "48x48",
-				 "32x32",
-				 "24x24",
-				 "16x16",
-				 NULL };
+	const gchar *sizes_lo_dpi[] = { "64x64",
+					"128x128",
+					"96x96",
+					"256x256",
+					"scalable",
+					"48x48",
+					"32x32",
+					"24x24",
+					"16x16",
+					NULL };
+	const gchar *sizes_hi_dpi[] = { "128x128",
+					"256x256",
+					"scalable",
+					NULL };
 	const gchar *types[] = { "actions",
 				 "animations",
 				 "apps",
@@ -849,12 +882,12 @@ as_utils_find_icon_filename (const gchar *destdir,
 
 	/* is this an absolute path */
 	if (search[0] == '/') {
-		_cleanup_free_ gchar *tmp;
+		_cleanup_free_ gchar *tmp = NULL;
 		tmp = g_build_filename (destdir, search, NULL);
 		if (!g_file_test (tmp, G_FILE_TEST_EXISTS)) {
 			g_set_error (error,
-				     AS_APP_ERROR,
-				     AS_APP_ERROR_FAILED,
+				     AS_UTILS_ERROR,
+				     AS_UTILS_ERROR_FAILED,
 				     "specified icon '%s' does not exist",
 				     search);
 			return NULL;
@@ -863,11 +896,12 @@ as_utils_find_icon_filename (const gchar *destdir,
 	}
 
 	/* icon theme apps */
+	sizes = flags & AS_UTILS_FIND_ICON_HI_DPI ? sizes_hi_dpi : sizes_lo_dpi;
 	for (k = 0; theme_dirs[k] != NULL; k++) {
 		for (i = 0; sizes[i] != NULL; i++) {
 			for (m = 0; types[m] != NULL; m++) {
 				for (j = 0; supported_ext[j] != NULL; j++) {
-					_cleanup_free_ gchar *tmp;
+					_cleanup_free_ gchar *tmp = NULL;
 					tmp = g_strdup_printf ("%s/usr/share/icons/"
 							       "%s/%s/%s/%s%s",
 							       destdir,
@@ -886,7 +920,7 @@ as_utils_find_icon_filename (const gchar *destdir,
 	/* pixmap */
 	for (i = 0; pixmap_dirs[i] != NULL; i++) {
 		for (j = 0; supported_ext[j] != NULL; j++) {
-			_cleanup_free_ gchar *tmp;
+			_cleanup_free_ gchar *tmp = NULL;
 			tmp = g_strdup_printf ("%s/usr/share/%s/%s%s",
 					       destdir,
 					       pixmap_dirs[i],
@@ -899,8 +933,448 @@ as_utils_find_icon_filename (const gchar *destdir,
 
 	/* failed */
 	g_set_error (error,
-		     AS_APP_ERROR,
-		     AS_APP_ERROR_FAILED,
+		     AS_UTILS_ERROR,
+		     AS_UTILS_ERROR_FAILED,
 		     "Failed to find icon %s", search);
 	return NULL;
 }
+
+/**
+ * as_utils_find_icon_filename:
+ * @destdir: the destdir.
+ * @search: the icon search name, e.g. "microphone.svg"
+ * @error: A #GError or %NULL
+ *
+ * Finds an icon filename from a filesystem root.
+ *
+ * Returns: (transfer full): a newly allocated %NULL terminated string
+ *
+ * Since: 0.2.5
+ **/
+gchar *
+as_utils_find_icon_filename (const gchar *destdir,
+			     const gchar *search,
+			     GError **error)
+{
+	return as_utils_find_icon_filename_full (destdir, search,
+						 AS_UTILS_FIND_ICON_NONE,
+						 error);
+}
+
+/**
+ * as_utils_get_string_overlap_prefix:
+ */
+static gchar *
+as_utils_get_string_overlap_prefix (const gchar *s1, const gchar *s2)
+{
+	guint i;
+	for (i = 0; s1[i] != '\0' && s2[i] != '\0'; i++) {
+		if (s1[i] != s2[i])
+			break;
+	}
+	if (i == 0)
+		return NULL;
+	if (s1[i - 1] == '-' || s1[i - 1] == '.')
+		i--;
+	return g_strndup (s1, i);
+}
+
+/**
+ * as_utils_get_string_overlap_suffix:
+ */
+static gchar *
+as_utils_get_string_overlap_suffix (const gchar *s1, const gchar *s2)
+{
+	guint i;
+	guint len1 = strlen (s1);
+	guint len2 = strlen (s2);
+	for (i = 0; i <= len1 && i <= len2; i++) {
+		if (s1[len1 - i] != s2[len2 - i])
+			break;
+	}
+	if (i <= 1)
+		return NULL;
+	return g_strdup (&s1[len1 - i + 1]);
+}
+
+/**
+ * as_utils_get_string_overlap:
+ * @s1: A string.
+ * @s2: Another string
+ *
+ * Return a prefix and sufffix that is common to both strings.
+ *
+ * Returns: (transfer full): a newly allocated %NULL terminated string, or %NULL
+ *
+ * Since: 0.3.1
+ */
+gchar *
+as_utils_get_string_overlap (const gchar *s1, const gchar *s2)
+{
+	_cleanup_free_ gchar *prefix = NULL;
+	_cleanup_free_ gchar *suffix = NULL;
+
+	g_return_val_if_fail (s1 != NULL, NULL);
+	g_return_val_if_fail (s2 != NULL, NULL);
+
+	/* same? */
+	if (g_strcmp0 (s1, s2) == 0)
+		return g_strdup (s1);
+
+	prefix = as_utils_get_string_overlap_prefix (s1, s2);
+	suffix = as_utils_get_string_overlap_suffix (s1, s2);
+	if (prefix == NULL && suffix == NULL)
+		return NULL;
+	if (prefix != NULL && suffix == NULL)
+		return g_strdup (prefix);
+	if (prefix == NULL && suffix != NULL)
+		return g_strdup (suffix);
+	return g_strdup_printf ("%s%s", prefix, suffix);
+}
+
+/**
+ * as_utils_location_get_prefix:
+ **/
+static const gchar *
+as_utils_location_get_prefix (AsUtilsLocation location)
+{
+	if (location == AS_UTILS_LOCATION_SHARED)
+		return "/usr/share";
+	if (location == AS_UTILS_LOCATION_CACHE)
+		return "/var/cache";
+	if (location == AS_UTILS_LOCATION_USER)
+		return "~/.local/share";
+	return NULL;
+}
+
+/**
+ * as_utils_install_icon:
+ **/
+static gboolean
+as_utils_install_icon (AsUtilsLocation location,
+		       const gchar *filename,
+		       const gchar *origin,
+		       const gchar *destdir,
+		       GError **error)
+{
+	const gchar *pathname;
+	const gchar *tmp;
+	gboolean ret = TRUE;
+	gsize len;
+	int r;
+	struct archive *arch = NULL;
+	struct archive_entry *entry;
+	_cleanup_free_ gchar *data = NULL;
+	_cleanup_free_ gchar *dir = NULL;
+
+	dir = g_strdup_printf ("%s%s/app-info/icons/%s",
+			       destdir,
+			       as_utils_location_get_prefix (location),
+			       origin);
+
+	/* load file at once to avoid seeking */
+	ret = g_file_get_contents (filename, &data, &len, error);
+	if (!ret)
+		goto out;
+
+	/* read anything */
+	arch = archive_read_new ();
+	archive_read_support_format_all (arch);
+	archive_read_support_filter_all (arch);
+	r = archive_read_open_memory (arch, data, len);
+	if (r) {
+		ret = FALSE;
+		g_set_error (error,
+			     AS_UTILS_ERROR,
+			     AS_UTILS_ERROR_FAILED,
+			     "Cannot open: %s",
+			     archive_error_string (arch));
+		goto out;
+	}
+
+	/* decompress each file */
+	for (;;) {
+		_cleanup_free_ gchar *buf = NULL;
+
+		r = archive_read_next_header (arch, &entry);
+		if (r == ARCHIVE_EOF)
+			break;
+		if (r != ARCHIVE_OK) {
+			ret = FALSE;
+			g_set_error (error,
+				     AS_UTILS_ERROR,
+				     AS_UTILS_ERROR_FAILED,
+				     "Cannot read header: %s",
+				     archive_error_string (arch));
+			goto out;
+		}
+
+		/* no output file */
+		pathname = archive_entry_pathname (entry);
+		if (pathname == NULL)
+			continue;
+
+		/* update output path */
+		buf = g_build_filename (dir, pathname, NULL);
+		archive_entry_update_pathname_utf8 (entry, buf);
+
+		/* update hardlinks */
+		tmp = archive_entry_hardlink (entry);
+		if (tmp != NULL) {
+			_cleanup_free_ gchar *buf_link = NULL;
+			buf_link = g_build_filename (dir, tmp, NULL);
+			archive_entry_update_hardlink_utf8 (entry, buf_link);
+		}
+
+		/* update symlinks */
+		tmp = archive_entry_symlink (entry);
+		if (tmp != NULL) {
+			_cleanup_free_ gchar *buf_link = NULL;
+			buf_link = g_build_filename (dir, tmp, NULL);
+			archive_entry_update_symlink_utf8 (entry, buf_link);
+		}
+
+		r = archive_read_extract (arch, entry, 0);
+		if (r != ARCHIVE_OK) {
+			ret = FALSE;
+			g_set_error (error,
+				     AS_UTILS_ERROR,
+				     AS_UTILS_ERROR_FAILED,
+				     "Cannot extract: %s",
+				     archive_error_string (arch));
+			goto out;
+		}
+	}
+out:
+	if (arch != NULL) {
+		archive_read_close (arch);
+		archive_read_free (arch);
+	}
+	return ret;
+}
+
+/**
+ * as_utils_install_xml:
+ **/
+static gboolean
+as_utils_install_xml (const gchar *filename,
+		      const gchar *origin,
+		      const gchar *dir,
+		      const gchar *destdir,
+		      GError **error)
+{
+	gchar *tmp;
+	_cleanup_free_ gchar *basename = NULL;
+	_cleanup_free_ gchar *path_dest = NULL;
+	_cleanup_free_ gchar *path_parent = NULL;
+	_cleanup_object_unref_ GFile *file_dest = NULL;
+	_cleanup_object_unref_ GFile *file_src = NULL;
+
+	/* create directory structure */
+	path_parent = g_strdup_printf ("%s%s", destdir, dir);
+	if (g_mkdir_with_parents (path_parent, 0777) != 0) {
+		g_set_error (error,
+			     AS_UTILS_ERROR,
+			     AS_UTILS_ERROR_FAILED,
+			     "Failed to create %s", path_parent);
+		return FALSE;
+	}
+
+	/* calculate the new destination */
+	file_src = g_file_new_for_path (filename);
+	basename = g_path_get_basename (filename);
+	if (origin != NULL) {
+		_cleanup_free_ gchar *basename_new = NULL;
+		tmp = g_strstr_len (basename, -1, ".");
+		if (tmp == NULL) {
+			g_set_error (error,
+				     AS_UTILS_ERROR,
+				     AS_UTILS_ERROR_FAILED,
+				     "Name of XML file invalid %s",
+				     basename);
+			return FALSE;
+		}
+		basename_new = g_strdup_printf ("%s%s", origin, tmp);
+		/* replace the fedora.xml.gz into %{origin}.xml.gz */
+		path_dest = g_build_filename (path_parent, basename_new, NULL);
+	} else {
+		path_dest = g_build_filename (path_parent, basename, NULL);
+	}
+
+	/* actually copy file */
+	file_dest = g_file_new_for_path (path_dest);
+	if (!g_file_copy (file_src, file_dest,
+			  G_FILE_COPY_OVERWRITE |
+			  G_FILE_COPY_TARGET_DEFAULT_PERMS,
+			  NULL, NULL, NULL, error))
+		return FALSE;
+
+	/* fix the origin */
+	if (origin != NULL) {
+		_cleanup_object_unref_ AsStore *store = NULL;
+		store = as_store_new ();
+		if (!as_store_from_file (store, file_dest, NULL, NULL, error))
+			return FALSE;
+		as_store_set_origin (store, origin);
+		if (!as_store_to_file (store, file_dest,
+				       AS_NODE_TO_XML_FLAG_ADD_HEADER |
+				       AS_NODE_TO_XML_FLAG_FORMAT_MULTILINE,
+				       NULL, error))
+			return FALSE;
+	}
+	return TRUE;
+}
+
+/**
+ * as_utils_install_filename:
+ * @location: the #AsUtilsLocation, e.g. %AS_UTILS_LOCATION_CACHE
+ * @filename: the full path of the file to install
+ * @origin: the origin to use for the installation, or %NULL
+ * @destdir: the destdir to use, or %NULL
+ * @error: A #GError or %NULL
+ *
+ * Installs an AppData, MetaInfo, AppStream XML or AppStream Icon metadata file.
+ *
+ * Returns: %TRUE for success, %FALSE if error is set
+ *
+ * Since: 0.3.4
+ **/
+gboolean
+as_utils_install_filename (AsUtilsLocation location,
+			  const gchar *filename,
+			  const gchar *origin,
+			  const gchar *destdir,
+			  GError **error)
+{
+	gboolean ret = FALSE;
+	gchar *tmp;
+	_cleanup_free_ gchar *basename = NULL;
+	_cleanup_free_ gchar *path = NULL;
+
+	/* default value */
+	if (destdir == NULL)
+		destdir = "";
+
+	switch (as_app_guess_source_kind (filename)) {
+	case AS_APP_SOURCE_KIND_APPSTREAM:
+		if (g_strstr_len (filename, -1, ".yml.gz") != NULL) {
+			path = g_build_filename (as_utils_location_get_prefix (location),
+						 "app-info", "yaml", NULL);
+			ret = as_utils_install_xml (filename, origin, path, destdir, error);
+		} else {
+			path = g_build_filename (as_utils_location_get_prefix (location),
+						 "app-info", "xmls", NULL);
+			ret = as_utils_install_xml (filename, origin, path, destdir, error);
+		}
+		break;
+	case AS_APP_SOURCE_KIND_APPDATA:
+	case AS_APP_SOURCE_KIND_METAINFO:
+		if (location == AS_UTILS_LOCATION_CACHE) {
+			g_set_error_literal (error,
+					     AS_UTILS_ERROR,
+					     AS_UTILS_ERROR_INVALID_TYPE,
+					     "cached location unsupported for "
+					     "MetaInfo and AppData files");
+			return FALSE;
+		}
+		path = g_build_filename (as_utils_location_get_prefix (location),
+					 "appdata", NULL);
+		ret = as_utils_install_xml (filename, NULL, path, destdir, error);
+		break;
+	default:
+		/* icons */
+		if (origin != NULL) {
+			ret = as_utils_install_icon (location, filename, origin, destdir, error);
+			break;
+		}
+		basename = g_path_get_basename (filename);
+		tmp = g_strstr_len (basename, -1, "-icons.tar.gz");
+		if (tmp != NULL) {
+			*tmp = '\0';
+			ret = as_utils_install_icon (location, filename, basename, destdir, error);
+			break;
+		}
+
+		/* unrecognised */
+		g_set_error_literal (error,
+				     AS_UTILS_ERROR,
+				     AS_UTILS_ERROR_INVALID_TYPE,
+				     "No idea how to process files of this type");
+		break;
+	}
+	return ret;
+}
+
+/**
+ * as_utils_search_token_valid:
+ * @token: the search token
+ *
+ * Checks the search token if it is valid. Valid tokens are at least 3 chars in
+ * length, not common words like "and", and do not contain markup.
+ *
+ * Returns: %TRUE is the search token was valid
+ *
+ * Since: 0.3.4
+ **/
+gboolean
+as_utils_search_token_valid (const gchar *token)
+{
+	guint i;
+	const gchar *blacklist[] = {
+		"and", "the", "desktop", "application", "for", "you", "your",
+		"with", "can", "are", "from", "that", "use", "allows", "also",
+		"this", "other", "all", "using", "has", "some", "like", "them",
+		"well", "not", "using", "not", "but", "set", "its", "into",
+		"such", "was", "they", "where", "want", "only", "about",
+		NULL };
+	if (strlen (token) < 3)
+		return FALSE;
+	if (g_strstr_len (token, -1, "<") != NULL)
+		return FALSE;
+	if (g_strstr_len (token, -1, ">") != NULL)
+		return FALSE;
+	if (g_strstr_len (token, -1, "(") != NULL)
+		return FALSE;
+	if (g_strstr_len (token, -1, ")") != NULL)
+		return FALSE;
+	for (i = 0; blacklist[i] != NULL; i++)  {
+		if (g_strcmp0 (token, blacklist[i]) == 0)
+			return FALSE;
+	}
+	return TRUE;
+}
+
+/**
+ * as_utils_search_tokenize:
+ * @search: the search string
+ *
+ * Splits up a string into tokens and returns tokens that are suitable for
+ * searching. This includes taking out common words and casefolding the
+ * returned search tokens.
+ *
+ * Returns: (transfer full): Valid tokens to search for, or %NULL for error
+ *
+ * Since: 0.3.4
+ **/
+gchar **
+as_utils_search_tokenize (const gchar *search)
+{
+	gchar **values = NULL;
+	guint i;
+	guint idx = 0;
+	_cleanup_strv_free_ gchar **tmp = NULL;
+
+	/* only add keywords that are long enough */
+	tmp = g_strsplit (search, " ", -1);
+	values = g_new0 (gchar *, g_strv_length (tmp) + 1);
+	for (i = 0; tmp[i] != NULL; i++) {
+		if (!as_utils_search_token_valid (tmp[i]))
+			continue;
+		values[idx++] = g_utf8_casefold (tmp[i], -1);
+	}
+	if (idx == 0) {
+		g_free (values);
+		return NULL;
+	}
+	return values;
+}
